<?php

	if (!defined("ENV")) exit();
	
	if (!isset($settings['WholesaleCentralUsername']) || !isset($settings['WholesaleCentralPassword']) || !isset($settings['WholesaleCentralEnableCron'])
		|| !isset($settings['WholesaleCentralLastUpload']) || !isset($settings['WholesaleCentralUseOverviewDescription']))
	{
		pageNote(array(
			array("class"=>"", "text"=> '<img src="images/admin/wholesalecentral.png" />', "url"=>"", "visible"=>true),
			array("class"=>"ic-warning", "text"=> 'There is an error processing WholesaleCentral.com. Please check your installation/update and ensure all settings for WholesaleCentral.com are available.', "url"=>"", "visible"=>true)
		));
		return;
	}
	
	require_once('content/classes/class.products.php');
	
	$action = isset($action) ? trim($action) : '';
	
	if (!DEMO_LOCK)
	{
		$wcClient = new WholesaleCentral($settings['WholesaleCentralUsername'], $settings['WholesaleCentralPassword'], $db);
		$controller = new Controller_WholesaleCentral($wcClient, $db, $settings);
		$controller->indexAction();
		
		switch ($action)
		{
			case 'update':
				$data['WholesaleCentralUsername'] = isset($_POST['username']) ? $_POST['username'] : null;
				$data['WholesaleCentralPassword'] = isset($_POST['password']) ? $_POST['password'] : null;
				$data['WholesaleCentralEnableCron'] = isset($_POST['enable_cron']) ? $_POST['enable_cron'] : null;
				$data['WholesaleCentralUseOverviewDescription'] = isset($_POST['use_overview']) ? $_POST['use_overview'] : null;
				
				print json_encode($controller->updateAction($data));
				return;
				
				break;
			
			case 'downloadFeed':
				$controller->downloadFeedAction('content/cache/tmp/wholesalecentral.txt');
				return;
				break;
				
			case 'sendFtp':
				print json_encode($controller->sendFeedAction('content/cache/tmp/wholesalecentral.txt'));
				return;
				break;
			default: $controller->indexAction();
		}
	}
	
	pageHeader("WholesaleCentral.com EZFeed Export", "", "icb-shopping-cart");
	
	pageNote(array(
		array("class"=>"", "text"=> '<img src="images/admin/wholesalecentral.png" />', "url"=>"", "visible"=>true),
		array("class"=>"ic-info", "text"=> $label_app_name.' is now integrated with the world\'s largest wholesale trading market, Wholesale Central. This is a great opportunity for you to get your wholesale products into a massive marketplace (no retail!). Get in front of new wholesale bulk buyers looking for your products every single day. To use this integration you\'ll just need to sign up for a <a href="https://secure.wholesalecentral.com/addcompany_step1.cfm?dbid=1" target="_blank">www.wholesalecentral.com</a> and enter your EZFeed FTP information below.', "url"=>"", "visible"=>true),
		array("class"=>"ic-info", "text"=> 'Click here to visit WholesaleCentral.com', "url"=>"http://www.wholesalecentral.com", "visible"=>true),
		array("class"=>"ic-info", "text"=> 'Click here for help setting up your daily feed', "url"=>"http://www.cartmanual.com/url/setup-cron", "visible"=>true),
	));

	$last_active_tab = isset($last_active_tab)?trim($last_active_tab):"settings";
	
	$adminView = Ddm_AdminView::getInstance();
	
	$settingsFormBuilder = new core_Form_FormBuilder('settings');
	
	$settingsGroup = new core_Form_FormBuilder('basic', array('label'=>'WholesaleCentral.com EZFeed Settings', 'class'=>'ic-product', 'first'=>true));
	$settingsFormBuilder->add($settingsGroup);
	
	$settingsGroup->add('username', 'text', array('label'=>'EZFeed FTP Username', 'required'=>true, 'lockable'=>false, 'lockName'=>'name', 'value'=>$settings['WholesaleCentralUsername']));
	$settingsGroup->add('password', 'password', array('label'=>'EZFeed FTP Password', 'required'=>true, 'lockable'=>false, 'value'=>$settings['WholesaleCentralPassword']));
	$settingsGroup->add('enable_cron', 'checkbox', array('label'=>'Enable automatic daily feed', 'value'=>'1', 'current_value'=>$settings['WholesaleCentralEnableCron']));
	$settingsGroup->add('use_overview', 'checkbox', array('label'=>'Use product overview for description column?', 'value'=>'1', 'current_value'=>$settings['WholesaleCentralUseOverviewDescription']));
	
	
	$settingsForm = $settingsFormBuilder->getForm();
	$settingsForm->setTooltips(array(
		'username' => 'Please enter your EZFeed FTP username that was setup in WholesaleCentral.com.',
		'password' => 'Please enter your EZFeed FTP password that was setup in WholesaleCentral.com.',
		'enable_cron' => 'This option allows your EZFeed file to be uploaded automatically for you once per day. Additional cron setup may be required.',
		'use_overview' => 'If checked, the description column in the EZFeed file will contain the products overview description.'
	));
	
	
	
	$datafeedFormBuilder = new core_Form_FormBuilder('settings');
	
	$datafeedGroup = new core_Form_FormBuilder('basic', array('label'=>'WholesaleCentral.com EZFeed Settings', 'class'=>'ic-product', 'first'=>true));
	$datafeedFormBuilder->add($datafeedGroup);
	
	$datafeedGroup->add('sendftp', 'submit', array('label'=>'Send EZFeed via FTP', 'required'=>false, 'lockable'=>false, 'value'=>'Send Now'));
	$datafeedGroup->add('download', 'submit', array('label'=>'Download EZFeed for Manual Upload', 'required'=>false, 'lockable'=>false, 'value'=>'Download Now'));
	
	$datafeedForm = $datafeedFormBuilder->getForm();
	$datafeedForm->setTooltips(array(
		'sendftp' => 'Pushing this button will generate a new EZFeed file and send it via FTP to WholesaleCentral.com.',
		'download' => 'Pushing this button will generate a new EZFeed file that you can upload manually at WholesaleCentral.com'
	));
	
	
	
	$adminView->assign('settingsForm', $settingsForm);
	$adminView->assign('datafeedForm', $datafeedForm);
?>

<script type="text/javascript">
$(document).ready(function(){
	
	$('#wholesalecentral-update').click(function(){
		adminShowSpinner('Saving settings');
		
		$.ajax({
			url: 'admin.php?p=export_wholesalecentral&action=update&chart=true',
			dataType: 'json',
			type: 'POST',
			data: $("#wholesalecentral-settings").serialize(),
			success: function(response){
				if (response.status == 'success')
				{
					adminShowSpinner('Settings saved', null, 1000);
				}
			}
		});
		
		return false;
	});
	
	$('#field-sendftp').click(function(){
		
		if (validateCredentials() == false)
		{
			
			return false;
		}
		
		adminShowSpinner('Uploading via EZFeed FTP');
		$.getJSON('admin.php?p=export_wholesalecentral&action=sendFtp&chart=true', function(response){
			if (response.status == 'success')
			{
				adminShowSpinner('Upload successful', null, 1500);
			}
			else
			{
				adminHideSpinner();
				alert('The EZFeed file could not be uploaded\n\n'+response.message);
			}
		});
		
		return false;
	});
	
	$('#field-download').click(function(){
		$('#ezfeed-download').attr('src', 'admin.php?p=export_wholesalecentral&action=downloadFeed&chart=true&no-acceleration=true');
		return false;
	});
});

function validateCredentials()
{
	if ($.trim($('#field-username').val()) == '' || $.trim($('#field-password').val()) == '')
	{
		alert('Please enter your EZFeed username and password and try again');
		setActiveTab('tabs-wholesalecentral', 'settings');
		return false;
	}
}

</script>

<form id="wholesalecentral-settings" name="frmwholesalecentralSettings" action="admin.php?p=export_wholesalecentral" method="post">
	<input type="hidden" name="action" value="update">

	<div class="admin-tabs admin-tabs-framed" id="tabs-wholesalecentral">
<?php 
		pageTabs(array(
			array("title"=>"Settings", "id"=>"settings"),
			array("title"=>"Export Options", "id"=>"datafeed"),
		), "tabs-wholesalecentral", $last_active_tab);
	
	$adminView->render('pages/settings/wholesalecentral');
?>
	</div>
</form>