<?php

	require_once("content/classes/class.products_locations.php");
	require_once("content/classes/class.regions.php");
	
	$productsLocations = new ProductsLocations($db);
	$regions = new Regions($db);
	$regions->demo = defined('DEMO_LOCK') && DEMO_LOCK;
	
	$countries = $regions->getCountriesList();
	
	$countriesStates = $regions->getCountriesStates(true); 
	
	$mode = isset($_REQUEST["mode"]) ? $_REQUEST["mode"] : "new";
	$action = isset($_REQUEST["action"]) ? $_REQUEST["action"] : "";
	$productsLocationId = isset($_REQUEST["products_location_id"]) ? intval($_REQUEST["products_location_id"]) : 0;

	$data = isset($_POST["data"]) ? $_POST["data"] : array();
	
	$message = "";
	
	if ($action != '')
	{
		$nonce = isset($nonce) ? $nonce : '';
		$nonce_valid = verify_nonce($nonce, 'products_location_'.$mode);
		if (!$nonce_valid)
		{
			invalid_nonce_message();
			exit;
		}
	}
	
	/**
	 * Add/update record
	 */
	if (!DEMO_LOCK)
	{
		switch ($action)
		{
			/**
			 * Add a new record
			 */
			case "add" :
			{
				if ($productsLocations->validate($data))
				{
					$productsLocationId = $productsLocations->add($data);
					$mode = "update";
					$message = "New record has been created";
				}
				break;		
			}
			/**
			 * Update record
			 */
			case "update" :
			{
				if ($productsLocations->validate($data, $productsLocationId))
				{
					$productsLocations->update($productsLocationId, $data);
					$message = "Record has been updated";
				}
				break;
			}
		}
	}
	
	/**
	 * Get data for form
	 */
	switch ($mode)
	{
		case "update" :
		{
			if ($data = $productsLocations->getById($productsLocationId))
			{
				$action = "update";
				break;
			}
		}
		default :
		case "add" :
		{
			if ($action != "add")
			{
				$mode = "add";
				$data = array();
				$data["location_type"] = "";
				$data["country"] = "1";
				$data["send_notification"] = "1";
				$data["email_type"] = "html";
				$data["is_active"] = 1;
				$action = "add";
			}
			break;		
		}
	}
	
	pageHeader("Products Location - ".ucfirst($mode), '', 'icb-location');
	
	$errors = $productsLocations->errors();
	pageNote(array(
		array("text" => "Go back to product locations", "class"=>"ic-info", "url"=>"admin.php?p=products_locations", "visible"=>true),
		array("text" => $message, "class"=>"ic-info", "url"=>"", "visible"=>$message != ""),
		array("text" => "<b>Please check errors below:</b><br/>".implode("<br/>", $errors?$errors:array()), "class"=>"ic-cross", "url"=>"", "visible"=>$errors)
	));
?>
<form name="frmProductsLocation" action="admin.php?p=products_location" method="post">
	<input type="hidden" name="mode" value="<?=htmlspecialchars($mode)?>"/>
	<input type="hidden" name="action" value="<?=htmlspecialchars($action)?>"/>
	<input type="hidden" name="products_location_id" value="<?=$productsLocationId?>"/>
	<input type="hidden" name="nonce" value="<?=create_nonce('products_location_'.$mode)?>"/>
	<div class="admin-tabs admin-tabs-framed" id="tabs-location">
<?php 
	pageTabs(array(
		array("title"=>"Common Information", "id"=>"common"),
		array("title"=>"Notes &amp; Custom Text, etc", "id"=>"notes")
	), "tabs-location", false);
	
	loadJS('products_locations.js');
?>
<script type="text/javascript"><!--
	var countriesStates = <?=json_encode($countriesStates)?>;
	var stateId = '<?=isset($data["state"]) ? intval($data["state"]) : 0 ?>';
--></script>
		<div id="admin-tab-sheet-common" class="admin-tab-sheet hidden">
			<div class="admin-tab-sheet-wrap">
				<div class="admin-form-header first"><span class="icon ic-icon ic-location"></span>Product Location</div>
				<table width="100%" cellpadding="0" cellspacing="0" border="0">
					<tr valign="top">
						<td class="formItemCaption">Status</td>
						<td class="formItemControl">
							<select name="data[is_active]">
								<option value="1" <?=$data["is_active"]=="1"?"selected":""?>>Active</option>
								<option value="0" <?=$data["is_active"]=="0"?"selected":""?>>Disabled</option>
							</select>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption required">Name</td>
						<td class="formItemControl"><input maxlength="100" type="text" name="data[name]" value="<?=htmlspecialchars(isset($data["name"])?$data["name"]:"")?>"></td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption required">Unique ID / Code</td>
						<td class="formItemControl"><input type="text" name="data[code]" maxlength="100" value="<?=htmlspecialchars(isset($data["code"])?$data["code"]:"")?>"></td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Type</td>
						<td class="formItemControl">
							<select name="data[location_type]">
								<option value="drop-shipper" <?=$data["location_type"]=="drop-shipper"?"selected":""?>>Drop shipper</option>
								<option value="warehouse" <?=$data["location_type"]=="warehouse"?"selected":""?>>Warehouse</option>
								<option value="store" <?=$data["location_type"]=="store"?"selected":""?>>Store</option>
								<option value="manufacturer" <?=$data["location_type"]=="manufacturer"?"selected":""?>>Manufacturer</option>
								<option value="distributor" <?=$data["location_type"]=="distributor"?"selected":""?>>Distributor</option>
								<option value="other" <?=$data["location_type"]=="other"?"selected":""?>>Other</option>
							</select>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption required">Contact name</td>
						<td class="formItemControl"><input maxlength="100" type="text" name="data[contact_name]" value="<?=htmlspecialchars(isset($data["contact_name"])?$data["contact_name"]:"")?>"></td>
					</tr>
					
					<tr valign="top">
						<td class="formItemCaption required">Address line 1</td>
						<td class="formItemControl"><input maxlength="100" type="text" name="data[address1]" value="<?=htmlspecialchars(isset($data["address1"])?$data["address1"]:"")?>"></td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Address line 2</td>
						<td class="formItemControl"><input maxlength="100" type="text" name="data[address2]" value="<?=htmlspecialchars(isset($data["address2"])?$data["address2"]:"")?>"></td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption required">City</td>
						<td class="formItemControl"><input maxlength="100" type="text" name="data[city]" value="<?=htmlspecialchars(isset($data["city"])?$data["city"]:"")?>"></td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption required">Country</td>
						<td class="formItemControl">
							<select name="data[country]" id="locationCountry">
			<?php				foreach($countries as $countryId => $country)
								{
									echo '<option value="'.$country["coid"].'" '.($country["coid"] == $data["country"] ? 'selected="selected"' : "").'>'.htmlspecialchars($country["name"]).'</option>';
									
								} 
			?>
							</select>
						</td>
					</tr>
					<tr valign="top" id="stateRow">
						<td class="formItemCaption required">State</td>
						<td class="formItemControl">
							<select name="data[state]" id="locationState"></select>
						</td>
					</tr>
					<tr valign="top" id="provinceRow">
						<td class="formItemCaption required">Province</td>
						<td class="formItemControl"><input maxlength="100" type="text" name="data[province]" value="<?=htmlspecialchars(isset($data["province"])?$data["province"]:"")?>"></td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption required">ZIP</td>
						<td class="formItemControl"><input maxlength="100" class="short" type="text" name="data[zip]" value="<?=htmlspecialchars(isset($data["zip"])?$data["zip"]:"")?>"></td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption required">Phone number</td>
						<td class="formItemControl"><input maxlength="100" type="text" name="data[phone]" value="<?=htmlspecialchars(isset($data["phone"])?$data["phone"]:"")?>"></td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption required">Contact email address</td>
						<td class="formItemControl"><input maxlength="100" type="text" name="data[contact_email]" value="<?=htmlspecialchars(isset($data["contact_email"])?$data["contact_email"]:"")?>"></td>
					</tr>
				</table>
				<div class="admin-form-header"><span class="icon ic-icon ic-envelope"></span>Notification Settings</div>
				<table width="100%" cellpadding="0" cellspacing="0" border="0">
					<tr valign="top">
						<td class="formItemCaption">Notify facility about new orders?</td>
						<td class="formItemControl">
							<select name="data[send_notification]" class="short">
								<option value="1" <?=$data["send_notification"]=="1"?"selected":""?>>Yes</option>
								<option value="0" <?=$data["send_notification"]=="0"?"selected":""?>>No</option>
							</select>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Notification email address</td>
						<td class="formItemControl">
							<input maxlength="100" type="text" name="data[notify_email]" value="<?=htmlspecialchars(isset($data["notify_email"])?$data["notify_email"]:"")?>">
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Notification email type</td>
						<td class="formItemControl">
							<select name="data[email_type]" class="short">
								<option value="html" <?=$data["email_type"]=="html"?"selected":""?>>HTML</option>
								<option value="text" <?=$data["email_type"]=="text"?"selected":""?>>Text</option>
							</select>
						</td>
					</tr>
				</table>
			</div>
		</div>
		<div id="admin-tab-sheet-notes" class="admin-tab-sheet hidden">
			<div class="admin-tab-sheet-wrap">
				<div class="admin-form-header first"><span class="icon ic-icon ic-pencil"></span>Product Location Notes</div>
				<div class="formItemControl big">
					<textarea class="no-mce" name="data[notes]"><?=isset($data["notes"])?htmlspecialchars($data["notes"]):""?></textarea>
				</div>
				
				<div class="admin-form-header"><span class="icon ic-icon ic-pencil"></span>Notification Email Custom Text</div>
				<div class="formItemControl big">
					<textarea class="no-mce" style="width:100%;height:200px;" name="data[email_text]"><?=isset($data["email_text"])?htmlspecialchars($data["email_text"]):""?></textarea>
					<div class="formItemComment">
						To edit notification email template <a href="admin.php?p=email_notification&id=products_location_notification">please click here</a>.
					</div>
				</div>
			</div>
		</div>
	</div>
	<div class="admin-form-buttons">
		<input type="submit" value="Save changes" />
		<input type="reset" value="Reset Form" />
	</div>
</form>

