<?php
	if (!defined("ENV")) exit();

	pageHeader("Search Engine Settings", "", "icb-zoom");
	pageNote(array(
		array("class"=>"ic-info", "text"=>"<b>Before enabling or regenerating flat URLs, keep in mind the format in which current pages are indexed.</b>", "url"=>"", "visible"=>true)
	));
	
	$action = empty($action)?"":$action;
	$is_htaccess_error = false;
	$is_robots_error = false;
	
	if (in_array($action, array("update", "force-htaccess", "force-url", "force-unique")) && !DEMO_LOCK)
	{

		$nonce = isset($nonce) ? $nonce : '';
		$nonce_valid = verify_nonce($nonce, 'settings_search_'.$action);
		if (!$nonce_valid)
		{
			invalid_nonce_message();
			exit;
		}
		
		/**
		 * Save robots.txt file
		 */
		if ($action == "update")
		{
			$is_robots_error = !file_put_contents("robots.txt", $robots);
		}
		
		/**
		 * Update settings in DB
		 */
		if ($action == "update" && !DEMO_LOCK)
		{
			foreach($form as $var=>$value)
			{
				$db->reset();
				$db->assignStr("value", $value);
				$db->update(DB_PREFIX."settings", "WHERE name='".$db->escape($var)."'");
				$settings[$var] = $value;
			}
		}
		
		/**
		 * Create .htaccess file
		 */
		if (in_array($action, array("update", "force-htaccess")) && !DEMO_LOCK)
		{
			try
			{
				$form["USE_MOD_REWRITE"] = isset($form["USE_MOD_REWRITE"]) ? $form["USE_MOD_REWRITE"] : $settings["USE_MOD_REWRITE"];
				Seo::generateHtaccessFile(($form["USE_MOD_REWRITE"]=="YES"?"On":"Off"));
			} catch (Exception $e)
			{
				$is_htaccess_error = true;
			}
		}
		
		/**
		 * Generate flat URLs
		 */
		if (in_array($action, array("update", "force-url", "force-unique")) && !DEMO_LOCK)
		{
			//check are there changes in URL templates
			if (isset($_SESSION["seo-settings"]))
			{
				//category template changed
				$updateCategories = (
					$_SESSION["seo-settings"]["SearchURLCategoryTemplate"] != $settings["SearchURLCategoryTemplate"] ||
					$_SESSION["seo-settings"]["SearchAutoGenerateLowercase"] != $settings["SearchAutoGenerateLowercase"] ||
					$_SESSION["seo-settings"]["SearchURLJoiner"] != $settings["SearchURLJoiner"] || 
					$_SESSION["seo-settings"]["USE_MOD_REWRITE"] != $settings["USE_MOD_REWRITE"] ||
					$action == "force-url"
				);
			
				//product template changed
				$updateProducts = (
					$_SESSION["seo-settings"]["SearchURLProductTemplate"] != $settings["SearchURLProductTemplate"] ||
					$_SESSION["seo-settings"]["SearchAutoGenerateLowercase"] != $settings["SearchAutoGenerateLowercase"] ||
					$_SESSION["seo-settings"]["SearchURLJoiner"] != $settings["SearchURLJoiner"] ||
					$_SESSION["seo-settings"]["USE_MOD_REWRITE"] != $settings["USE_MOD_REWRITE"] ||
					$action == "force-url"
				);
			
				//page template changed
				$updatePages = (
					$_SESSION["seo-settings"]["SearchURLPageTemplate"] != $settings["SearchURLPageTemplate"] ||
					$_SESSION["seo-settings"]["SearchAutoGenerateLowercase"] != $settings["SearchAutoGenerateLowercase"] ||
					$_SESSION["seo-settings"]["SearchURLJoiner"] != $settings["SearchURLJoiner"] ||
					$_SESSION["seo-settings"]["USE_MOD_REWRITE"] != $settings["USE_MOD_REWRITE"] ||
					$action == "force-url"
				);
				
				//manufacturer template changed
				$updateManufacturers  = (
					$_SESSION["seo-settings"]["SearchURLManufacturerTemplate"] != $settings["SearchURLManufacturerTemplate"] ||
					$_SESSION["seo-settings"]["SearchAutoGenerateLowercase"] != $settings["SearchAutoGenerateLowercase"] ||
					$_SESSION["seo-settings"]["SearchURLJoiner"] != $settings["SearchURLJoiner"] ||
					$_SESSION["seo-settings"]["USE_MOD_REWRITE"] != $settings["USE_MOD_REWRITE"] ||
					$action == "force-url"
				);
				
				updateSeoURLs($db, $settings, $updateCategories, $updateProducts, $updatePages, $updateManufacturers);
			}
		}
		
		/**
		 * Generate unique URL's
		 */
		if ($action == "force-unique" && !DEMO_LOCK)
		{
			$unique_count = Seo::generateUniqueUrls();
			
			$note_message = ($unique_count > 0)
				? 'There was '.$unique_count.' duplicate URL(s) to process.'
				: 'There were '.$unique_count.' duplicate URLs to process.';
			
			pageNote(array(
				array("class"=>"ic-info", "text"=>'<strong>'.$note_message.'</strong>', "url"=>"", "visible"=>true)
			));
		}
	}
	if ($is_htaccess_error || $is_robots_error)
	{
		pageNote(array(
			array("class"=>"ic-cross", "text"=>"<font color=\"#990000\"><b>Please note: Error writing '.htaccess' file. Please check file permissions.</b></font>", "url"=>"", "visible"=>$is_htaccess_error),
			array("class"=>"ic-cross", "text"=>"<font color=\"#990000\"><b>Please note: Error writing 'robots.txt' file. Please check file permissions.</b></font>", "url"=>"", "visible"=>$is_robots_error)
		));
	}
	
	$f = @fopen("robots.txt", "r");
	if ($f)
	{
		$robots = "";
		while(($s = fgets($f, 1024)) != false) $robots.=$s;
		fclose($f);
	}
	
	$_SESSION["seo-settings"] = array(
		"SearchURLCategoryTemplate" => $settings["SearchURLCategoryTemplate"],
		"SearchAutoGenerateLowercase" => $settings["SearchAutoGenerateLowercase"],
		"SearchURLProductTemplate" => $settings["SearchURLProductTemplate"],
		"SearchURLPageTemplate" => $settings["SearchURLPageTemplate"],
		"SearchURLManufacturerTemplate" => $settings["SearchURLManufacturerTemplate"],
		"SearchURLJoiner" => $settings["SearchURLJoiner"],
		"USE_MOD_REWRITE" => $settings["USE_MOD_REWRITE"]
	);
	pageNote(array(
		array("class"=>"ic-technical-wrench", "text"=>"Regenerate .htaccess file", "url"=>"admin.php?p=settings_search&nonce=".create_nonce('settings_search_force-htaccess')."&action=force-htaccess", "visible"=>true),
		array("class"=>"ic-technical-wrench", "text"=>"Regenerate flat URLs", "url"=>"admin.php?p=settings_search&nonce=".create_nonce('settings_search_force-url')."&action=force-url", "visible"=>true),
		array("class"=>"ic-checkmark", "text"=>"Check for duplicate URL's and regenerate unique flat URLs", "url"=>"admin.php?p=settings_search&nonce=".create_nonce('settings_search_force-unique')."&action=force-unique", "visible"=>true)
	));
?>
<form name="frmSettings" action="admin.php?p=settings_search" method="post">
	<input type="hidden" name="action" value="update">
	<input type="hidden" name="nonce" value="<?=create_nonce('settings_search_update')?>" />
	<div class="admin-tabs admin-tabs-framed" id="tabs-search">
<?php
	pageTabs(array(
		array("title"=>"Search Engine Settings", "id"=>"main")
	), "main", false); 
?>

		<div id="admin-tab-sheet-main" class="admin-tab-sheet">
			<div class="admin-tab-sheet-wrap">
				<div class="admin-form-header first"><span class="icon ic-icon"></span>Search Engine Optimization Settings</div>
				
				<table width="100%" cellpadding="0" cellspacing="0" border="0">
					<tr valign="top">
						<td class="formItemCaption">Page (Site) Title</td>
						<td class="formItemControl">
							<input type="text" name="form[SearchTitle]" style="width:100%;" value="<?=htmlspecialchars($settings["SearchTitle"])?>">
							<div class="formItemComment">
								Site title tag (&lt;title&gt; tag), should not be more than 60 characters.
							</div>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Enable flat URLs generation?</td>
						<td class="formItemControl">
<?php	
	if (AccessManager::checkAccess('FLAT_URL'))
	{
?>
							<select name="form[USE_MOD_REWRITE]" class="short">
								<option value="YES">Yes</option>
								<option value="NO" <?=$settings["USE_MOD_REWRITE"]=="NO"?"selected":""?>>No</option>
							</select>
<?php
	}
	else
	{
		AccessManager::notAllowed();
	}
?>
							<div class="formHelp">Please note: this feature requires MOD_REWRITE Apache Web server Module and does not work on all servers.</div>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Lowercase generated URLs?</td>
						<td class="formItemControl">
							<select name="form[SearchAutoGenerateLowercase]" class="short">
								<option value="YES">Yes</option>
								<option value="NO" <?=$settings["SearchAutoGenerateLowercase"]=="NO"?"selected":""?>>No</option>
							</select>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">URL words joiner</td>
						<td class="formItemControl">
							<select name="form[SearchURLJoiner]" class="short">
								<option value="-">- (dash)</option>
								<option value="_" <?=$settings["SearchURLJoiner"]=="_"?"selected":""?>>_ (underscore)</option>
								<option value="" <?=$settings["SearchURLJoiner"]==""?"selected":""?>>none</option>
							</select>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Category page URL template</td>
						<td class="formItemControl">
							<input type="text" name="form[SearchURLCategoryTemplate]" style="width:100%;" value="<?=htmlspecialchars($settings["SearchURLCategoryTemplate"])?>">
							<div class="formItemComment">
								Template Variables:<br/>
								%CategoryKey%, %CategoryName%, %CategoryDbId%, %CategoryPath% - at least one variable required<br/>
							</div>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Category page pagination</td>
						<td class="formItemControl">
							<input type="text" name="form[SearchURLCategoryPaginatorTemplate]" style="width:100%;" value="<?=htmlspecialchars($settings["SearchURLCategoryPaginatorTemplate"])?>">
							<div class="formItemComment">
								Template Variables:<br/>
								%PageNumber% - required<br/>
								Will be added before extension (if exists) or closing slash symbol. Resulting URL will look like:<br/>
								[category-url][page-number].html<br/>
								[category-url][page-number]<br/>
								[category-url][page-number]/<br/>
							</div>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Product page URL template</td>
						<td class="formItemControl">
							<input type="text" name="form[SearchURLProductTemplate]" style="width:100%;" value="<?=htmlspecialchars($settings["SearchURLProductTemplate"])?>">
							<div class="formItemComment">
								Template variables:<br/>
								%CategoryKey%, %CategoryName%, %CategoryDbId%, %CategoryPath%, %CategoryURL%<br/>
								%ProductId%, %ProductName%, %ProductDbId% - at least one of product variables is required<br/>
							</div>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Manufacturer page URL template</td>
						<td class="formItemControl">
							<input type="text" name="form[SearchURLManufacturerTemplate]" style="width:100%;" value="<?=htmlspecialchars($settings["SearchURLManufacturerTemplate"])?>">
							<div class="formItemComment">
								Template Variables:<br/>
								%ManufacturerKey%, %ManufaturerName%, %ManufacturerDbId% - at least one variable required<br/>
							</div>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Manufacturer page pagination</td>
						<td class="formItemControl">
							<input type="text" name="form[SearchURLManufacturerPaginatorTemplate]" style="width:100%;" value="<?=htmlspecialchars($settings["SearchURLManufacturerPaginatorTemplate"])?>">
							<div class="formItemComment">
								Template Variables:<br/>
								%PageNumber% - required<br/>
								Will be added before extension (if exists) or closing slash symbol. Resulting URL will look like:<br/>
								[manufacturer-url][page-number].html<br/>
								[manufacturer-url][page-number]<br/>
								[manufacturer-url][page-number]/<br/>
							</div>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Text page URL template</td>
						<td class="formItemControl">
							<input type="text" name="form[SearchURLPageTemplate]" style="width:100%;" value="<?=htmlspecialchars($settings["SearchURLPageTemplate"])?>">
							<div class="formItemComment">
								Template variables:<br/>
								%PageName%, %PageTitle%
							</div>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Allow keyword auto-generation?</td>
						<td class="formItemControl">
							<select name="form[SearchAutoGenerate]" class="short">
								<option value="YES">Yes</option>
								<option value="NO" <?=$settings["SearchAutoGenerate"]=="NO"?"selected":""?>>No</option>
							</select>			
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">.htaccess Overrides</td>
						<td class="formItemControl">
							<textarea class="no-mce" cols="50" rows="8" name="form[SearchHtaccessOverrides]"><?=htmlspecialchars($settings["SearchHtaccessOverrides"])?></textarea>
							<div class="formItemComment">
								<span style="color:red">Note: Use extreme caution when customizing the .htaccess file.  A misconfiguration may cause your site to be inaccessible.</span><br/>
								These changes will be added to the .htaccess file prior to the default RewriteRules.
							</div>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Meta Keywords</td>
						<td class="formItemControl">
							<textarea class="no-mce" cols="50" rows="8" name="form[SearchMetaKeywords]"><?=htmlspecialchars($settings["SearchMetaKeywords"])?></textarea>
							<div class="formItemComment">Recommended length is 200-1000 characters. Separate keywords and phrases by using a comma</div>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Meta Description</td>
						<td class="formItemControl">
							<textarea class="no-mce" cols="50" rows="3" name="form[SearchMetaDescription]"><?=htmlspecialchars($settings["SearchMetaDescription"])?></textarea>
							<div class="formItemComment">Recommended length is 6-125 characters.</div>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">File "robots.txt"</td>
						<td class="formItemControl">
							<textarea class="no-mce" cols="50" wrap="off" rows="8" name="robots"><?=htmlspecialchars($robots)?></textarea>
							<div class="formItemComment">This file creates a robots.txt file that will be located in your main directory. This file makes it easier for the search engines to index your website.</div>
						</td>
					</tr>
				</table>
			</div>
		</div>
	</div>
	<div class="admin-form-buttons">
		<input type="submit" value="Save changes"/>
		<input type="reset" value="Reset form"/>
	</div>
</form>	
