<?php
	if (!defined("ENV")) exit();

	require_once("content/classes/class.regions.php");
	require_once("content/classes/class.taxes.php");

	$taxes = new Taxes($db);
	$taxes->demo = defined('DEMO_LOCK') && DEMO_LOCK;
	
	$action = isset($_REQUEST["action"])?trim($_REQUEST["action"]):"";
	
	if ($action != '')
	{
		$nonce = isset($nonce) ? $nonce : '';
		$nonce_valid = verify_nonce($nonce, 'tax_'.$action);
		if (!$nonce_valid)
		{
			invalid_nonce_message();
			exit;
		}
	}
	$tax_rate_added = false;
	$tax_rate_updated = false;
	switch($action)
	{
		case "add" :
		{
			$rate_id = $taxes->addRate($_REQUEST); 
			$_REQUEST["rate_id"] = $rate_id;
			$_REQUEST["mode"] = "edit";
			$tax_rate_added = true;
			break;
		}
		case "save" :
		{
			$rate_id = isset($_REQUEST["rate_id"])?intval($_REQUEST["rate_id"]):0;
			$taxes->saveRate($rate_id, $_REQUEST);
			$_REQUEST["mode"] = "edit";
			$tax_rate_updated = true;
			break;
		}
	}
	
	$mode = isset($_REQUEST["mode"])?$_REQUEST["mode"]:"add";
	$mode = in_array($mode, array("add", "edit"))?$mode:"add";
	switch($mode)
	{
		case "edit" :
		{
			$rate_id = isset($_REQUEST["rate_id"])?intval($_REQUEST["rate_id"]):0;
			$_tax_rate = $taxes->getRateById($rate_id);
			if ($_tax_rate)
			{
				$class_id = $_tax_rate["class_id"];
				$zone_id = $_tax_rate["zone_id"];
				$rate_priority = $_tax_rate["rate_priority"];
				$tax_rate = $_tax_rate["tax_rate"];
				$user_level = explode(",", $_tax_rate["user_level"]);
				$rate_description = $_tax_rate["rate_description"];
				$action = "save";
				break;
			}
		}
		default :
		case "add" :
		{
			$mode = "add";
			$class_id = 0;
			$zone_id = 0;
			$rate_priority = 1;
			$tax_rate = 0.00;
			$user_level = array("0","1","2","3");
			$rate_description = "";
			$action = "add";
			$rate_id=0;
			break;
		}
	}
	
	$taxes_zones = $taxes->getTaxZonesList();
	$taxes_classes = $taxes->getTaxClassesList();
	
	pageHeader("Tax Rate", ucfirst($mode), "icb-tax");
	
	pageNote(array(
		array("class"=>"ic-world", "text"=>"Back to Taxes Rates", "url"=>"admin.php?p=taxes&amp;last_active_tab=rates&amp;tax_mode=advanced", "visible"=>true),
		array("class"=>"ic-warning", "text"=>"<b>Please note: There are no tax zones defined.</b><br/>Please <a href=\"admin.php?p=taxes&last_active_tab=zones\">click here</a> to add", "url"=>"", "visible"=>(count($taxes_zones) < 1)),
		array("class"=>"ic-warning", "text"=>"<b>Please note: There are no tax classes defined.</b><br/>Please <a href=\"admin.php?p=taxes&last_active_tab=classes\">click here</a> to add", "url"=>"", "visible"=>(count($taxes_classes) < 1)),
		array("class"=>"ic-warning", "text"=>"<b>Tax rate added.</b>", "url"=>"", "visible"=>$tax_rate_added),
		array("class"=>"ic-warning", "text"=>"<b>Tax rate updated.</br>", "url"=>"", "visible"=>$tax_rate_updated),
	));
	
	if (count($taxes_zones) > 0 && count($taxes_classes) > 0)
	{
?>
<div class="admin-tabs admin-tabs-framed" id="tabs-rate">
<?php 
		pageTabs(array(
				array("title"=>"Tax Rates", "id"=>"main"),
		), "tabs-rate", false);
	
?>
	<div id="admin-tab-sheet-main" class="admin-tab-sheet">
		<div class="admin-tab-sheet-wrap">
			<div class="admin-form-header first"><span class="icon ic-icon"></span>Taxes Rate Data</div>
			<form name="frmTaxesRate" method="POST" action="admin.php?p=taxes_rate">
				<input type="hidden" name="action" value="<?=$action?>"/>
				<input type="hidden" name="rate_id" value="<?=$rate_id?>"/>
				<input type="hidden" name="nonce" value="<?=create_nonce('tax_'.$action)?>" />
				<table width="100%" cellpadding="0" cellspacing="0" border="0">
					<tr valign="top">
						<td class="formItemCaption">Tax Zone</td>
						<td class="formItemControl">
							<select name="zone_id">
<?php	
		for ($i=0; $i<count($taxes_zones); $i++)
		{
?>
							<option value="<?=$taxes_zones[$i]["zone_id"]?>" <?=$taxes_zones[$i]["zone_id"]==$zone_id?'selected="selected"':''?>><?=htmlspecialchars($taxes_zones[$i]["zone_name"])?></option>
<?php
		}
?>					
							</select>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Tax Class</td>
						<td class="formItemControl">
							<select name="class_id">
<?php
		for ($i=0; $i<count($taxes_classes); $i++)
		{
?>
								<option value="<?=$taxes_classes[$i]["class_id"]?>" <?=$taxes_classes[$i]["class_id"]==$class_id?'selected="selected"':''?>><?=htmlspecialchars($taxes_classes[$i]["class_name"])?></option>
<?php
		}
?>							
							</select>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Rate Priority</td>
						<td class="formItemControl">
							<select name="rate_priority" class="short">
<?php
		for ($i=1; $i<=20; $i++)
		{
?>
								<option value="<?=$i?>" <?=$rate_priority==$i?'selected="selected"':''?>><?=$i?></option>
<?php
		}
?>					
							</select>
							<div class="">Rates at the same priority are added, others are compounded</div>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption"><b>Tax Rate (%)</b></td>
						<td class="formItemControl">
							<input type="text" name="tax_rate" class="short" value="<?=myNum($tax_rate)?>"/>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption">Apply To</td>
						<td class="formItemControl">
							<input type="checkbox" name="user_level[0]" value="0" <?=in_array(0, $user_level)?'checked="checked"':''?>/> Simple Users<br/>
							<input type="checkbox" name="user_level[1]" value="1" <?=in_array(1, $user_level)?'checked="checked"':''?>/> Wholesalers Level 1<br/>
							<input type="checkbox" name="user_level[2]" value="2" <?=in_array(2, $user_level)?'checked="checked"':''?>/> Wholesalers Level 2<br/>
							<input type="checkbox" name="user_level[3]" value="3" <?=in_array(3, $user_level)?'checked="checked"':''?>/> Wholesalers Level 3<br/>
						</td>
					</tr>
					<tr valign="top">
						<td class="formItemCaption"><b>Tax Rate Description</b> :</td>
						<td class="formItemControl">
							<input type="text" name="rate_description" maxlength="255" value="<?=htmlspecialchars($rate_description)?>"/>
						</td>
					</tr>
				</table>
				<div class="admin-form-buttons">
					<input type="submit" value="Save changes" />
					<input type="reset" value="Reset form" />
				</div>
			</form>
		</div>
	</div>
</div>
<?php 
	}
?>