<?php
//plugin class
class plugin_dollar_days_import extends cartPlugin
{
	//protected
	var $plugin_id = "dollar_days_import";
	var $plugin_group = "datafeed";
	var $plugin_caption = "Dollar Days";
	var $plugin_description = "AMERICA'S #1 WHOLESALE MERCHANDISE DISTRIBUTOR!<br>Calendars, Housewares, Jewelery and more. View all available products at <a href=\"http://www.wowpricing.com\" target=\"_blank\">http://www.wowpricing.com</a><br><br>For additional information call our Customer Service Department at 1-800-506-0398 or email <a href=\"mailto:cartpartners@dollardays.com\">cartpartners@dollardays.com</a><br><br><a href=\"admin.php?p=settings_dollar_days\">Setup Dollar Days</a>";

	//private
	var $datafeed_cats_url = "";
	var $datafeed_cats_file = "content/cache/tmp/dollar-days-categories.csv";

	var $datafeed_prod_url = "";
	var $datafeed_prod_file = "content/cache/tmp/dollar-days-products.csv";

	var $curl_error = "";
	var $curl_errno = 0;

	var $events_log = false;
	var $events_log_file = "content/cache/tmp/dollar-days-log.csv";

	//constructor
	function plugin_dollar_days_import(){
		$this->datafeed_cats_url = "http://www.dollardays.com/feeds/".getp()."-list.aspx";
		$this->datafeed_prod_url = "http://www.dollardays.com/feeds/".getp().".aspx?cid=0";
		return $this;
	}

	//log
	function log($msg){
		if($this->events_log){
			$f = @fopen($this->events_log_file, "a+");
			if($f){
				@fputs($f, date("Y/m/d-H:i:s  ").$msg."\r\n");
				@fclose($f);
			}
		}
	}

	//get curl data
	function getCURL($url, $file = false)
	{
		global $settings;
		
		if ($file == $this->datafeed_prod_file)
		{
			$c = curl_init($url);
			
			if ($settings["ProxyAvailable"] == "YES")
			{
				//curl_setopt($c, CURLOPT_VERBOSE, 1);
				if (defined("CURLOPT_PROXYTYPE") && defined("CURLPROXY_HTTP") && defined("CURLPROXY_SOCKS5"))
				{
					curl_setopt($c, CURLOPT_PROXYTYPE, $settings["ProxyType"] == "HTTP" ? CURLPROXY_HTTP : CURLPROXY_SOCKS5);
				}
				curl_setopt($c, CURLOPT_PROXY, $settings["ProxyAddress"].":".$settings["ProxyPort"]);
				if ($settings["ProxyRequiresAuthorization"] == "YES")
				{
					curl_setopt($c, CURLOPT_PROXYUSERPWD, $settings["ProxyUsername"].":".$settings["ProxyPassword"]);
				}
				curl_setopt($c, CURLOPT_TIMEOUT, 120);
			}
			curl_setopt($c, CURLOPT_HEADER, 1);
			curl_setopt($c, CURLOPT_RETURNTRANSFER, 1);
			
			$buffer = curl_exec($c);
			
			curl_close($c);
			
			$matches = array();
			if (preg_match('/Location: (.*)/', $buffer, $matches) && isset($matches[1]))
			{
				$location = $matches[1];
				if (substr($location, 0, 5) !== 'http' && substr($location, 0, 2) === '//')
					$location = 'http:'.$location;

				return $this->_getCURL($location, $file);
			}
		}
		else
		{
			return $this->_getCURL($url, $file);
		}
	}
	
	function _getCURL($url, $file = false)
	{
		global $settings;
		
		$c = curl_init($url);
		
		if ($settings["ProxyAvailable"] == "YES")
		{
			//curl_setopt($c, CURLOPT_VERBOSE, 1);
			if (defined("CURLOPT_PROXYTYPE") && defined("CURLPROXY_HTTP") && defined("CURLPROXY_SOCKS5"))
			{
				curl_setopt($c, CURLOPT_PROXYTYPE, $settings["ProxyType"] == "HTTP" ? CURLPROXY_HTTP : CURLPROXY_SOCKS5);
			}
			curl_setopt($c, CURLOPT_PROXY, $settings["ProxyAddress"].":".$settings["ProxyPort"]);
			if ($settings["ProxyRequiresAuthorization"] == "YES")
			{
				curl_setopt($c, CURLOPT_PROXYUSERPWD, $settings["ProxyUsername"].":".$settings["ProxyPassword"]);
			}
			curl_setopt($c, CURLOPT_TIMEOUT, 120);
		}
		curl_setopt($c, CURLOPT_FOLLOWLOCATION, true);
		curl_setopt($c, CURLOPT_MAXREDIRS, 5);
		
		if ($file)
		{
			$f = fopen($settings["GlobalServerPath"]."/".$file, "w");
			curl_setopt($c, CURLOPT_FILE, $f);	
		}
		else
		{
			curl_setopt($c, CURLOPT_RETURNTRANSFER, 1);
		}
		
		$this->log("Get data from ".$url);
		
		curl_setopt($c, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($c, CURLOPT_CAPATH, $settings['GlobalServerPath'].$settings['SecuritySslDirectory']);
		curl_setopt($c, CURLOPT_CAINFO, $settings['GlobalServerPath'].$settings['SecuritySslDirectory'].'/'.$settings['SecuritySslPem']);
		
		$buffer = curl_exec($c);
		if (curl_errno($c) > 0)
		{
			$this->curl_errno = curl_errno($c);
			$this->curl_error = curl_error($c);
			$this->log("ERROR: ".$this->curl_error);
			curl_close($c);
			return false;
		}
		curl_close($c);
		if ($file && $f) fclose($f);
		return $buffer;
	}

	//run dollar days plugin
	public function runPlugin(&$db, &$settings, &$request)
	{
		parent::runPlugin($db, $settings, $request);

		global $_SESSION;
		
		if (!isset($_REQUEST["chart"]))
		{
			pageHeader("DollarDays.com Import", '', 'icb-currency-sign-dollar2');
		}
		
		extract($request);
		$step = isset($step)?intval($step):1;

		$dd_step = isset($_SESSION["admin_dd_step"])?intval($_SESSION["admin_dd_step"]):1;
		
		if ($dd_step > $step)
		{
			$step = 1;
		}
		
		// set some default values if this is a DEMO admin user since these values will not be inserted into database
		if (DEMO_LOCK)
		{
			$settings["plugin_dollar_days_import_categories"] = "";
			$settings["plugin_dollar_days_import_last_update"] = "";
			$settings["plugin_dollar_days_import_price_option"] = "inc";
			$settings["plugin_dollar_days_import_price_value"] = "0.00";
		}
		
		switch($step)
		{
			/**
			 * FIRST PAGE - DISPLAY CONTINU BUTTON AND CHECK FOR SETTINGS IN DB
			 */
			default:
			case 1:
			{
				$_SESSION["admin_dd_step"] = 1;
				//check settings
				
				if (!DEMO_LOCK)
				{
					if (!isset($settings["plugin_dollar_days_import_categories"]))
					{
						$db->query("INSERT INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) ".
							"VALUES('No','hidden','plugin_dollar_days_import','plugin_dollar_days_import_categories','','')");
						$settings["plugin_dollar_days_import_categories"] = "";
					}
				
					if (!isset($settings["plugin_dollar_days_import_price_option"]))
					{
						$db->query("INSERT INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) ".
							"VALUES('No','hidden','plugin_dollar_days_import','plugin_dollar_days_import_price_option','inc','inc,dec')");
						$settings["plugin_dollar_days_import_price_option"] = "inc";
					}
				
					if (!isset($settings["plugin_dollar_days_import_price_value"]))
					{
						$db->query("INSERT INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) ".
							"VALUES('No','hidden','plugin_dollar_days_import','plugin_dollar_days_import_price_value','0.00','')");
						$settings["plugin_dollar_days_import_price_value"] = "0.00";
					}
				
					if (!isset($settings["plugin_dollar_days_import_last_update"]))
					{
						$db->query("INSERT INTO ".DB_PREFIX."settings (visible,input_type,group_name,name,value,options) ".
							"VALUES('No','hidden','plugin_dollar_days_import','plugin_dollar_days_import_last_update','','')");
						$settings["plugin_dollar_days_import_last_update"] = "";
					}
				}
				
				pageNote(array(
					array("class"=>"ic-info", "text"=>"This process will guide you through importing Dollar Days products into your store.<br>Click continue below to select to the categories you would like to import.<br>&nbsp;<br><i>Note: It may take up to one minute while updater receives information required for update.</i>"),
					array("class"=>"ic-info", "text"=>"<u>Last Update: ".$settings["plugin_dollar_days_import_last_update"]."</u>", "visible"=>$settings["plugin_dollar_days_import_last_update"]!=""),
					array("class"=>"ic-info", "text"=>"To setup a cron job please use following command:<br/><textarea class=\"no-mce\" style=\"width:500px;height:50px;font-family:arial;font-size:11px;\" readonly>php ".$settings["GlobalServerPath"]."/content/admin/plugins/dollar_days_import/cron.php \"".$settings["GlobalServerPath"]."\"</textarea>", "visible"=>true),
					array("class"=>"ic-info", "text"=>"Need to setup a DollarDays.com account, <a href=\"admin.php?p=settings_dollar_days\">click here</a>", "visible"=>true),
				));
				echo '<div style="clear:both;padding-left:30px;padding-top:10px;"><input onClick="document.location=\'admin.php?p=plugin&plugin_id='.$this->plugin_id.'&step=2\'" type="submit" value="Continue" /></div>';

				break;
			}

			/**
			 * SECOND PAGE - READ AND DISPLAY CATEGORIES TREE WITH PREENTERED SAVED CATEGORIES
			 */ 
			case 2 : 
			{
				$_SESSION["admin_dd_step"] = 2;
				if (!safe_mode()) set_time_limit(100000);
				//downoad data from remove server
				$buffer = $this->getCURL($this->datafeed_cats_url, $this->datafeed_cats_file);

				if (!$buffer)
				{
					pageNote(array(
						array("class"=>"ic-cross", "text"=>"<b>Error occurecd while connecting to Dollar Days server.</b><br/>".$this->curl_error, "url"=>"", "visible"=>true),
						array("class"=>"ic-arrow-left", "text"=>"Back to previous page", "url"=>"javascript:history.back();", "visible"=>true)
					));
					break;
				}

				//save categories data into file
				//$f = fopen($this->datafeed_cats_file, "w");
				//fputs($f, $buffer, strlen($buffer));
				//fclose($f);
				//unset($buffer);

				//read CSV from file
				$fields_separator = ",";

				$f = @fopen($this->datafeed_cats_file, "r");
				if ($f)
				{
					//read categories data into array and tree
					$saved_categories = explode(",", $settings["plugin_dollar_days_import_categories"]);
					$all_categories = array();
					$tree = new DB_TREE();
					$tree->collection[TREE_ROOT] = (new DOLLAR_DAYS_IMPORT_NODE(TREE_ROOT, -1, 0, false, "Catalog root"));
					
					while (($line = fgetcsv($f, 4096, $fields_separator, '"')))
					{
						//"Name","URL","CategoryID","ParentCatID"
						if (isset($line[2]) && is_numeric($line[2]))
						{
							$category_key = "dollar_days_".$line[2];
							$category_parent = $line[3] == "0" ? "0" : ("dollar_days_".$line[3]);
							$line[0] = str_replace("&amp;", "&", $line[0]);
							$line[2] = $category_key;
							$line[3] = $category_parent;

							if (array_key_exists($line[2], $all_categories))
							{
								//echo "Duplicate Key:".$line[2]."<br/>";
							}
							else
							{
								$line["cid"] = "0";
								$line["import"] = false;
								$all_categories[$line[2]] = $line;
								$tree->addNode(new DOLLAR_DAYS_IMPORT_NODE($line[2], $line[3], 1, in_array($line[2], $saved_categories)?true:false, $line[0]));
							}
						}
					}
					fclose($f);
					//unlink($this->datafeed_cats_file);

					//draw tree
					global $clTableFrame;
					pageNote(array(
						array("class"=>"ic-info", "text"=>"Please select which categories you would like to import into your store.<br/>Make sure you don't have a duplicate category already existing within your store as this process may overwrite existing data.<br/>Also, be sure to check the product ID's of the categories you are importing to insure that you will not be importing duplicates.<br/>You can view an example of the products at <a href=\"http://www.wowpricing.com\" target=\"_blank\">http://www.wowpricing.com</a>", "url"=>"", "visible"=>true)
					));
					echo '<form action="admin.php?p=plugin&plugin_id='.$this->plugin_id.'&step=3" method="POST">';
					
					echo '<div style="padding:5px 50px;">';
						echo '<input type="checkbox" name="all_cats" value="1" '.(in_array("all", $saved_categories)?" checked ":"").' id="all_cats">';
						echo '<label for="all_cats">Import all categories & products</label>';
						echo '<p><input type="submit" value="Continue" default></p>';
					echo '</div>';
					
					echo '<table width="100%" cellpadding="0" cellspacing="0" border="0" class="admin-list-table">';
					echo '<thead><tr><th class="icon"></th><th>Dollar Days Categories</th><th style="width:100px">Import</th></tr></thead>';
					echo '<tbody>';
					$tree->collection[TREE_ROOT]->drawNode(1);
					echo '</tbody>';
					echo '</table>';
					
					
					$_SESSION["admin_dd_all_categories"] = $all_categories;
				}
				break;
			}

			/**
			 * DISPLAY PRICE CHANGE OPTIONS
			 */
			case 3 : 
			{
				$_SESSION["admin_dd_step"] = 3;
				$cat = isset($_POST["cat"])?$_POST["cat"]:array();
				$cat = is_array($cat)?$cat:false;
				$all_cats = isset($_POST["all_cats"])?$_POST["all_cats"]:false;
				$_SESSION["admin_dd_import_all_cattegories"] = $all_cats;
				$_SESSION["admin_dd_selected_categories"] = $cat;

				//here we save EXACTLY what user selected to do not confuse him
				$saved_categories = $all_cats ? "all" : implode(",", $cat);
				if (!DEMO_LOCK)
				{
					$db->query("UPDATE ".DB_PREFIX."settings SET value='".$db->escape($saved_categories)."' WHERE name='plugin_dollar_days_import_categories'");
				}

				if ((count($cat) > 0) || ($all_cats))
				{
					pageNote(array(
						array("class"=>"ic-info", "text"=>"<strong>Global Price Changes</strong><br/>All products selected will be inserted into your database with Dollar Days Pricing.<br/>Depending on your agreement with Dollardays, you may want to increase or decrease this pricing.<br/>This area will allow you to globally increase or decrease pricing."),
						array("class"=>"ic-info", "text"=>"<strong>Click \"Continue\" button to import products right now, OR click \"Save Changes\" button to save your selection for a future usage</strong>", "visible"=>true),
					));
					echo '<form action="admin.php?p=plugin&plugin_id='.$this->plugin_id.'&step=4" method="POST">';
					echo '<input type="hidden" name="pressed_button" value="Save" id="hidden_pressed_button"/>';
					echo '<table cellpadding="0" cellspacing="0" border="0">';
					echo '<tr><td width="50">&nbsp;</td>';
					echo '<td width="140"><select name="price_option" class="short"><option value="inc">Increase</option><option value="dec" '.($settings["plugin_dollar_days_import_price_option"] == "dec"?'selected="selected"':'').'>Decrease</select></td>';
					echo '<td valign="top" style="padding-top:4px;" width="75">pricing by</td>';
					echo '<td width="140"><input type="text" name="price_value" value="'.$settings["plugin_dollar_days_import_price_value"].'" class="short"></td>';
					echo '<td valign="top" style="padding-top:4px;">%</td></tr>';
					echo '</table><br/><br/>';
					
					echo '<div style="padding:10px 50px;">';
						echo '<strong>Clicking continue will start the importing process</strong><br/><br/>';
						echo '<input type="submit" onclick="document.getElementById(\'hidden_pressed_button\').value=\'Save\'" value="Save" value=\"Save changes\" default="default"/> &nbsp; ';
						echo '<input type="submit" onclick="document.getElementById(\'hidden_pressed_button\').value=\'continue\'" value="Continue"/>';
					echo '</div>';
					echo '</form>';
				}
				else
				{
					pageNote(array(
						array("class"=>"ic-cross", "text"=>"<b>There no categories selected</b><br>Please select at least one category to import")
					));
					echo '<div style="padding-left:25px;padding-top:0px;"><input onClick="document.location=\'admin.php?p=plugin&plugin_id='.$this->plugin_id.'&step=2\'" type="button" value="Back"/></div>';
				}

				break;
			}

			/**
			 * SAVE SETTINGS / START DOWNLOAD PRODUCTS FEED FILES
			 */
			case 4 :
			{
				$_SESSION["admin_dd_step"] = 4;

				//save price change settings
				$price_option = isset($_REQUEST["price_option"])?$_REQUEST["price_option"]:"decrease";
				$price_value = isset($_REQUEST["price_value"])?floatval($_REQUEST["price_value"]):"0.00";
				if (!DEMO_LOCK)
				{
					$db->query("UPDATE ".DB_PREFIX."settings SET value='".$db->escape($price_option)."' WHERE name='plugin_dollar_days_import_price_option'");
					$db->query("UPDATE ".DB_PREFIX."settings SET value='".$db->escape($price_value)."' WHERE name='plugin_dollar_days_import_price_value'");
				}

				//download products files
				if ($_POST["pressed_button"] == "continue")
				{
					$this->log("Start download products feed file");

					echo '<script type="text/javascript">'."\n";
					echo 'timerLogoutActive = false;'."\n";
					echo '</script>'."\n";

					echo '<div id="div_iframe">';

					pageNote(array(
						array("class"=>"ic-info", "text"=>"<b>Downloading Products Datafeed...</b>", "url"=>"", "visible"=>true)
					));

					echo '<br/><div style="padding-left:48px;padding-top:10px;clear:both;"><iframe style="width:480px;height:130px;border:0px;border:none;" frameborder="0" border="0" src="admin.php?p=plugin&plugin_id='.$this->plugin_id.'&step=5&init=true&chart=true"></iframe></div>';
					echo '<div id="div_message" style="display:none;clear:both;">';

					pageNote(array(
						array("class"=>"ic-info", "text"=>"Downloading of Products and Categories completed<br>&nbsp;", "url"=>"", "visible"=>true),
						array("class"=>"ic-info", "text"=>"<b>Please note: The importing of products can take up to five minutes based upon the number of products you have selected.</b><br>&nbsp;", "url"=>"", "visible"=>true)

					));

					//echo '<form name="form_next" method="POST" id="form_next" action="admin.php?p=plugin&plugin_id='.$this->plugin_id.'&step=6"></form>';
					echo '</div>';

				}
				else
				{
					//just display message and we are done
					unset($_SESSION["admin_dd_all_categories"]);
					unset($_SESSION["admin_dd_import_all_categories"]);
					unset($_SESSION["admin_dd_selected_categories"]);
					pageNote(array(
						array("class"=>"ic-info", "text"=>"<b>Your Dollar Days settings are now saved</b><br>&nbsp;", "url"=>"", "visible"=>true)
					));
					echo '<div style="padding-left:30px;padding-top:0px;"><input onClick="document.location=\'admin.php?p=plugin&plugin_id='.$this->plugin_id.'&step=1\'" type="button" value="Back"/></div>';
				}
				break;
			}

			/**
			 * DOWNLOAD PRODUCTS
			 */
			case 5 :
			{
				$_SESSION["admin_dd_step"] = 5;

				echo '<html><body style="padding:0px;margin:0px;font-family:Arial;font-size:12px;">';

				if (isset($_REQUEST["init"]))
				{
					echo '<div style="padding:4px;color:black;">Downloading products datafeed file. This may take several minutes. Please wait...</div>';
					echo '<script language="JavaScript">';
					echo 'var to = setTimeout(\'document.location = "admin.php?p=plugin&plugin_id='.$this->plugin_id.'&step=5&chart=true";\', 500)';
					echo '</script>';
				}
				else
				{
					if (!safe_mode()) set_time_limit(100000);
					echo '<div style="padding:4px;color:black;">Products datafeed file downloade completed. Updating categories, please wait...</div>';
					$buffer = (!DEMO_LOCK) ? $this->getCURL($this->datafeed_prod_url, $this->datafeed_prod_file) : 'DEMO';
					
					if ($buffer)
					{
						//$f = fopen($this->datafeed_prod_file, "w");
						//fputs($f, $buffer, strlen($buffer));
						//fclose($f);
						$this->log("Downloading of products feed file done");
						//$_SESSION["admin_dd_counter"] = 0;
						echo '<script language="JavaScript">';
						echo 'var to = setTimeout(\'document.location = "admin.php?p=plugin&plugin_id='.$this->plugin_id.'&step=6&chart=true";\', 500)';
						echo '</script>';
						die();
					}
				}
				echo '</html></body>';
				break;
			}

			/**
			 * IMPORT / RESTORE / UPDATE dollar days CATEGORIES
			 */
			case 6 : 
			{
				if (!safe_mode()) set_time_limit(100000);
				$_SESSION["admin_dd_step"] = 6;

				echo '<html><body style="padding:0px;margin:0px;font-family:Arial;font-size:12px;">';

				//restore data from session
				$all_categories = $_SESSION["admin_dd_all_categories"]; // categories from datafeed file
				$import_all_categories = $_SESSION["admin_dd_import_all_cattegories"]; // is there was checked Import All Categories checkbox?
				$selected_categories = $_SESSION["admin_dd_selected_categories"]; // categories selected for import

				//select current dollar days categories from database
				$current_categories = array();
				$db->query("SELECT cid, key_name FROM ".DB_PREFIX."catalog WHERE key_name LIKE 'dollar_days_%' ORDER BY level, priority");
				while (($category = $db->moveNext()) != false)
				{
					$current_categories[$category["key_name"]] = $category["cid"];
				}

				//build categories tree
				$this->log("Build categories tree");
				foreach ($all_categories as $category_key=>$category_data)
				{
					//get parent
					$category_parent_key = $category_data[3];

					//set initial properties
					$all_categories[$category_key]["import_products"] = false;
					$all_categories[$category_key]["import_category"] = false;
					$all_categories[$category_key]["level"] = 1;

					if (array_key_exists($category_key, $current_categories))
					{
						$all_categories[$category_key]["cid"] = $current_categories[$category_key];
					}
					else
					{
						$all_categories[$category_key]["cid"] = 0;
					}

					//set children - innitially there are no children
					$all_categories[$category_key]["children"] = array();

					//if parent exists, add current category to its parent and update category level
					if (array_key_exists($category_parent_key, $all_categories))
					{
						$all_categories[$category_parent_key]["children"][$category_key] = &$all_categories[$category_key];
						$all_categories[$category_key]["level"] = $all_categories[$category_parent_key]["level"] + 1;

						if($all_categories[$category_parent_key]["import_products"]){
							$all_categories[$category_key]["import_products"] = true;
							$all_categories[$category_key]["import_category"] = true;
						}
					}

					//check is this category selected for import by user
					if ($import_all_categories || in_array($category_key, $selected_categories))
					{
						$all_categories[$category_key]["import_products"] = true;
						$all_categories[$category_key]["import_category"] = true;

						//set parent category to be imported too
						$_category_parent_key = $category_parent_key;
						if (!$import_all_categories)
						{
							while (array_key_exists($_category_parent_key, $all_categories))
							{
								$all_categories[$_category_parent_key]["import_category"] = true;
								$_category_parent_key = $all_categories[$_category_parent_key][3];
							}
						}
					}
				}
				$this->log("Build categories tree done");

				//update categories in database
				$categories_cid = array();
				$categories_added = 0;
				$categories_updated = 0;
				
				foreach ($all_categories as $category_key=>$category_data)
				{
					if ($all_categories[$category_key]["import_category"])
					{
						//mark category to do not delete it
						$current_categories[$category_key] = 0;
						//update or insert category
						$db->reset();
						$db->assignStr("parent", $category_data["level"] > 1 ? $all_categories[$category_data[3]]["cid"]:0);
						$db->assignStr("level", $category_data["level"]);
						$db->assignStr("name", $category_data[0]);
						$db->assignStr("is_visible", "Yes");
						if ($category_data["cid"] == 0)
						{
							//add new category
							if (!DEMO_LOCK)
							{
								$db->assignStr("key_name", $category_key);
								$all_categories[$category_key]["cid"] = $db->insert(DB_PREFIX."catalog");
								$__cid = $categories_cid[$category_key] = $all_categories[$category_key]["cid"];
							}
							else
							{
								$__cid = 0;
							}
							if (!DEMO_LOCK) $categories_added++;
						}
						else
						{
							//update category
							if (!DEMO_LOCK)
							{
								$db->update(DB_PREFIX."catalog", "WHERE cid='".$category_data["cid"]."'");
								$__cid = $categories_cid[$category_key] = $category_data["cid"];
							}
							else
							{
								$__cid = 0;
							}
							$categories_updated++;
						}
					}
				}
				$this->log("Categories added: ".$categories_added);
				$this->log("Categories updated: ".$categories_updated);

				unset($all_categories);

				//delete old categories
				$where = "";
				$categories_deleted = 0;
				foreach ($current_categories as $category_key=>$cid)
				{
					if ($cid > 0)
					{
						$where .= ($where == ""?"":" OR ")." (cid='".$cid."') ";
						$categories_deleted++;
					}
				}
				$this->log("Categories deleted: ".$categories_deleted);

				if ($where != "" && !DEMO_LOCK)
				{
					$db->query("DELETE FROM ".DB_PREFIX."catalog WHERE ".$where);
					$db->query("DELETE FROM ".DB_PREFIX."products_categories WHERE ".$where);
				}

				//echo "Added cats:".$categories_added."<br/>";
				//echo "Updated cats:".$categories_updated."<br/>";
				//echo "Deleted cats:".$categories_deleted."<br/>";

				unset($_SESSION["admin_dd_all_categories"]);
				unset($_SESSION["admin_dd_import_all_cattegories"]);
				unset($_SESSION["admin_dd_selected_categories"]);
				$_SESSION["admin_dd_categories_cid"] = $categories_cid;
				$_SESSION["admin_dd_categories_added"] = $categories_added;
				$_SESSION["admin_dd_categories_updated"] = $categories_updated;
				$_SESSION["admin_dd_categories_deleted"] = $categories_deleted;

				echo '<div style="padding:4px;color:black;">Categories updated. Please Wait...</div>';
				echo '<script language="JavaScript">';
				echo 'var to = setTimeout(\'document.location = "admin.php?p=plugin&plugin_id='.$this->plugin_id.'&step=7&chart=true&init=true";\', 500)';
				echo '</script>';
				echo '</html></body>';
				break;
			}//end of importing categories

			/**
			 * IMPORT PRODUCTS
			 */
			case 7 : 
			{
				$_SESSION["admin_dd_step"] = 7;

				echo '<html><body style="padding:0px;margin:0px;font-family:Arial;font-size:12px;">';

				if (isset($_REQUEST["init"]))
				{
					$this->log("Starting to update products");
					$db->query("SELECT pid, product_id FROM ".DB_PREFIX."products WHERE is_dollar_days='Yes'");
					$current_products = array();

					while (($product = $db->moveNext()) != false)
					{
						$current_products[$product["product_id"]] = $product["pid"];
					}
					$_SESSION["admin_dd_current_products"] = $current_products;
					$_SESSION["admin_dd_seek"] = 0;

					//init counters
					$_SESSION["admin_dd_products_added"] = 0;
					$_SESSION["admin_dd_products_updated"] = 0;
					$_SESSION["admin_dd_products_removed"] = 0;

					echo '<div style="padding:4px;color:black;">Starting import products. Please wait...</div>';
					echo '<script language="JavaScript">';
					echo 'var to = setTimeout(\'document.location = "admin.php?p=plugin&plugin_id='.$this->plugin_id.'&step=7&chart=true";\', 500)';
					echo '</script>';
				}
				else
				{
					if (!safe_mode()) set_time_limit(100000);

					//prepare step
					$seek = $_SESSION["admin_dd_seek"];
					$current_products = $_SESSION["admin_dd_current_products"];
					$categories_cid = $_SESSION["admin_dd_categories_cid"];
					$price_option = $settings["plugin_dollar_days_import_price_option"];
					$price_value = $settings["plugin_dollar_days_import_price_value"];
					$fsize = filesize($this->datafeed_prod_file);

					$products_added = $_SESSION["admin_dd_products_added"];
					$products_updated = $_SESSION["admin_dd_products_updated"];
					$products_removed = $_SESSION["admin_dd_products_removed"];

					//open datafeed file
					$f = fopen($this->datafeed_prod_file, "r");
					if ($f)
					{
						fseek($f, $seek);
						$read_lines = 0;
						while ($line = fgetcsv($f, 4096, ","))
						{
							if (!DEMO_LOCK && floatval($line[2]) && array_key_exists("dollar_days_". $line[3], $categories_cid))
							{
								//get category id
								$cid = $categories_cid["dollar_days_". $line[3]];
								//get price
								if ($price_option == "inc")
								{
									$product_price = round($line[2] + $line[2] / 100 * $price_value, 2);
								}
								else
								{
									$product_price = round($line[2] - $line[2] / 100 * $price_value, 2);
								}

								//process product
								$db->reset();
								$db->assignStr("cid", $cid);
								$db->assignStr("product_id", $line[0]);
								$db->assignStr("title", $line[1]);
								$db->assignStr("price", $product_price);
								$db->assignStr("case_pack", $line[4]);
								$db->assignStr("description", $line[5]);
								
								if (substr($line[6], 0, 4) === 'http' || substr($line[6], 0, 2) === '//' || $line[6] == '')
								{
									$db->assignStr("image_url", $line[6]);	
								}
								else
								{
									$db->assignStr("image_url", 'https://www.dollardays.com/'.$line[6]);	
								}

								if (array_key_exists($line[0], $current_products))
								{
									//product exists, update it
									$pid = $current_products[$line[0]];
									$current_products[$line[0]] = 0;
									if (!DEMO_LOCK)
									{
										$db->update(DB_PREFIX."products", "WHERE pid='".$pid."'");
										$db->query("DELETE FROM ".DB_PREFIX."products_categories WHERE pid='".$pid."'");
									}
									$products_updated++;
								}
								else
								{
									//product does not exist, add it
									if (!DEMO_LOCK)
									{
										$db->assignStr("is_dollar_days", "Yes");
										$db->assign("added", "NOW()");
										$pid = $db->insert(DB_PREFIX."products");
									}
									$products_added++;
								}
								if (!DEMO_LOCK)
								{
									$db->query("INSERT INTO ".DB_PREFIX."products_categories(pid,cid,is_primary) VALUES ('".$pid."','".$cid."',1)");
								}
							}
							$read_lines++;
							if ($read_lines > 100)
							{
								break;
							}
						}
						$_SESSION["admin_dd_current_products"] = $current_products;
						$_SESSION["admin_dd_seek"] = ftell($f);
						fclose($f);

						$_SESSION["admin_dd_products_added"] = $products_added;
						$_SESSION["admin_dd_products_updated"] = $products_updated;

						if ($line)
						{
							$this->log("Importing products: ".number_format($_SESSION["admin_dd_seek"] / ($fsize/100), 2, ".", "")."% done (New: ".$products_added."; Updated:".$products_updated.")");
							echo '<div style="padding:4px;color:black;">Importing products - '.(number_format($_SESSION["admin_dd_seek"] / ($fsize/100), 2, ".", "")).'% done. Please wait...</div>';
							echo '<script language="JavaScript">';
							echo 'var to = setTimeout(\'document.location = "admin.php?p=plugin&plugin_id='.$this->plugin_id.'&step=7&chart=true";\', 250)';
							echo '</script>';
						}
						else
						{
							//delete old products
							$this->log("Remoding old products");
							$where = "";
							foreach ($current_products as $product_id=>$pid)
							{
								if ($pid > 0)
								{
									$where .= ($where == ""?"":" OR ")." (pid='".$pid."') ";
									$products_removed++;
									if ($products_removed % 100 == 0)
									{
										if (!DEMO_LOCK)
										{
											$db->query("UPDATE ".DB_PREFIX."orders_content SET product_removed='Yes' WHERE ".$where);
											$db->query("DELETE FROM ".DB_PREFIX."products_attributes WHERE ".$where);
											$db->query("DELETE FROM ".DB_PREFIX."products_categories WHERE ".$where);
											$db->query("DELETE FROM ".DB_PREFIX."products_shipping_price WHERE ".$where);
											$db->query("DELETE FROM ".DB_PREFIX."products WHERE ".$where);
										}
										$where = "";
									}
								}
							}
							if ($where != "")
							{
								if (!DEMO_LOCK)
								{
									$db->query("UPDATE ".DB_PREFIX."orders_content SET product_removed='Yes' WHERE ".$where);
									$db->query("DELETE FROM ".DB_PREFIX."products_attributes WHERE ".$where);
									$db->query("DELETE FROM ".DB_PREFIX."products_categories WHERE ".$where);
									$db->query("DELETE FROM ".DB_PREFIX."products_shipping_price WHERE ".$where);
									$db->query("DELETE FROM ".DB_PREFIX."products WHERE ".$where);
								}
							}

							$this->log("Products removed: ".$products_removed);

							unset($current_products);
							unset($categories_cid);

							//update
							if (!DEMO_LOCK)
							{
								$db->query("UPDATE ".DB_PREFIX."products SET meta_title=title, image_location='Web' WHERE is_dollar_days='Yes'");
								$db->query("UPDATE ".DB_PREFIX."settings SET value='".date("Y/m/d H:i:s")."' WHERE name='plugin_dollar_days_import_last_update'");
							}

							unset($_SESSION["admin_dd_current_products"]);
							unset($_SESSION["admin_dd_categories_cid"]);
							unset($_SESSION["admin_dd_seek"]);

							$_SESSION["admin_dd_products_removed"] = $products_removed;
							echo '<div style="padding:4px;color:black;">Importing products completed</div>';
							echo '<script language="JavaScript">';
							echo 'var to = setTimeout(\'parent.document.location = "admin.php?p=plugin&plugin_id='.$this->plugin_id.'&step=8";\', 250)';
							echo '</script>';
						}
					}
				}
				echo '</html></body>';
				break;
			}

			/**
			 * DONE
			 */
			case 8:
			{
				if (!DEMO_LOCK)
				{
					updateSeoURLs($db, $settings, true, true, false, true);
				}
				$this->log("UPDATE DONE. Results are:\r\n".
					"New products added: ".$_SESSION["admin_dd_products_added"]."\r\n".
					"Products updated: ".$_SESSION["admin_dd_products_updated"]."\r\n".
					"Products removed: ".$_SESSION["admin_dd_products_removed"]."\r\n".
					"New categories created: ".$_SESSION["admin_dd_categories_added"]."\r\n".
					"Categories updated: ".$_SESSION["admin_dd_categories_updated"]."\r\n".
					"Categories deleted: ".$_SESSION["admin_dd_categories_deleted"]."\r\n"."\r\n"."\r\n"
				);
				pageNote(array(
					array("class"=>"ic-info", "text"=>"<b>Categories & Products have been successfully imported from<br/>DollarDays.com datafeed server</b><br>&nbsp;"),
					array("class"=>"ic-info",
						"text"=>"New products added: ".$_SESSION["admin_dd_products_added"].
						"<br>Products updated: ".$_SESSION["admin_dd_products_updated"].
						"<br>Products removed: ".$_SESSION["admin_dd_products_removed"].
						"<br/>New categories created: ".$_SESSION["admin_dd_categories_added"].
						"<br/>Categories updated: ".$_SESSION["admin_dd_categories_updated"].
						"<br/>Categories deleted: ".$_SESSION["admin_dd_categories_deleted"]
					)
				));

				unset($_SESSION["admin_dd_products_added"]);
				unset($_SESSION["admin_dd_products_updated"]);
				unset($_SESSION["admin_dd_products_removed"]);
				unset($_SESSION["admin_dd_categories_added"]);
				unset($_SESSION["admin_dd_categories_updated"]);
				unset($_SESSION["admin_dd_categories_deleted"]);

				unlink($this->datafeed_cats_file);
				unlink($this->datafeed_prod_file);

				break;
			}
		}//end of switch($step)
	}//end of runPlugin procedure
}//end of class


class DOLLAR_DAYS_IMPORT_NODE extends DB_TREE_NODE
{
	function drawNode($level)
	{
		if ($this->id != TREE_ROOT."")
		{
			echo '<tr>';
			echo '<td colspan="2" style="padding-left:'.($level * 18).'px;"><img src="images/icons/a/folder.png"> <label for="dd-node-'.$this->id.'" style="cursor:pointer;">'.htmlspecialchars($this->caption).' ('.$this->id.')</label></td>';
			echo '<td><input id="dd-node-'.$this->id.'" type="checkbox" name="cat['.$this->id.']" id="cat['.$this->id.']" value="'.$this->id.'" '.($this->is_visible?'checked="checked"':'').'></td>';
			echo '</tr>';
		}

		if (count($this->nodes)>0)
		{
			foreach ($this->nodes as $id=>$node)
			{
				$node->drawNode($level + 1);
			}
			
		}
	}
}
