<?php
class plugin_openapi extends cartPlugin {
	//protected
	var $plugin_id = "openapi";
	var $plugin_group = "export";
	var $plugin_caption = "Shopping Cart Unified API";
	var $plugin_description = "API allows you to export customer and order data using common methods including SOAP, XML, RSS, and JSON.  Recommended for advanced users only.";

 	//run API plugin
	function runPlugin(&$db, &$settings, &$request)
	{
		parent::runPlugin($db, $settings, $request);
		
		pageHeader("Shopping Cart Unified API", '', 'icb-protocol1');
		
		if (!AccessManager::checkAccess('FACEBOOK_STORE'))
		{
			AccessManager::notAllowed();
			return;
		}

		//check settings
		if(!array_key_exists("plugin_openapi_active", $settings)){
			$db->reset();
			$db->assignStr("visible", "No");
			$db->assignStr("input_type", "hidden");
			$db->assignStr("group_name", "plugin_api");
			$db->assignStr("name", "plugin_openapi_active");
			$db->assignStr("value", "NO");
			$db->assignStr("options", "YES,NO");
			$db->insert(DB_PREFIX."settings");
			$settings["plugin_openapi_active"] = "NO";
		}
		if(!array_key_exists("plugin_openapi_username", $settings)){
			$db->reset();
			$db->assignStr("visible", "No");
			$db->assignStr("input_type", "hidden");
			$db->assignStr("group_name", "plugin_api");
			$db->assignStr("name", "plugin_openapi_username");
			$db->assignStr("value", "");
			$db->insert(DB_PREFIX."settings");
			$settings["plugin_openapi_username"] = "";
		}

		if (!array_key_exists("plugin_openapi_password", $settings))
		{
			$db->reset();
			$db->assignStr("visible", "No");
			$db->assignStr("input_type", "hidden");
			$db->assignStr("group_name", "plugin_api");
			$db->assignStr("name", "plugin_openapi_password");
			$db->insert(DB_PREFIX."settings");
			$db->assignStr("value", "");
			$settings["plugin_openapi_password"] = "";
		}

		if(!array_key_exists("plugin_openapi_token", $settings)){
			$db->reset();
			$db->assignStr("visible", "No");
			$db->assignStr("input_type", "hidden");
			$db->assignStr("group_name", "plugin_api");
			$db->assignStr("name", "plugin_openapi_token");
			$db->assignStr("value", "");
			$db->insert(DB_PREFIX."settings");
			$settings["plugin_openapi_token"] = "";
		}

		//save settings
		if (!DEMO_LOCK && isset($_REQUEST["action"]) && $_REQUEST["action"] == "update_settings")
		{
			$db->reset();
			$db->assignStr("value", $_REQUEST["plugin_openapi_active"]);
			$db->update(DB_PREFIX."settings", "WHERE name='plugin_openapi_active'");
			$settings["plugin_openapi_active"] = $_REQUEST["plugin_openapi_active"];
			
			$db->reset();
			$db->assignStr("value", $_REQUEST["plugin_openapi_username"]);
			$db->update(DB_PREFIX."settings", "WHERE name='plugin_openapi_username'");
			$settings["plugin_openapi_username"] = $_REQUEST["plugin_openapi_username"];
			
			if (isset($_REQUEST["plugin_openapi_password"]) && $_REQUEST["plugin_openapi_password"] != '')
			{
				$db->reset();
				$db->assignStr("value", getPasswordHash($_REQUEST["plugin_openapi_password"]));
				$db->update(DB_PREFIX."settings", "WHERE name='plugin_openapi_password'");
				$settings["plugin_openapi_password"] = getPasswordHash($_REQUEST["plugin_openapi_password"]);
			}
			
			$db->reset();
			$db->assignStr("value", $_REQUEST["plugin_openapi_token"]);
			$db->update(DB_PREFIX."settings", "WHERE name='plugin_openapi_token'");
			$settings["plugin_openapi_token"] = $_REQUEST["plugin_openapi_token"];
		}


		//form HEADER
		echo '<script type="text/javascript" src="content/admin/plugins/openapi/openapi.js"></script>';
		pageNote(array(
			array("class"=>"ic-info", "text"=>"API allows you to export customer and order data using coming methods including SOAP, XML, RSS and JSON. " . (IS_LABEL ? '' : "For information regarding the API, please direct your browser to <a href=\"http://api.".getp().".com/docs/\">http://api.".getp().".com/docs/</a>"), "visible"=>true),
			array("class"=>"ic-info", "text"=>"The connection URL to your API is ".$settings["GlobalHttpsUrl"]."/content/admin/plugins/openapi/index.php", "url"=>"", "visible"=>true),
			array("class"=>"ic-info", "text"=>"Please update the access information for the API below", "url"=>"", "visible"=>true),
			array("class"=>"ic-info", "text"=>"<b>Your information has been updated</b>", "url"=>"", "visible"=>isset($_REQUEST["action"]) && $_REQUEST["action"] == "update_settings")
		));

		//form table
		echo '<form name="frmPlugin" action="admin.php?p=plugin&plugin_id='.$this->plugin_id.'" method="POST" style="padding:0px;spacing:0px;margin:0px;">';
		echo '<input type="hidden" name="action" value="update_settings">';
		echo '<div class="admin-tabs admin-tabs-framed" id="tabs-api">';
		pageTabs(array(array("title"=>"Plugin Settings", "id"=>"main")), "tabs-api", true);
		echo '<div id="admin-tab-sheet-main" class="admin-tab-sheet hidden"><div class="admin-tab-sheet-wrap">';
		echo '<div class="admin-form-header first"><span class="icon ic-icon"></span>Cart API Settings</div>';
		echo '<table width="100%" cellpadding="0" cellspacing="0" border="0">';
		echo '<tr valign="top"><td class="formItemCaption">Enable Cart API?</td>';
		echo '<td class="formItemControl">';
		echo '<select name="plugin_openapi_active" style="width:123px;">';
		echo '<option value="YES">Yes</option>';
		echo '<option value="NO" '.($settings["plugin_openapi_active"]=="NO"?"selected":"").'>No</option>';
		echo '</select><div class="formItemComment">Selecting NO will disable export. Authentification failed message will be returned.</div>';
		echo '</td></tr>';

		echo '<tr valign="top"><td class="formItemCaption">Username</td>';
		echo '<td class="formItemControl">';
		echo '<input type="text" name="plugin_openapi_username" value="'.htmlspecialchars($settings["plugin_openapi_username"]).'">';
		echo '</td></tr>';

		echo '<tr valign="top"><td class="formItemCaption">Password</td>';
		echo '<td class="formItemControl">';
		echo '<input type="password" name="plugin_openapi_password" value="">';
		echo '</td></tr>';

		echo '<tr valign="top"><td class="formItemCaption">Security Token</td>';
		echo '<td class="formItemControl">';
		echo '<input type="text" id="plugin_openapi_token" name="plugin_openapi_token" value="'.htmlspecialchars($settings["plugin_openapi_token"]).'">';
		echo '&nbsp;<input type="button" id="button-token" value="Generate"/>';
		echo '</td></tr>';

		echo '</table>';
		echo '</div></div></div>';
		echo '<div class="admin-form-buttons">';
		echo '<input type="submit" value="Save changes"/>';
		echo '<input type="reset" value="Reset form"/>';
		echo '</div>';
		echo '</form>';

	}//end of runPlugin procedure
}//end of class
?>