<?php

/**
* 
*/
class Controller_ThemeManager extends Admin_ViewController
{
	public function indexAction()
	{
		$lastActiveTab = (isset($_GET['last_active_tab'])) ? $_GET['last_active_tab'] : false;
		
		$this->view->assign('tabs', pageTabs(array(
				array("title"=>"All Themes", "id"=>"themes",),
				array("title"=>"Manage Themes", "id"=>"manage",),
			), "tabs-design", $lastActiveTab, true)
		);
		
		$this->view->assign('themeSelection', $this->displaySelectionAction());
		$this->view->assign('manageThemes', $this->displayThemeManager());
		
		return $this->view->fetch('pages/thememanager/layout');
	}
	
	public function displaySelectionAction()
	{
		static $initialized = null;
		$initialized = is_null($initialized);
		
		
		$themesFolderPath = 'content/skins';
		$themes = DataAccess_Repository::get('SiteThemeRepository')->getInstalledThemes($themesFolderPath);
		
		$themeCollection = array();
		$existingThemes = array();
		$selectedTheme = null;
		foreach ($themes as $themeId => $theme)
		{
			$pathToImages = $themesFolderPath.'/'.$themeId.'/';
			
			$themeHelper = array(
				'name' => htmlentities($theme->getName()),
				'title' => $theme->getTitle(),
				'author' => $theme->getAuthor(),
				'thumbnail_image' => ($theme->getThumbnailImage()) ? $pathToImages.escapeFileName($theme->getThumbnailImage()) : null,
				'preview_image' => ($theme->getFullsizeImage()) ? $pathToImages.escapeFileName($theme->getFullsizeImage()) : null,
				'tag' => strtolower($theme->getName()),
				'images' => $theme->getImages(),
				'rel_group_name' => (count($theme->getImages())) ? 'rel="group_'.$theme->getName().'"' : ''
			);
			
			$existingThemes[] = $theme->getName();
			
			if ($theme->getName() == $this->settings['DesignSkinName'])
			{
				$selectedTheme = $themeHelper;
			}
			else
			{
				$themeCollection[$theme->getName()] = $themeHelper;
			}
		}
		
		$this->view->assign('themeCollection', $themeCollection);
		$this->view->assign('selectedTheme', $selectedTheme);
		$this->view->assign('hasBeenInitialized', $initialized);
		$this->view->assign('existingThemes', json_encode($existingThemes));
		$this->view->assign('themesInstallable', is_writable($themesFolderPath) ? 'true' : 'false');
		
		return $this->view->fetch('pages/thememanager/selection');
	}
	
	public function displayThemeManager()
	{
		$themes = DataAccess_Repository::get('SiteThemeRepository')->getInstalledThemes();
		$this->view->assign('themes', $themes);
		$this->view->assign('activeTheme', $this->settings['DesignSkinName']);
		return $this->view->fetch('pages/thememanager/manager');
	}
	
	
	/**
	 * Sets the site theme
	 *
	 * @param string $themeName 
	 * @param boolean $restoreLayout Whether the current site theme layout should be restored
	 * @return boolean TRUE if theme successfully set, FALSE otherwise
	 */
	public function activateThemeAction($themeName, $restoreLayout = false)
	{
		// Make sure the skin exists
		if (is_dir('content/skins/'.escapeFileName($themeName)))
		{
			// read in the skin xml file
			$template_xml = simplexml_load_string(file_get_contents('content/skins/'.escapeFileName($themeName).'/skin.xml'), 'SimpleXMLElement', LIBXML_NOCDATA);

			// Update the current selected skin in settings
			$this->db->reset();
			$this->db->assignStr('value', $themeName);
			$this->db->update(DB_PREFIX.'settings', "WHERE name='DesignSkinName'");
			$this->settings['DesignSkinName'] = $themeName;

			// Set the Site Layout Width to the default as defined in skin xml
			$layoutSiteWidth = $template_xml->xpath("/skin/settings/option[@name='LayoutSiteWidth']");
			$layoutSiteWidthValue = (count($layoutSiteWidth) && $layoutSiteWidth !== false)
			? $layoutSiteWidth[0]->attributes()->value
			: $this->settings['LayoutSiteWidthDefault'];
				
			$this->db->reset();
			$this->db->assignStr('value', $layoutSiteWidthValue);
			$this->db->update(DB_PREFIX.'settings', "WHERE name='LayoutSiteWidth' LIMIT 1");
			$this->settings['LayoutSiteWidth'] = $layoutSiteWidthValue;

			// check if we need to override the current layout settings with theme defaults,
			// otherwise, do nothing
			if ((bool) $restoreLayout)
			{
				$designMode = new Ddm_Design_Mode($this->settings);
				$designMode->restoreLayout();
				$designMode->restoreSkin();
			}
			
			return true;
		}
		
		return false;
	}
	
	public function deleteThemeAction($theme, $dryrun = false)
	{
		if (!is_array($theme)) $theme = array($theme);
		
		$errors = array();
		foreach ($theme as $themeName)
		{
			if (trim($themeName) != '')
			{
				File_Utility::removeDirectory('content/skins/'.escapeFileName($themeName), $errors, $dryrun);
			}
		}
		
		return array(
			'status' => (empty($errors) ? 'success' : 'error'),
			'errors' => $errors
		);
	}
}
