<?php
/**
 * Chart repository class
 */
class DataAccess_ChartRepository extends DataAccess_Repository
{
	protected $chartDataRepository = null;
	
	public function __construct()
	{
		$this->chartDataRepository = DataAccess_Repository::get('ChartDataRepository');
	}
	
	public function getUsageHostsByMonth($month, $year, $cache = true)
	{
		$data = $this->chartDataRepository->getUsageHostsByMonth($month, $year, $cache);
		
		$chart = new Chart();
		$series = new ChartSeries();
		
		$days_in_month = (int) date('t', mktime(0,0,0,$month,1,$year));
		for ($i = 1; $i <= $days_in_month; $i++)
		{
			$date = strtotime("{$year}-{$month}-{$i} 00:00:00 UTC");
			$series->addData($date, 0, date('M d', mktime(0,0,0,$month,$i,$year)));
		}
		
		$series->setTitle('Monthly Statistics for '.date("F Y", mktime(0,0,0, $month, 1, $year)).' (unique hosts)');
		$series->setXAxisLabel('Date ('.date("F Y", mktime(0,0,0, $month, 1, $year)).')');
		$series->setYAxisLabel('Hosts');
		$series->setXAxisMode(ChartSeries::AXIS_MODE_TIME);
		
		if ($data)
		{
			foreach ($data as $chart_data)
			{
				
				list($data_month,$data_day,$data_year) = preg_split('/:/', date('m:d:Y', strtotime($chart_data['visit_date'])));
				$data_date = strtotime("{$data_year}-{$data_month}-{$data_day} 00:00:00 UTC");
				$series->addData($data_date, intval($chart_data['day_hosts']), date('M d', mktime(0,0,0,$data_month,$data_day,$data_year)));
			}
		}
		$chart->addSeries($series);
		return $chart;
	}
	
	public function getUsageHitsByMonth($month, $year, $cache = true)
	{
		$data = $this->chartDataRepository->getUsageHitsByMonth($month, $year, $cache);
		
		$chart = new Chart();
		$series = new ChartSeries();
		
		$days_in_month = (int) date('t', mktime(0,0,0,$month,1,$year));
		for ($i = 1; $i <= $days_in_month; $i++)
		{
			$date = strtotime("{$year}-{$month}-{$i} 00:00:00 UTC");
			$series->addData($date, 0, date('M d', mktime(0,0,0,$month,$i,$year)));
		}
		
		$series->setTitle('Monthly Statistics for '.date("F Y", mktime(0,0,0, $month, 1, $year)).' (total pages viewed)');
		$series->setXAxisLabel('Date ('.date("F Y", mktime(0,0,0, $month, 1, $year)).')');
		$series->setYAxisLabel('Pages');
		$series->setXAxisMode(ChartSeries::AXIS_MODE_TIME);
		
		if ($data)
		{
			foreach ($data as $chart_data)
			{
				list($data_month,$data_day,$data_year) = preg_split('/:/', date('m:d:Y', strtotime($chart_data['visit_date'])));
				$data_date = strtotime("{$data_year}-{$data_month}-{$data_day} 00:00:00 UTC");
				$series->addData($data_date, intval($chart_data['day_hits']), date('M d', mktime(0,0,0,$data_month,$data_day,$data_year)));
			}
		}
		$chart->addSeries($series);
		return $chart;
	}
	
	public function getSalesByMonth($month, $year, $cache = true)
	{
		$data = $this->chartDataRepository->getSalesByMonth($month, $year, $cache);
		
		$currencies = new Currencies(self::$db);
		$default_currency = $currencies->getDefaultCurrency();
		
		$tooltipFormat = $default_currency['symbol_left'].'%s'.$default_currency['symbol_right'];
		
		$chart = new Chart();
		$series = new ChartSeries();
		
		$days_in_month = (int) date('t', mktime(0,0,0,$month,1,$year));
		for ($i = 1; $i <= $days_in_month; $i++)
		{
			$date = strtotime("{$year}-{$month}-{$i} 00:00:00 UTC");
			$series->addData($date, 0, date('M d', mktime(0,0,0,$month,$i,$year)).'<br />'.sprintf($tooltipFormat, '0.00'));
		}
		
		$series->setTitle('Monthly Sales Statistics for '.date("F Y", mktime(0,0,0, $month, 1, $year)).' ('.$default_currency['title'].')');
		$series->setXAxisLabel('Date ('.date("F Y", mktime(0,0,0, $month, 1, $year)).')');
		$series->setYAxisLabel('Sales ('.$default_currency['title'].')');
		$series->setYAxisFormat($default_currency['symbol_left'].'%s'.$default_currency['symbol_right']);
		$series->setXAxisMode(ChartSeries::AXIS_MODE_TIME);
		
		if ($data)
		{
			foreach ($data as $chart_data)
			{
				$amount = ($chart_data['total_sum'] > 0)
					? $chart_data['total_sum']
					: 0;
				
				list($data_month,$data_day,$data_year) = preg_split('/:/', date('m:d:Y', strtotime($chart_data['placed_date'])));
				$amount = (float) number_format($amount, 2, '.', '');
				$data_date = strtotime("{$data_year}-{$data_month}-{$data_day} 00:00:00 UTC");
				
				$series->addData($data_date, $amount, date('M d', mktime(0,0,0,$data_month,$data_day,$data_year)).'<br />'.sprintf($tooltipFormat, number_format($amount, 2, '.', ',')));
			}
			
		}
		$chart->addSeries($series);
		return $chart;
	}
}
