<?php
class BongoConnect
{
	protected $partnerKey;

	protected $lastError = null;
	protected $lastErrorDetails = null;

	/**
	 * Class constructor
	 */
	public function __construct($partnerKey)
	{
		$this->partnerKey = $partnerKey;
	}

	/**
	 * Set last error 
	 */
	public function setLastError($lastError)
	{
		$this->lastError = $lastError;
	}

	/**
	 * Get last error
	 */
	public function getLastError()
	{
		return $this->lastError;
	}

	/**
	 * Set last error details
	 */
	public function setLastErrorDetails($lastErrorDetails)
	{
		$this->lastErrorDetails = $lastErrorDetails;
	}

	/**
	 * Get last error details
	 */
	public function getLastErrorDetails()
	{
		return $this->lastErrorDetails;
	}

	/**
	 * Update order tracking information
	 */
	public function orderTrackingUpdate($orderItems, $bongoOrderId, $shippingCarrier, $shippingTrackingNumber)
	{
		$this->setLastError(null);
		$this->setLastErrorDetails(null);

		try
		{
			$client = new SoapClient("https://api.bongous.com/services/v4?wsdl");

			$shippingCarrier = trim(strtoupper($shippingCarrier));

			$shippingCarriers = array(
				"UPS" => 1,
				"FEDEX" => 2,
				"DHL" => 3,
				"USPS" => 4,
				"EMS" => 5
			);

			$shippingCarrierCode = isset($shippingCarriers[$shippingCarrier]) ? $shippingCarriers[$shippingCarrier] : 6; // all others

			$trackList = array();
			foreach ($orderItems as $orderItem)
			{
				$trackList[] = array(
					'productID' => $orderItem["id"],
					'trackingNumber' => $shippingTrackingNumber,
					'quantity' => $orderItem["quantity"],
					'carrier' => $shippingCarrierCode
				);
			}

			if (count($trackList) == 0) return false;

			$request = (object)array(
				'partnerKey' => $this->partnerKey,
				'language' => 'en',
				'orderNumber' => $bongoOrderId,
				'trackList' => $trackList
			);

			if (defined('DEVMODE') && DEVMODE)
			{
				@file_put_contents(
					dirname(dirname(dirname(__FILE__))).'/cache/log/bongo.txt',
					date("Y/m/d-H:i:s")." - REQUEST - ".print_r($request, 1)."\n\n\n", 
					FILE_APPEND
				);
			}

			$response = $client->ConnectOrderTrackingUpdate($request);

			if (defined('DEVMODE') && DEVMODE)
			{
				@file_put_contents(
					dirname(dirname(dirname(__FILE__))).'/cache/log/bongo.txt',
					date("Y/m/d-H:i:s")." - RESPONSE - ".print_r($response, 1)."\n\n\n", 
					FILE_APPEND
				);
			}

			if ($response)
			{
				if (isset($response->error))
				{
					if ($response->error == 0) return true;

					$this->setLastError($response->errorMessage);
					$this->setLastErrorDetails($response->errorMessageDetail);
					return false;
				}
			}
			
			$this->setLastError('Cannot send tracking information to Bongo');
			$this->setLastErrorDetails('Please check your credentials');
			
		}
		catch (Exception $e)
		{
			$this->setLastError($e->getCode());
			$this->setLastErrorDetails($e->getMessage());
		}

		return false;
	}
}
