<?php

/**
* Session management for the admin area
*/
class Admin_SessionHandler
{
	protected static $instance = null;
	
	public static function setSessionInstance($session_instance)
	{
		if (get_class($session_instance) == 'Session' || get_parent_class($session_instance) == 'Session')
			self::$instance = $session_instance;
	}
	
	public static function getSessionInstance()
	{
		if (is_null(self::$instance)) self::$instance = Session::getInstance();
		return self::$instance;
	}
	
	/**
	 * Initializes the session as used on the login page
	 *
	 * @param string $settings 
	 * @return void
	 * @author Sebastian Nievas
	 */
	public static function loginInit($settings)
	{
		$session = self::getSessionInstance();
		self::setSessionSavePath($settings);

		$session->setCookieParams(43200, '/', null, null, true);
		$session->setCacheExpire(1140);

		$session->setName('ShoppingCartSessionAdmin');
		$session->start();
	}
	
	/**
	 * Initalizes the session in the admin area
	 *
	 * @param string $settings 
	 * @param string $session_id 
	 * @return void
	 */
	public static function init($settings, $session_id = null)
	{
		$session = self::getSessionInstance();

		self::setSessionSavePath($settings);

		$session->setCacheLimiter('must-revalidate');
		if (!is_null($session_id)) $session->setId($session_id);
		
		$session->setCookieParams(43200, '/', null, null, true);
		$session->setName('ShoppingCartSessionAdmin');
		$session->start();
	}

	public static function designModeInit($settings)
	{
		self::loginInit($settings);
	}

	/**
	 * Sets the session save path
	 *
	 * @param string $settings 
	 * @return void
	 */
	private static function setSessionSavePath($settings)
	{
		$session = self::getSessionInstance();
		$file_utils = file_utils::getInstance();

		$session_dir = getSessionSavePath();
		if ($file_utils->is_dir($session_dir) && $file_utils->is_writable($session_dir))
		{
			$session->setSavePath($session_dir);
		}
	}

	public static function logout()
	{
		unset($_SESSION["admin_session_id"]);
		unset($_SESSION["admin_auth_id"]);
		unset($_SESSION["admin_auth_time"]);
		unset($_SESSION["admin_design_mode"]);

		$session = self::getSessionInstance();

		$session->destroy('ShoppingCartSessionAdmin', '/', null, null, true);
	}
}
