<?php

require_once('content/vendors/php-mobile-detect/Mobile_Detect.php');

class MobileHelper
{
	protected $settings;
	
	public function __construct($settings)
	{
		$this->settings = $settings;
	}
	
	/**
	 * Detects mobile browser
	 * @return bool 
	 */
	public function browserMobile()
	{
		$mobileDetect = new Mobile_Detect();
		return $mobileDetect->isMobile();
	}
	
	/**
	 * Checks does cookie exist
	 * @return bool 
	 */
	public function cookieSet()
	{
		return isset($_COOKIE['_pcmb']);
	}
	
	/**
	 * Checks is mobile cookie send and is it mobile
	 * @return bool 
	 */
	public function cookieMobile()
	{
		return isset($_COOKIE['_pcmb']) && ($_COOKIE['_pcmb'] == '1');
	}
	
	/**
	 *
	 * @param type $isMobile
	 * @return bool 
	 */
	public function setCookie($isMobile)
	{
		setcookie(
			'_pcmb', 
			$isMobile ? '1' : '0', 
			time()+60*60*24*180, 
			"/", 
			$_SERVER["SERVER_PORT"] == "443" ? $this->settings["GlobalHttpsCookieDomain"] : $this->settings["GlobalHttpCookieDomain"]
		); // 6-month timeout
		return $isMobile;
	}
	
	/**
	 * Checks does session var exist
	 * @return bool 
	 */
	public function sessionSet()
	{
		return isset($_SESSION['_pcmb']);
	}
	
	/**
	 * Checks is mobile session var set and is it mobile
	 * @return bool 
	 */
	public function sessionMobile()
	{
		return isset($_SESSION['_pcmb']) && ($_SESSION['_pcmb'] == true);
	}
	
	/**
	 * Set mobile session variable
	 * @param type $isMobile
	 * @return bool 
	 */
	public function setSession($isMobile)
	{
		$_SESSION['_pcmb'] = $isMobile;
		return $isMobile;
	}

	/**
	 * Return current url with mobile param flipped
	 */
	public function getToggleMobileUrl($currentPage, $isMobile = false)
	{
		$url_parts = parse_url($currentPage);
		
		if (!isset($url_parts['query']))
			$url_parts['query'] = '';

		$params = explode('&', $url_parts['query']);
		$temp = array();
		foreach ($params as $p)
		{
			$tp = urldecode($p);
			if (!($tp == 'mobile=1' || $tp == 'mobile=0' || $tp == ''))
			{
				if (strpos($tp, '=') >= 0)
				{
					$param_parts = explode('=', $tp);
					$var = $param_parts[0];
					unset($param_parts[0]);
					$temp[] = $var.'='.urlencode(implode('=', $param_parts));
				}
				else
				{
					$temp[] = urlencode($tp);
				}
			}
		}
		$temp[] = 'mobile='.($isMobile ? '0' : '1');
		$query = implode('&', $temp);

		$url = '';
		if (isset($url_parts['scheme']))
			$url .= $url_parts['scheme'].'://';
		if (isset($url_parts['host']))
			$url .= $url_parts['host'];
		if (isset($url_parts['path']))
			$url .= $url_parts['path'];

		$url .= '?'.$query;

		if (isset($url_parts['fragment']))
			$url .= '#'.$url_parts['fragment'];

		return encodeUrlParameters($url, false);
	}

	/**
	 * Return mobile site status
	 */
	public function isMobile()
	{
		$isMobile = false;
		
		//check is mobile forced to change status; if it is forced - save selection in cookie
		if (isset($_REQUEST["mobile"]))
		{
			$isMobile = $_REQUEST["mobile"] == '1';
			$isMobile = $this->setSession($isMobile);
		}
		//checks is browser mobile
		else if ($this->browserMobile())
		{
			//is session set - use value from there
			if ($this->sessionSet())
			{
				$isMobile = $this->sessionMobile();
			}
			//if it is not set - set it to mobile
			else
			{
				$isMobile = $this->setSession(true);
			}
		}
		else
		{
			//is session set - use value from there
			if ($this->sessionSet())
			{
				$isMobile = $this->sessionMobile();
			}
			//if it is not set - set it to non-mobile
			else
			{
				$isMobile = $this->setSession(false);
			}
		}
		
		return $isMobile;
	}
	
	/**
	 * Set custom to mobile app settings
	 */
	public function setSettings(&$settings)
	{
		$settings['GlobalFullWebsiteUrl'] = $settings['GlobalHttpUrl'];
		$settings['DesignSkinName'] = '_mobile';
		$settings['CatalogShowNested'] = 'NO';
		$settings['testimonials_Enabled'] = 'No';
		$settings['AfterProductAddedGoTo'] = 'Cart Page';
		
		$this->settings = $settings;
	}


	/**
	 * 
	 */
	public function optOutMobile()
	{
		setcookie(
			'_pcmb', 
			'non-mobile', 
			time()+60*60*24*180,
			"/", 
			$_SERVER["SERVER_PORT"] == "443" ? $this->settings["GlobalHttpsCookieDomain"] : $this->settings["GlobalHttpCookieDomain"]
		); // 6-month timeout
	}

	/**
	 * Old mobile function
	 */
	public function isMobile2()
	{
		$isMobile = false;
		if (isset($_REQUEST['mobile']) && $_REQUEST['mobile'] == 'false')
		{
			$this->optOutMobile();
			return false;
		}

		if ($this->isMobileCookieSet())
		{
			$isMobile = $_COOKIE['_pcmb'] == '1';
		}
		else
		{
			$isMobile = $this->isMobileUrl();
			if (!$isMobile)
			{
				$mobileDetect = new Mobile_Detect();
				$isMobile = $mobileDetect->isMobile();
			}
			
			$expires = time()+60*60*24;  // 1 day on auto detection
			setcookie(
				'_pcmb', 
				$isMobile ? '1' : '0', 
				$expires,
				"/", 
				$_SERVER["SERVER_PORT"] == "443" ? $this->settings["GlobalHttpsCookieDomain"] : $this->settings["GlobalHttpCookieDomain"]	
			);
		}
		return $isMobile;
	}
	
	/**
	 *
	 * @return type 
	 */
	public function isMobileUrl()
	{
		$currentHost = strtolower($_SERVER['HTTP_HOST']);
		
		$host = parse_url($_SERVER["SERVER_PORT"] == 443 ? $this->settings['GlobalMobileHttpsUrl'] : $this->settings['GlobalMobileHttpUrl'], PHP_URL_HOST);
		
		if ($host == '')
			return false;

		return $currentHost == $host;
	}

	/**
	 *
	 * @return type 
	 */
	public function isMobileCookieSet()
	{
		return isset($_COOKIE['_pcmb']) && trim($_COOKIE['_pcmb']) != '';
	}
}