<?php
/**
 * @class NewsletterConnector
 */
class Newsletter_Connector
{
	/**
	 * Subscribe to newsletter
	 * @param string $email - valid email address
	 * @param string $email_mode - Expects html or text 
	 * @return boolean - true if email added to emails table, false otherwise
	 */
	static function subscribe($email, $email_mode, $suppressDb = false)
	{
		global $settings, $db, $user;

		$ret = false;
		if (!$suppressDb)
		{
			$db->query("SELECT * FROM ".DB_PREFIX."emails WHERE email LIKE BINARY '".$db->escape($email)."'");
			if (!$db->moveNext())
			{
				$db->query("INSERT INTO ".DB_PREFIX."emails VALUES(NULL, NOW(), '".$db->escape($email)."','".$db->escape($email_mode)."', 0, 'No', 'Yes', 'No')");
				$ret = true;
				if ($user->auth_ok)
				{
					$db->reset();
					$db->assignStr('newsletters', 'Yes');
					$db->update(DB_PREFIX.'users', "WHERE uid='$user->id'");
				}
			}
		}

		if (defined('NEWSLETTER_SUB_TEST') && NEWSLETTER_SUB_TEST)
		{
			error_log('Newsletter Subscribe - email: '.$email.', email_mode: '.$email_mode);
		}
		else
		{
			//check if mailchimp is enabled on this cart,
			//if so, then subscribe the user to mailchimp newsletter
			$chimpenabled = $settings["mailchimp_enabled"];
			$chimpkey = $settings["mailchimp_apikey"];
			$chimplist = $settings["mailchimp_newsletter_list"];
		
			if ($chimpenabled && $chimpkey && $chimplist)
			{
				$basepath = $settings["GlobalServerPath"];
				require_once("$basepath/content/vendors/mailchimp/MCAPI.class.php");
		
				$mc = new MCAPI($chimpkey);
				$subscribeResult = $mc->listSubscribe($chimplist, $db->escape($email), array(""), $email_mode, true, true);
			}
		}

		return $ret;
	}
	
	/**
	 * Unsubscribe to newsletter
	 * @param string $cancel_email - valid email address
	 * @return boolean - true if email added to emails table, false otherwise
	 */
	static function unsubscribe($cancel_email)
	{
		global $settings, $db;

		$status_changed = false;
		$db->query("SELECT * FROM ".DB_PREFIX."emails WHERE email LIKE BINARY '".$db->escape($cancel_email)."'");
		
		if ($db->moveNext())
		{
			$db->query("DELETE FROM ".DB_PREFIX."emails WHERE eid='".intval($db->col["eid"])."'");
			$status_changed = true;
		}
		
		$db->query("SELECT uid, email FROM ".DB_PREFIX."users WHERE email LIKE BINARY '".$db->escape($cancel_email)."' AND newsletters='Yes'");
		
		if ($db->moveNext())
		{
			$db->query("UPDATE ".DB_PREFIX."users SET newsletters='No', last_update = NOW() WHERE email LIKE BINARY '".$db->escape($cancel_email)."' AND newsletters='Yes'");
			$status_changed = true;
		}

		if (defined('NEWSLETTER_SUB_TEST') && NEWSLETTER_SUB_TEST)
		{
			error_log('Newsletter Unsubscribe - email: '.$cancel_email);
		}
		else
		{
			//check if mailchimp is enabled on this cart,
		    //if so, then subscribe the user to mailchimp newsletter
		    $chimpenabled = $settings["mailchimp_enabled"];
		    $chimpkey = $settings["mailchimp_apikey"];
		    $chimplist = $settings["mailchimp_newsletter_list"];
		
		    if ($chimpenabled && $chimpkey && $chimplist)
		    {
		        $basepath = $settings["GlobalServerPath"];
		        require_once("$basepath/content/vendors/mailchimp/MCAPI.class.php");
		        $mc = new MCAPI($chimpkey);
		        $mc->listUnsubscribe($chimplist, $db->escape($cancel_email), false, true, true);
		    }
		}

		return $status_changed;
	}
}