<?php
/**
 * Process one page checkout request
 */

	if (!defined("ENV")) exit();
	
	define('OPC_MSG_ERROR_DATA_MISSED', 1);
	
	/**
	 * OPC Response class
	 */
	class OnePageCheckoutResponse
	{
		private $_errors = array();
		private $_data = array();

		/**
		 * Saves error into array
		 * @param int, array, or string $msg
		 */
		public function setError($msg)
		{
			$a = is_array($msg) ? $msg : array($msg);
			foreach ($a as $msg)
			{ 
				switch ($msg)
				{
					case OPC_MSG_ERROR_DATA_MISSED : $this->_errors[] = ""; break;
					default : $this->_errors[] = $msg;
				}
			}
		}
		
		/**
		 * Return errors
		 */
		public function errors()
		{
			return count($this->_errors) > 0 ? $this->_errors : false;
		}
		
		/**
		 * Sets data value to be send in response
		 * @param string $varName
		 * @param mixed $varValue
		 */
		public function setData($varName, $varValue)
		{
			$this->_data[$varName] = $varValue;
		}
		
		/**
		 * Build OPC response
		 */
		public function getResponse()
		{
			$a = array(
				"result" => count($this->_errors) == 0,
				"errors" => $this->_errors,
				"data" => $this->_data
			);
			return json_encode($a);
		}
		
		/**
		 * @param string $data
		 */
		public function unserialize($data)
		{
			parse_str($data, $arr);
			return $arr;
		}
	}
	
	/**
	 * Validate shipping address
	 */
	function opcValidateUspsAddress(&$db, &$opc, &$addressToValidate, $msg)
	{
		if (!is_null($addressToValidate))
		{
			$addressToValidate['name'] = isset($addressToValidate['name']) ? $addressToValidate['name'] : $addressToValidate['fname']." ".$addressToValidate['lname'];
			$addressToValidate['country'] = isset($addressToValidate['country_id']) ? $addressToValidate['country_id'] : $addressToValidate['country'];
			$addressToValidate['state'] = isset($addressToValidate['state_id']) ? $addressToValidate['state_id'] : $addressToValidate['state'];
			$useUserAddress = isset($addressToValidate['use_user_address']) && $addressToValidate['use_user_address'] == '1';

			// validate onlye for US addresses and when user override is not active
			if ($addressToValidate['country'] == 1 && !$useUserAddress)
			{
				$addressValidator = new Shipping_DialAZip('400540', '400540', 'HOCKEYSTAR', $settings);

				// selecte state code
				$stateCodeResult = $db->query('SELECT short_name FROM '.DB_PREFIX.'states WHERE coid="1" AND stid="'.intval($addressToValidate['state']).'"');
				if ($db->moveNext($stateCodeResult, MYSQL_ASSOC))
				{
					$stateCode = $db->col['short_name'];

					// validate address
					$validationResult = $addressValidator->validateAddress(
						$addressToValidate['name'],
						$addressToValidate['address1'],
						$addressToValidate['address2'],
						$addressToValidate['city'],
						$stateCode,
						$addressToValidate['zip']
					);

					switch ($validationResult)
					{
						case 10 : $opc->setError($msg['shipping']['validation_error_invalid_dual_address']); return false; break;
						case 11 : $opc->setError($msg['shipping']['validation_error_invalid_zip_code']); return false; break;
						case 12 : $opc->setError($msg['shipping']['validation_error_invalid_state']); return false; break;
						case 13 : $opc->setError($msg['shipping']['validation_error_invalid_city']); return false; break;
						case 21 : $opc->setError($msg['shipping']['validation_error_address_not_found']); return false; break;
						case 25 : $opc->setError($msg['shipping']['validation_error_street_error']); return false; break;
						case 22 : 
						{
							// preselect state ids
							$db->query('SELECT stid, short_name, name FROM '.DB_PREFIX.'states WHERE coid=1');
							$stateIds = array();
							while ($db->moveNext())
							{
								$stateIds[strtoupper($db->col['short_name'])] = array('id' => $db->col['stid'], 'name' => $db->col['name']);
							}
							
							// build array for address variants
							$addressVariants = $addressValidator->getAddressVariants(
								$addressToValidate['name'], 
								$addressToValidate['address1'], 
								$addressToValidate['address2'], 
								$addressToValidate['city'], 
								$stateCode, 
								$addressToValidate['zip'], 
								$stateIds
							);
							
							if ($addressVariants)
							{
								$opc->setData("shippingAddressVariants", $addressVariants);
							}

							// technically validation did not pass so return false
							return false;
							break;
						}
						
						case 31 :
						case 32 :
						{
							$validationResponse = $addressValidator->getLastResponse();

							if (is_string($validationResponse['RDI']) && $validationResponse['RDI'] != '')
							{
								$addressToValidate['address_type'] =  $validationResponse['RDI'] == 'B' ? 'Business' : 'Residential';
							}

							$addressToValidate['address1'] = $validationResponse['AddrLine1'];
							$addressToValidate['address2'] = is_string($validationResponse['AddrLine2']) ? $validationResponse['AddrLine2'] : '';
							$addressToValidate['city'] = $validationResponse['City'];
							$addressToValidate['zip'] = $validationResponse['ZIP5'];

							if (strtoupper($stateCode) != strtoupper($validationResponse['State']))
							{
								$result = $db->query('SELECT stid FROM '.DB_PREFIX.'states WHERE coid="1" AND short_name LIKE "'.strtoupper($validationResponse['State']).'"');

								if ($db->moveNext($result, MYSQL_ASSOC))
								{
									$addressToValidate['state'] = $db->col['stid'];
									$addressToValidate['state_id'] = $db->col['stid'];
								}
							}

							if (is_numeric($validationResponse['Plus4'])) $addressToValidate['zip'].='-'.$validationResponse['Plus4'];

							break;
						}
					}
				}
			}
		}

		return true;
	}

	/**
	 * Init 
	 */
	$action = isset ($_POST["action"]) ? $_POST["action"] : false;
	$data = isset ($_POST["data"]) ? json_decode($_POST["data"]) : false;
	
	$opc = new OnePageCheckoutResponse();
	$getShippingMethods = false;
	
	//check min subtotal amount
	$userLevel = ($user->auth_ok) ? ($user->level) : ($userCookie ? $userCookie["l"] : 0);
	$min_subtotal_amount = normalizeNumber($settings["MinOrderSubtotalLevel".$userLevel]);
	
	if ($min_subtotal_amount > $order->subtotalAmount)
	{
		$opc->setError($msg["cart"]["minimal_sustotal_not_reached"]."\n".$msg["cart"]["minimal_sustotal"].": ".getPrice($min_subtotal_amount));
	}
	elseif (!$action || !$data)
	{
		$opc->setError("Error processing form");
	}
	else
	{
		/**
	 	 * Process action
 	 	 */
		switch ($action)
		{
			case "doLogin" :
			{
				$_POST["login"] = $data->username;
				$_POST["password"] = $data->password;
				$opc_login = true;
				unset($_POST["remember_me"]); 
				require_once("content/engine/actions/action.user_login.php");
				if (!$login_successfull)
				{
					$opc->setError("Please enter valid username and password");
				}
				break;
			}
			/**
			 * Save billing data
			 */
			case "saveBillingData" : 
			{
				//billing form
				$billingForm = $opc->unserialize($data->billingForm);

				$customFieldsData = isset($billingForm["custom_field"]) ? $billingForm["custom_field"] : array();
				unset($billingForm["custom_field"]);
				unset($billingForm["custom_field_required"]);
				unset($billingForm["custom_field_name"]);
				unset($billingForm["custom_field_place"]);

				// try to validate billing data with USPS
				if ($data->shippingAsBilling && $settings['EndiciaUspsAddressValidation'] == 1)
				{
					if (!isset($billingForm["payment-profile-id"]) || (isset($billingForm["payment-profile-id"]) && $billingForm["payment-profile-id"] == 'billing'))
					{
						// check is address correct, and if it is not, just exit
						if (!opcValidateUspsAddress($db, $opc, $billingForm, $msg)) break;
					}
				}

				//check data to convert current express checkout to normal user
				$positiveResult = false;
				if (isset($billingForm["create_account"]) && $billingForm["create_account"])
				{
					//try to create account
					if (!$user->signUp($billingForm, $customFieldsData, $user->id))
					{
						$opc->setError($user->errors);		
					}
					else
					{
						emailNewUserRegistered($db, $user->id, $customFields);
						
						$user->login($billingForm["login"], $billingForm["password"]);
						
						if ($user->auth_ok)
						{
							//set tracking cookie
							$user->setCookie();
							
							if (parse_url($settings["GlobalHttpUrl"], PHP_URL_HOST) != parse_url($settings["GlobalHttpsUrl"], PHP_URL_HOST))
							{
								$opc->setData(
									"backgroundURL",
									$url_http."ua=".USER_SET_COOKIE."&data=".base64_encode(serialize(array("b"=>false, "i"=>$user->id, "s"=>md5($user->session_id), "l"=>$user->level, "u"=>$billingForm["login"])))
								);
							}
							
							unset($_SESSION["user_started_checkout"]);
							
							$positiveResult = true;
						
							$opc->setData("accountCreated", true);
						}
						else
						{
							$opc->setError("Unknown error happened");	
						}
					}
				}
				else
				{
					//check is payment profile used
					if (isset($billingForm["payment-profile-id"]) && $billingForm["payment-profile-id"] != 'billing')
					{
						$paymentProfileId = $billingForm["payment-profile-id"];
						require_once("content/classes/class.payment_profile.php");
						$paymentProfileManager = new Payment_Profile();
						$paymentProfile = $paymentProfileManager->getPaymentProfileById($db, $user, $paymentProfileId);
						
						if ($paymentProfile)
						{
							$user->updateBillingInfoFromPaymentProfile($paymentProfile);
							$positiveResult = true;
						}
					}
					else
					{
						//try to store billing data
						if (!$user->updateBillingInfo($billingForm, $customFieldsData))
						{
							$opc->setError($user->errors);
						}
						else
						{
							$positiveResult = true;
							$opc->setData("accountCreated", false); 
						}
					}
				}

				if ($positiveResult)
				{
					$billingData = $user->getBillingInfo();
					$opc->setData("billingData", $billingData);

					//update shipping address when needed
					if ($data->shippingAsBilling)
					{
						$billingData['address_type'] = isset($billingData['company']) && trim($billingData['company']) != '' ? 'Business' : 'Residential';
						$order->updateShippingAddress($billingData);
						$shippingAddress = $order->getShippingAddress();
						$opc->setData("shippingAddress", $shippingAddress);
						
						$getShippingMethods = true;
					}
				}
				break;
			}
			
			/**
			 * Check promo code
			 */
			case "checkPromoCode" :
			{
				$promoResult = false;
				
				// Get form data
				$promoCode = $data->promo;
				
				// Check promo code
				if (trim($promoCode) != '' && validate($promoCode, VALIDATE_STRING))
				{
					$_SESSION["order_promo_code"] = $promoCode;
					$order->resetPromo();
					$promoResult = $order->checkPromoCode($promoCode);
				}
				
				if (!$promoResult)
				{
					$_SESSION["order_promo_code"] = '';
					$opc->setError("Entered promo code is expired, cannot be used with your order or is invalid");
				}
				
				// Set response
				$opc->setData("promoResult", $promoResult);
				
				$getShippingMethods = true;
				
				break;
			}
			
			/**
			 * Save shipping data
			 */
			case "saveShippingAddress" : 
			{
				//update shipping address when needed
				$shippingForm = $opc->unserialize($data->shippingForm);
				
				//custom fields
				$customFieldsData = isset($shippingForm["custom_field"]) ? $shippingForm["custom_field"] : array();
				unset($shippingForm["custom_field"]);
				unset($shippingForm["custom_field_required"]);
				unset($shippingForm["custom_field_name"]);
				unset($shippingForm["custom_field_place"]);
				
				$_SESSION["opc-shipping-address-id"] = $shippingForm["shipping-address-id"];
				
				$shippingAddressChanged = false;

				/**
				 * Shipping the same as billing
				 */
				if ($shippingForm["shipping-address-id"] == "billing")
				{
					$billingData = $user->getBillingInfo();
					$billingData['address_type'] = isset($billingData['company']) && trim($billingData['company']) != '' ? 'Business' : 'Residential';
					$shippingForm = array_merge($shippingForm, $billingData);
				}
				/**
				 * Shipping address from address book
				 */
				elseif (is_numeric($shippingForm["shipping-address-id"]))
				{
					$shippingForm = array_merge($shippingForm, $user->getShippingAddressById(intval($shippingForm["shipping-address-id"])));
				}
				else
				{
					// we suppose that address is new
				}

				/**
				 * Validate address
				 */
				if (!$user->validateAddress($shippingForm))
				{
					$opc->setError($user->errors);
				}

				if (is_array($customFieldsData))
				{
					$order->customFields()->resetErrors();
					$order->customFields()->validateCustomFields("shipping", $customFieldsData);

					if ($order->customFields()->is_error)
					{
						$opc->setError($order->customFields()->errors);
					} 
				}

				if ($opc->errors()) break;
				
				// try to validate shipping data with USPS
				if ($settings['EndiciaUspsAddressValidation'] == 1)
				{
					// check is address correct, and if it is not, just exit
					if (!opcValidateUspsAddress($db, $opc, $shippingForm, $msg)) break;
				}

				/**
				 * Save shipping address
				 */
				$order->updateShippingAddress($shippingForm, $customFieldsData);
				
				/**
				 * Set updated data
				 */
				$shippingAddress = $order->getShippingAddress();
				$opc->setData("shippingAddress", $shippingAddress);
				
				/**
				 * Get shipping methos
				 */
				$getShippingMethods = true;
				break;
			}
			
			/**
			 * Get shipping methos
			 */
			case "getShippingMethods" :
			{
				$getShippingMethods = true;
				break;
			}
			
			/**
			 * Apply gift certificate
			 */
			case "applyGiftCertificate" :
			{
				// Check shipping method
				if (trim($data->shippingMethodId) != '')
				{
					$order->updateShippingMethod($data->shippingMethodId);
					$errorMessage = '';
					$order->getShippingAmount($errorMessage);
					if ($errorMessage != '')
					{
						$opc->setError($errorMessage);
					}
					else 
					{
						$order->closeShipping();	
					}
					$order->recalcOrderData();
				}
				
				// Check and apply gift certificate
				$giftCertificateForm = $opc->unserialize($data->giftCertificateForm);
				
				$giftError = true;
				
				$gcFirstName = trim($giftCertificateForm["gift_cert_first_name"]);
				$gcLastName = trim($giftCertificateForm["gift_cert_last_name"]);
				$gcVoucher = trim($giftCertificateForm["gift_cert_voucher"]);
				
				if ($gcFirstName != '' && $gcLastName != '' && $gcVoucher != '')
				{
					$giftCert = new GiftCertificates($db);
					$giftCertData = $giftCert->GetOrderedGiftCert($gcFirstName, $gcLastName, $gcVoucher);
					
					// Note: we only validating gift certificate without applying
					if ($giftCertData) 
					{
						// gift certificate is correct
						$giftError = false;
						$opc->setData("giftCertificateAmount", $giftCertData['balance']);
						
						$_SESSION["order_gift_certificate"] = array(
							"first_name" => $gcFirstName,
							"last_name" => $gcLastName,
							"voucher" => $gcVoucher
						);
					}
					else
					{
						unset($_SESSION["order_gift_certificate"]);
					}
				}

				if ($giftError)
				{
					$opc->setError("Provided gift certificate is not valid");
				}	
			
				break;
			
			}
		
			/**
			 * Process payment
			 */
			case "processPayment" :
			{
				// Check additional information
				if ($data->additionalInformation)
				{
					$additionalInformation = $opc->unserialize($data->additionalInformation);
					
					//gift message
					if ($settings["GiftCardActive"] == "YES")
					{
						if (isset($additionalInformation["add_gift"]) && $additionalInformation["add_gift"] == "yes")
						{
							$order->storeGiftMessage(isset($additionalInformation["gift_message"]) ? $additionalInformation["gift_message"] : "");
						}
					}
					
					//custom fields
					$customFieldsData = isset($additionalInformation["custom_field"]) ? $additionalInformation["custom_field"] : array();
					
					//store custom fields
					if (is_array($customFieldsData))
					{
						if (!$user->customFields()->validateCustomFields("invoice", $customFieldsData))
						{
							$opc->setError($user->customFields()->errors);
						}
						else
						{
							$customFields->storeCustomFields("invoice", $customFieldsData, $user->id, $order->oid, 0);
						}
					}
				}

				// Check shipping method
				if (trim($data->shippingMethodId) != '')
				{
					$order->updateShippingMethod($data->shippingMethodId);
					$errorMessage = '';
					$order->getShippingAmount($errorMessage);
					
					if ($errorMessage != '')
					{
						$opc->setError($errorMessage);
					}
					else 
					{
						$order->closeShipping();	
					}
					$order->recalcOrderData();
				}
				else
				{
					$order->getHandlingAmount(true, true);
					$order->recalcOrderData();
				}
				
				// Recheck promo code
				if (isset($_SESSION["order_promo_code"]) && $_SESSION["order_promo_code"] != '')
				{
					$promoResult = $order->checkPromoCode($_SESSION["order_promo_code"]);
					if (!$promoResult)
					{
						$_SESSION["order_promo_code"] = '';
						$opc->setError("Entered promo code is expired, cannot be used with your order or is invalid");
					}
				}
				
				// Recheck and apply gift certificate
				if ($data->giftCertificateForm)
				{
					$giftError = true;
					
					$giftCertificateForm = $opc->unserialize($data->giftCertificateForm);
					
					$gcFirstName = trim($giftCertificateForm["gift_cert_first_name"]);
					$gcLastName = trim($giftCertificateForm["gift_cert_last_name"]);
					$gcVoucher = trim($giftCertificateForm["gift_cert_voucher"]);
				
					if ($gcFirstName != '' && $gcLastName != '' && $gcVoucher != '')
					{
						$giftCert = new GiftCertificates($db);
						$giftCertData = $giftCert->GetOrderedGiftCert($gcFirstName, $gcLastName, $gcVoucher);
					
						if ($giftCertData) 
						{
							// gift certificate is correct
							$giftError = false;
							$giftCert->SetOrderGiftCert($order->oid, $gcFirstName, $gcLastName, $gcVoucher);
						
							if ($order->totalAmount > $giftCertData['balance'])
							{
								$giftCert->UpdateOrderGCAmount($order->oid, $giftCertData['balance']);
								$order->gift_cert_amount = $giftCertData['balance'];
							}
							else
							{
								$giftCert->UpdateOrderGCAmount($order->oid, $order->totalAmount);
								$order->gift_cert_amount = $order->totalAmount;
								$order->updatePaymentMethod("free");
							}
						}
					}
					
					if ($giftError)
					{
						$opc->setError("Please check gift certificate");
					}
				}
				
				// define the backurl var
				$backurl = getBackUrl();
				
				// extract data from post
				$paymentMethodsForm = $opc->unserialize($data->paymentMethodsForm);
				$paymentMethodForm = $opc->unserialize($data->paymentMethodForm);
				
				// get order data
				$order->getOrderData();
				
				// validate and update payment method
				$paymentMethodFreeError = false;
				
				if ((strtolower($paymentMethodsForm["payment-method-id"]) == "free" || $paymentMethodsForm["payment-method-id"] == 0) && ($order->totalAmount > 0) && ($order->gift_cert_amount != $order->totalAmount))
				{
					$paymentMethodFreeError = true;
				}
				
				if ($paymentMethodFreeError || !$order->updatePaymentMethod($paymentMethodsForm["payment-method-id"]))
				{
					$opc->setError("Please select payment method");
				}
				
				// Process payment
				if (!$opc->errors())
				{
					if ($data->paymentMethodType != 'ipn')
					{
						foreach ($paymentMethodForm as $key=>$value)
						{
							$_REQUEST[$key] = $value;
							$_POST[$key] = $value;
						}
						if ($data->paymentMethodType != 'custom' && $data->paymentMethodType != 'free')
						{
							require_once("content/engine/actions/action.order_process_payment.php");
							
							if (isset($payment) && get_class($payment) == 'PAYMENT')
							{
								if ($payment->redirect && $payment->redirectURL)
								{
									$opc->setData("backURL", $payment->redirectURL);
									$opc->setData("verification", true);
								}
								else if (isset($is_payment_error) && $is_payment_error)
								{
									if (isset($account_blocked) && $account_blocked)
									{
										$opc->setData("backURL", $backurl);
									}
									$opc->setData("paymentErrors", $payment->error_message);
									$opc->setData("success", false);
								}
								else
								{
									$opc->setData("backURL", $backurl);
									$opc->setData("success", true);
								}
							}
						}
						else
						{
							require_once("content/engine/actions/action.order_complete_order.php");
							$opc->setData("backURL", $backurl);
							$opc->setData("success", true);
						}	
							
						break;	
					}
				}
				// NOTE: do not break here. Under some conditions next step may be executed
			}
			
			/**
			 * Get payment form
			 */
			case "getPaymentForm" : 
			{
				// Get form data
				$paymentMethodsForm = $opc->unserialize($data->paymentMethodsForm);
				$paymentMethodId = $paymentMethodsForm["payment-method-id"];
				$paymentProfileId = isset($data->paymentProfileId) ? $data->paymentProfileId : 'billing';
				
				$payment = isset($payment) && is_object($payment) && get_class($payment) != "PAYMENT" ? $payment : new PAYMENT($db);
				
				// Update payment method
				$order->updatePaymentMethod($paymentMethodId);
				
				$override_payment_form = false;
				$payment_form_wide = false;
				$payment_form_override = false;
				$payment_in_popup = false;
				$payment_in_popup_width = 800;
				$payment_in_popup_height = 600;
				$payment_in_popup_title = 'Payment';
				
				$paymentForm = "";
				
				$_SESSION["opc-payment-method-id"] = $_SESSION["pc_method_id"] = $order->paymentMethodId;

				// Free order
				if ($paymentMethodId == 0)
				{
					$opc->setData('paymentForm', false);
					$opc->setData('paymentFormValidatorJS', false);
				}
				
				// Render form based on gateway properties / order data
				else if ($payment->isMethod($paymentMethodId))
				{
					// Get payment method
					$paymentMethod = $payment->methods[$order->paymentMethodId];
					
					// PayPalEC
					if ($paymentMethod->id == "paypalec" && !isset($_SESSION["paypal_express_checkout_details"]))
					{
						$paymentMethod->type = "ipn";
						$paymentMethod->steps = 2;
					}
					
					// Get data required to render form 
					$order->getOrderData();
					
					// ..and set template varialbes
					view()->assign("order_num", $order->order_num);
					view()->assign("order", $order);
					view()->assign("payment_method", $paymentMethod);
					
					if ($paymentProfileId != "billing")
					{
						require_once("content/classes/class.payment_profile.php");
						$paymentProfileManager = new Payment_Profile();

						//check are there payment profiles
						$paymentProfile = $paymentProfileManager->getPaymentProfileById($db, $user, $paymentProfileId);
						if ($paymentProfile["pid"] == $order->paymentMethodId)
						{
							view()->assign("paymentProfile", $paymentProfile ? $paymentProfile : false);
						}
					}
					
					if ($order->paymentIsRealtime == "Yes")
					{
						$order->getShippingAddress();
						
						// Create gateway class instance
						$gatewayFile = escapeFileName($paymentMethod->id);
						@require_once("content/engine/payment/payment_".$gatewayFile.".php");
						
						$db->query("SELECT * FROM ".DB_PREFIX."payment_methods WHERE id='".$db->escape($paymentMethod->id)."'");
						
						if ($db->moveNext())
						{
							$payment_processor_id = $db->col["id"];
							$payment_processor_class = $db->col["class"];
							$payment_processor_name = $db->col["name"];
						}
						eval("\$paymentGateway = new ".$payment_processor_class."();"); // $payment_processor_class loaded from gateway file
						
						// Init gateway
						$paymentGateway->db_id = $order->paymentMethodId;
						$paymentGateway->enableLog = defined('DEVMODE') && DEVMODE;
						$paymentGateway->getSettingsData($db);
						$paymentGateway->assignUser($user);
						$paymentGateway->assignOrder($order);
						$orderItems = $order->getOrderItems();
						$paymentGateway->assignOrderItems($orderItems);
						$paymentGateway->getCommonData($user, $order);
						
						// Check test/demo/sandbox mode
						$paymentTestMode = $paymentGateway->isTestMode();
						
						// Get payment URL 
						$paymentURL = $paymentGateway->getPostUrl();
						
						// PayPalEC
						if ($payment_processor_id == "paypalec" && !isset($_SESSION["paypal_express_checkout_details"]))
						{
							$paymentGateway->type = "ipn";
							$paymentGateway->steps = 2;
							
							$returnUrl = $url_dynamic.'p=one_page_checkout&oa='.ORDER_PROCESS_PAYMENT;
							$cancelUrl = $url_dynamic.'p=one_page_checkout';
							
							$cartItems = $order->getOrderItemsExtended();

							$expressCheckoutUrl = $paymentGateway->expressCheckout($order, $cartItems, false, $returnUrl, $cancelUrl);

							if ($paymentGateway->is_error)
							{
								$opc->setError($paymentGateway->error_message);
							}
							else
							{
								$paymentURL = $expressCheckoutUrl;
							}
						}
						
						if (trim($paymentURL) == "") $paymentURL = $url_https."p=one_page_checkout"; // Force HTTPS here
						
						$validatorJS = $paymentGateway->getValidatorJS();
						
						$payment_form_wide = $paymentGateway->payment_form_wide;
						$payment_in_popup = $paymentGateway->payment_in_popup;
						$payment_in_popup_width = $paymentGateway->payment_in_popup_width;
						$payment_in_popup_height = $paymentGateway->payment_in_popup_height;
						$payment_in_popup_title = $paymentGateway->payment_in_popup_title;
						
						if (!$paymentGateway->override_payment_form)
						{
							view()->assign("payment_test_mode", $paymentTestMode);
							view()->assign("auth_express", $user->express ? "yes" : "no");
							view()->assign("payment_in_popup", $payment_in_popup);
							view()->assign("payment_in_popup_width", $payment_in_popup_width);
							view()->assign("payment_in_popup_height", $payment_in_popup_height);
							view()->assign("payment_in_popup_title", $payment_in_popup_title);
							view()->assign("payment_form", $paymentGateway->getPaymentForm($db));
							view()->assign("payment_profile_form", $paymentGateway->getPaymentProfileForm($db));
							view()->assign("show_cvv2_info", $paymentGateway->show_cvv2);
							view()->assign("payment_url", $paymentURL);
							$paymentForm = view()->fetch("templates/pages/checkout/opc/opc-payment-form.html");
						}
						else
						{
							$paymentForm = $paymentGateway->getPaymentForm($db);
							$paymentProfileForm = $paymentGateway->getPaymentProfileForm($db);
							$payment_form_override = true;
						}
					}
					else
					{
						// Custom payment method
						$validatorJS = '';
						$paymentURL = $url_https."p=one_page_checkout";
						view()->assign("auth_express", $user->express ? "yes" : "no");
						view()->assign("payment_url", $paymentURL);
						$paymentForm = view()->fetch("templates/pages/checkout/opc/opc-payment-form.html");
					}
					
					// TODO: gift cerificate
					/**
					if ($order->gift_cert_amount > 0 && $order->totalAmount > $order->gift_cert_amount)
					{
						//$objResponse->addAssign("gift_cert_error", "style.display", "none");
						//$objResponse->addAssign("gift_cert_alt_payment", "style.display", "block");
						//$objResponse->addScript("altPaymentGiftCert('$order->totalAmount', '$order->gift_cert_amount')");
					}
					**/
					
					$opc->setData("paymentMethodId", $paymentMethodId);
					$opc->setData("paymentForm", $paymentForm);
					$opc->setData("paymentFormWide", $payment_form_wide);
					$opc->setData("paymentFormPopUp", $payment_in_popup);
					$opc->setData("paymentFormPopUpWidth", $payment_in_popup_width);
					$opc->setData("paymentFormPopUpHeight", $payment_in_popup_height);
					$opc->setData("paymentFormPopUpTitle", $payment_in_popup_title);
					$opc->setData("paymentFormOverride", $payment_form_override);
					$opc->setData("paymentFormUrl", $paymentURL);
					$opc->setData("paymentFormValidatorJS", $validatorJS);
				}
				else
				{
					// Error: cannot find class by ID
					$opc->setError("Cannot find payment gateway by ID");
				}
					
				/**
				 * Set session variables
				 */
				$_SESSION["pc_order_id"] = $order->oid;
				$_SESSION["pc_order_amount"] = $order->totalAmount - $order->shippingAmount - $order->taxAmount - ($order->handlingSeparated ? $order->handlingAmount : 0.00);
				$_SESSION["pc_method_id"] = $order->paymentMethodId;
				$_SESSION["pc_order_quantity"] = $order->itemsCount;
				$_SESSION["pc_order_num"] = $order->order_num;

				break;
			}					
		}
	}
	
	//get shipping methods
	if ($getShippingMethods)
	{

		// check for bongo shipping
		$bongoActive = $db->selectOne('SELECT * FROM '.DB_PREFIX.'payment_methods WHERE id="bongocheckout" AND active="Yes"') ? true : false;
		$bongoUsed = false;

		// get rates
		$rates = false;
		
		require_once('content/classes/class.currencies.php');
		$currencies = new Currencies($db);
		$defaultCurrency = $currencies->getDefaultCurrency();
		$currencyExchangeRates = $currencies->getExchangeRates();

		$opc->setData('shippingIsFree', $order->shippingIsFree ? "YES" : "NO");

		if ($order->shippingIsFree)
		{	
			if ($settings["ShippingShowAlternativeOnFree"] == "YES")
			{
				$ratesA = getShippingQuotes(
					$db, $user, $order, false, false, true, 
					$defaultCurrency['code'], $currencyExchangeRates,
					$bongoActive, $bongoUsed
				);
				
				$opc->setData("shippingMethodsAlternative", $ratesA);
			}
			else
			{
				if ($bongoActive)
				{
					$order->getShippingAddress();
					$bongoUsed = $order->shippingAddress['country_id'] != '1';
				}

				$opc->setData("shippingMethodsAlternative", false);
			}
		}
		else
		{
			$rates = getShippingQuotes(
				$db, $user, $order, false, false, false, 
				$defaultCurrency['code'], $currencyExchangeRates,
				$bongoActive, $bongoUsed
			);
			
			// show alternative shipping methods for free shipping products
			if ($settings["ShippingShowAlternativeOnFree"] == "YES" && $order->shippingFreeProductsCount > 0)
			{
				$ratesA = getShippingQuotes(
					$db, $user, $order, false, false, true, 
					$defaultCurrency['code'], $currencyExchangeRates,
					$bongoActive, $bongoUsed
				);
				
				$ratesA = filterAlternativeShippingQuotes($rates, $ratesA);
				$opc->setData("shippingMethodsAlternative", $ratesA);
			}
		}

		$opc->setData("bongoActive", $bongoActive);
		$opc->setData("bongoUsed", $bongoUsed);
		$opc->setData("shippingMethods", $rates && is_array($rates) && count($rates) > 0 ? $rates : false);
	}

	$user->getUserData();
	$order->findTaxZone();
	$order->recalcOrderData();
	$order->getOrderData();
	$order->getOrderItems();
	
	//tax-related for shipping
	$shippingTaxable = $settings["ShippingTaxable"];
	$shippingTaxRate = 0;
	$shippingTaxDescription = "";
	
	if ($shippingTaxable)
	{
		$shippingTaxClassId = $settings["ShippingTaxClassId"];
		
		if (array_key_exists($shippingTaxClassId, $order->taxRates))
		{
			$shippingTaxRate = $order->taxRates[$shippingTaxClassId]["tax_rate"];
			$shippingTaxDescription = $order->taxRates[$shippingTaxClassId]["description"];
		}
	}
	
	$opc->setData("orderData", array(
		"subtotalAmount" => $order->subtotalAmount,
		"subtotalAmountWithTax" => $order->subtotalAmountWithTax,
		"discountAmount" => $order->discountAmount,
		"promoDiscountAmount" => $order->promoDiscountAmount,
		// Promo Code calculations
		"promoDiscountValue" => $order->promoDiscountValue,
		"promoDiscountType" => $order->promoDiscountType,
		"promoType" => $order->promoType, // Global, Product, or Shipping
		"shippingAmount" => $order->shippingAmount,
		"handlingSeparated" => $settings["ShippingHandlingSeparated"], 
		"handlingSeparatedWhenShippingNotFree" => $settings["ShippingHandlingSeparated"],
		"handlingSeparatedWhenShippingFree" => 1,
		"handlingAmount" => $order->getHandlingAmount(false, false),
		"handlingAmountWhenShippingNotFree" => $order->getHandlingAmount(false, false),
		"handlingAmountWhenShippingFree" => $order->getHandlingAmount(true, false),
		"handlingTaxable" => $order->handlingTaxable,
		"handlingTaxRate" => $order->handlingTaxRate,
		"handlingTaxDescription" => $order->handlingTaxDescription,
		"taxAmount" =>
			$order->taxAmount -
			($order->shippingTaxable ? $order->shippingTaxAmount : 0) -
			($order->handlingTaxable ? $order->handlingTaxAmount : 0),
		"totalAmount" => $order->totalAmount,
		"shippingTaxable" => $shippingTaxable,
		"shippingTaxRate" => $shippingTaxRate,
		"shippingTaxDescription" => $shippingTaxDescription,
		"insureShipActive" => intval($order->insureShipActive),
		"insureShipAmount" => $order->insureShipAmount,
		"giftCertificate" => isset($_SESSION["order_gift_certificate"]) ? $_SESSION["order_gift_certificate"] : false
	));
	
	echo $opc->getResponse();
	
	_session_write_close();
	$db->done();
	die();
	