<?php

	if (!ini_get("date.timezone"))
	{
		date_default_timezone_set("America/Phoenix");
	}

	if (!defined('DS')) define("DS", DIRECTORY_SEPARATOR);
	if (!defined('APP_SYSTEM_TIMEZONE')) define('APP_SYSTEM_TIMEZONE', date_default_timezone_get());

	class tmp_object{};

	function create_nonce($action)
	{
		global $auth_ok, $settings;

		$uid = 0;
		if ($auth_ok)
		{
			global $admin, $user;
			if ($admin)
			{
				$uid = $admin['aid'];
			}
			else if ($user)
			{
				$uid = $user->id;
			}
		}
		
		$salt = nonce_salt();

		$i = nonce_time();

		$hash = nonce_hash($i, $uid, $action);
		$_SESSION[$hash] = true;

		return $hash;
	}

	function nonce_time()
	{
		return rand(0,10000);
	}

	function verify_nonce($nonce, $action)
	{
		if (isset($_SESSION[$nonce]))
		{
			unset($_SESSION[$nonce]);
			return true;
		}
		return false;
	}

	function nonce_hash($i, $uid, $action)
	{
		$salt = nonce_salt();
		
		$nonce = hash_hmac('md5', $i.$uid.$action, $salt);

		return $nonce;
	}
	
	/**
	 * Displays a generic message when an invalid nonce value was submmited
	 */
	function invalid_nonce_message()
	{
		$adminView = Ddm_AdminView::getInstance();
		$adminView->assign('PageHeader', array("Invalid form submission", "", "ic-cross"));
		$adminView->assign('PageWhat',array(
			array("text"=>"Click here to check error", "url"=>"javascript:history.back()")
		));

		$adminView->render('pages/invalid-nonce');		
	}
	
	/**
	 * 
	 */
	function safe_mode()
	{
		if (floatval(PHP_VERSION) < 5.3)
		{
			$safe_mode = ini_get('safe_mode');
			if ($safe_mode === true || strtolower($safe_mode) == 'on')
				return true;
		}
		//safe_mode deprecated in 5.3
		return false;
	}
	
	/**
	 * Resolve magic quotes issue
	 * @param $el
	 * @return unknown_type
	 */
	function strips(&$el)
	{
		if (is_array($el))
		{
			foreach ($el as $k=>$v)
			{
				if ($k!='GLOBALS')
				{
					strips($el[$k]);
				}
			}
		}
		else
		{
			$el = stripslashes($el);
		}
	}
	
	/**
	 * Transforms any string to UTF8
	 * @param string $string
	 */
	function getUTF8($string, $encoding = false)
	{
		mb_detect_order("UTF8, ASCII, auto");
		$enc = mb_detect_encoding($string);
		return mb_convert_encoding($string, "UTF-8", $enc == "" ? "ASCII" : $enc);
	}
	
	/**
	 * Get XML or SimpleXML Object and transform it to array
	 * @return array
	 */
	function xml2array($input, $callback = null, $recurse = false)
	{
	    // Get input, loading an xml string with simplexml if its the top level of recursion
	    $data = ((!$recurse) && is_string($input))? simplexml_load_string($input): $input;
	    // Convert SimpleXMLElements to array
	    if ($data instanceof SimpleXMLElement || $data instanceof stdClass)
	    {
	    	$data = (array) $data;
			if (count($data) == 0) $data = '';
	    } 

	    // Recurse into arrays
	    if (is_array($data)) foreach ($data as &$item) $item = xml2array($item, $callback, true);
	    // Run callback and return
	    return (!is_array($data) && is_callable($callback))? call_user_func($callback, $data): $data;
	}
	
	/**
	 * Transforms array to readable text
	 * @param array $array 
	 */
	function array2text($array, &$text = '', $level = 0)
	{
		foreach ($array as $key=>$value)
		{
			
			if (is_array($value))
			{
				$text.=str_repeat("  ", $level).$key."\n";
				array2text($value, $text, $level + 1);
			}
			else
			{
				if (is_bool($value))
				{
					$value = $value ? "true" : "false";
				}
				elseif (is_null($value))
				{
					$value = 'null';
				}
				$text.=str_repeat("  ", $level).$key." = ".$value."\n";
			}
		}
		if ($level == 0)
		{
			return $text;
		}
	}
	
	/**
	 * Resolve double encoding in smarty htmlspecialchars
	 * @param string $string
	 * @return string
	 */
	function smarty_modifier_htmlspecialchars($string)
	{
		return gs($string);
	}
	
	/**
	 * Return current URL
	 */
	function currentPageURL($encodeParameters = false)
	{
		$pageURL = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') ? 'https://' : 'http://';
		$pageURL .= $_SERVER['SERVER_PORT'] != '80' ? $_SERVER["SERVER_NAME"].":".$_SERVER["SERVER_PORT"].$_SERVER["REQUEST_URI"] : $_SERVER['SERVER_NAME'] . $_SERVER['REQUEST_URI'];

		if ($encodeParameters)
		{
			$pageURL = encodeUrlParameters($pageURL, true, true);
		}

		return $pageURL;
	}

	function encodeUrlParameters($pageUrl, $useGetParams = true, $removeUrlParam = false)
	{
		$parts = parse_url($pageUrl);

		$url = $parts['scheme'].'://'.$parts['host'];
		$url .= isset($parts['port']) ? ':'.$parts['port'] : '';
		$url .= isset($parts['path']) ? $parts['path'] : '';

		$qs = '';
		if ($useGetParams)
		{
			foreach ($_GET as $key => $value)
			{
				if (!($removeUrlParam && $key == 'url')) $qs .= urlencode($key).'='.urlencode($value).'&';
			}
		}
		else if (isset($parts['query']))
		{
			$qsParts = array();
			parse_str($parts['query'], $qsParts);

			foreach ($qsParts as $key => $value)
			{
				$qs .= urlencode($key).'='.urlencode($value).'&';
			}
		}

		if (strlen($qs) > 0)
		{
			$url .= '?'.ltrim(rtrim($qs, '&'), '?');
		}

		return $url;
	}

	function getSessionSavePath()
	{
		global $settings;

		$session_dir = $settings['GlobalServerPath'].DS.'content'.DS.'cache'.DS.'sessiondata';

		if (isset($settings['session_save_path']) && !is_null($settings['session_save_path']) && trim($settings['session_save_path']) != '')
		{
			if (is_dir($settings['session_save_path']) && is_writeable($settings['session_save_path']))
			{
				$session_dir = $settings['session_save_path'];
			}
			else
			{
				error_log('Save session path is not a directory or is not writable: '.$settings['session_save_path']);
			}
		}

		return $session_dir;
	}

	function setSessionSavePath()
	{
		global $settings;

		$session_dir = getSessionSavePath();

		ini_set('session.save_path', $session_dir);
	}

	/**
	 * Write and close current session , but also copy vars changed in HTTPS mode to HTTP mode 
	 */
	function _session_write_close()
	{
		session_write_close();
	}
	
	/**
	 * Generate MySQL replacement SQL
	 * @param $patterns
	 * @param $replacements
	 * @param $subject
	 * @return string
	 */
	function getMySQLReplace($patterns, $replacements, $subject)
	{
		$s = $subject;
		for ($i=0; $i<count($patterns); $i++)
		{
			$s = "REPLACE(".$s.", ".$patterns[$i].",".$replacements[$i].")"; 
		}
		return $s;
	}
	
	/**
	 * Returns category URL SQL
	 * @param $template
	 * @return string
	 */
	function setCategoryUrlSQL($lowerCase = false, $joiner = "-", $where = false)
	{
		$lowerCase = false;
		//lowerCase does not work - MySQL issue - http://bugs.mysql.com/bug.php?id=12903
		$joiner = mysql_real_escape_string($joiner);
		
		$r = getMySQLReplace(
			array(
				"'_'","'-'","'%CategoryName%'", "'%CategoryKey%'", "'%CategoryDbId%'", "'%CategoryPath%'",
				"'~'", "'`'", "'!'", "'@'", "'#'", "'$'", "'%'", "'^'", "'&'", "'*'", "'™'", 
				"'®'", "'℠'", "'₤'", "'£'", "'¥'", "'€'",
				"'('", "')'", "' '", "'_'", "'-'", "'+'", "'='", "'{'", "'}'", "'['",
				"']'", "':'", "';'", "'\"'", "'\\''", "'<'", "'>'", "','", "'?'",
				"'{$joiner}{$joiner}{$joiner}{$joiner}{$joiner}'", "'{$joiner}{$joiner}{$joiner}{$joiner}'", "'{$joiner}{$joiner}{$joiner}'", "'{$joiner}{$joiner}'",
				"'./'", "'/.'", "'-/'", "'/-'", "'///'", "'//'", "'777underscore777'", "'777dash777'"
			),				
			array(
				"'777underscore777'","'777dash777'","c.name", "c.key_name", "c.cid", "c.category_path",
				"'{$joiner}'", "''", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "''", "'{$joiner}'", "''", "''",
				"''", "''", "''", "''", "''", "''",
				"'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'",
				"'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "''", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'",
				"'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'",
				"'/'", "'/'", "'/'", "'/'", "'/'", "'/'", "'_'", "'-'"
			),
			"c.url_default"
		);
		return "UPDATE LOW_PRIORITY IGNORE ".DB_PREFIX."catalog AS c
				SET c.url_default = ".($lowerCase?"LCASE(":"")."TRIM(BOTH '".$joiner."' FROM ".$r.")".($lowerCase?")":"")."
				WHERE c.cid > 0 ".($where ? (" AND (".$where.")") : "");
	}
	
	/**
	 * Returns category URL SQL
	 * @param $template
	 * @return string
	 */
	function setManufacturerUrlSQL($lowerCase = false, $joiner = "-", $where = false)
	{
		$lowerCase = false;
		//lowerCase does not work - MySQL issue - http://bugs.mysql.com/bug.php?id=12903
		$joiner = mysql_real_escape_string($joiner);
		
		$r = getMySQLReplace(
			array(
				"'_'","'-'","'%ManufacturerName%'", "'%ManufacturerKey%'", "'%ManufacturerDbId%'",
				"'~'", "'`'", "'!'", "'@'", "'#'", "'$'", "'%'", "'^'", "'&'", "'*'", "'™'",
				"'®'", "'℠'", "'₤'", "'£'", "'¥'", "'€'",
				"'('", "')'", "' '", "'_'", "'-'", "'+'", "'='", "'{'", "'}'", "'['",
				"']'", "':'", "';'", "'\"'", "'\\''", "'<'", "'>'", "','", "'?'",
				"'{$joiner}{$joiner}{$joiner}{$joiner}{$joiner}'", "'{$joiner}{$joiner}{$joiner}{$joiner}'", "'{$joiner}{$joiner}{$joiner}'", "'{$joiner}{$joiner}'",
				"'./'", "'/.'", "'-/'", "'/-'", "'///'", "'//'", "'777underscore777'", "'777dash777'"
			),				
			array(
				"'777underscore777'","'777dash777'","c.manufacturer_name", "c.manufacturer_code", "c.manufacturer_id",
				"'{$joiner}'", "''", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "''", "'{$joiner}'", "''", "''",
				"''", "''", "''", "''", "''", "''",
				"'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'",
				"'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "''", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'",
				"'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'",
				"'/'", "'/'", "'/'", "'/'", "'/'", "'/'", "'_'", "'-'"
			),
			"c.url_default"
		);
		return "UPDATE LOW_PRIORITY IGNORE ".DB_PREFIX."manufacturers AS c
				SET c.url_default = ".($lowerCase?"LCASE(":"")."TRIM(BOTH '".$joiner."' FROM ".$r.")".($lowerCase?")":"")."
				WHERE c.manufacturer_id > 0 ".($where ? (" AND (".$where.")") : "");
	}
	
	/**
	 * Returns product URL SQL
	 * @param $template
	 * @return string
	 */
	function setProductUrlSQL($lowerCase = false, $joiner = "-", $where = false)
	{
		//lowerCase does not work - MySQL issue - http://bugs.mysql.com/bug.php?id=12903
		$joiner = mysql_real_escape_string($joiner);
		
		$r = getMySQLReplace(
			array(
				"'_'","'-'","'%ProductId%'", "'%ProductName%'", "'%ProductDbId%'",
				"'%CategoryName%'", "'%CategoryKey%'", "'%CategoryDbId%'", "'%CategoryPath%'", "'%CategoryURL%'",
				"'~'", "'`'", "'!'", "'@'", "'#'", "'$'", "'%'", "'^'", "'&'", "'*'", "'™'",
				"'®'", "'℠'", "'₤'", "'£'", "'¥'", "'€'",
				"'('", "')'", "' '", "'_'", "'-'", "'+'", "'='", "'{'", "'}'", "'['",
				"']'", "':'", "';'", "'\"'", "'\\''", "'<'", "'>'", "','", "'?'",
				"'{$joiner}{$joiner}{$joiner}{$joiner}{$joiner}'", "'{$joiner}{$joiner}{$joiner}{$joiner}'", "'{$joiner}{$joiner}{$joiner}'", "'{$joiner}{$joiner}'",
				"'./'", "'/.'", "'-/'", "'/-'", "'///'", "'//'", "'777underscore777'", "'777dash777'"
			),				
			array(
				"'777underscore777'","'777dash777'","p.product_id", "p.title", "p.pid",
				"c.name", "c.key_name", "c.cid", "c.category_path", "IF(c.url_custom='', c.url_default, c.url_custom)",
				"'{$joiner}'", "''", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "''", "'{$joiner}'", "''", "''",
				"''", "''", "''", "''", "''", "''",
				"'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'",
				"'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "''", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'",
				"'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'",
				"'/'", "'/'", "'/'", "'/'", "'/'", "'/'", "'_'", "'-'"
			),
			"p.url_default"
		);
		
		return "UPDATE LOW_PRIORITY IGNORE ".DB_PREFIX."products AS p, ".DB_PREFIX."catalog AS c
				SET p.url_default = ".($lowerCase?"LCASE(":"")."TRIM(BOTH '".$joiner."' FROM ".$r.")".($lowerCase?")":"")."	
				WHERE p.cid = c.cid ".($where ? (" AND (".$where.")") : "");
	}
	
	
	/**
	 * Returns page URL SQL
	 * @param $template
	 * @return string
	 */
	function setPageUrlSQL($lowerCase = false, $joiner = "-", $where = false)
	{
		//lowerCase does not work - MySQL issue - http://bugs.mysql.com/bug.php?id=12903
		$joiner = mysql_real_escape_string($joiner);
		
		$r = getMySQLReplace(
			array(
				"'%PageName%'", "'%PageTitle%'",
				"'~'", "'`'", "'!'", "'@'", "'#'", "'$'", "'%'", "'^'", "'&'", "'*'", "'™'",
				"'®'", "'℠'", "'₤'", "'£'", "'¥'", "'€'",
				"'('", "')'", "' '", "'_'", "'-'", "'+'", "'='", "'{'", "'}'", "'['",
				"']'", "':'", "';'", "'\"'", "'\\''", "'<'", "'>'", "','", "'?'",
				"'{$joiner}{$joiner}{$joiner}{$joiner}{$joiner}'", "'{$joiner}{$joiner}{$joiner}{$joiner}'", "'{$joiner}{$joiner}{$joiner}'", "'{$joiner}{$joiner}'",
				"'./'", "'/.'", "'-/'", "'/-'"
			),				
			array(
				"p.name", "p.title",
				"'{$joiner}'", "''", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "''", "'{$joiner}'", "''", "''",
				"''", "''", "''", "''", "''", "''",
				"'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'",
				"'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "''", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'",
				"'{$joiner}'", "'{$joiner}'", "'{$joiner}'", "'{$joiner}'",
				"'/'", "'/'", "'/'", "'/'"
			),
			"p.url_default"
		);
		
		return "UPDATE LOW_PRIORITY IGNORE ".DB_PREFIX."pages AS p
				SET p.url_default = ".($lowerCase?"LCASE(":"")."TRIM(BOTH '".$joiner."' FROM ".$r.")".($lowerCase?")":"")."	
				WHERE p.pid > 0 ".($where ? (" AND (".$where.")") : "");

	}
	
	function getManufacturerSEOUrl($url, $mid, $page = false, $force_http = false)
	{
		global $settings;
		if ($page && intval($page) != 1)
		{
			$pageNum = $settings["SearchURLManufacturerPaginatorTemplate"];
			if (($l = strlen($url)) > 0)
			{
				if ($url[$l-1] == "/")
				{
					$url = substr($url, 0, $l-1).$pageNum."/";
				}
				else
				{
					$p = pathinfo($url);
					if (isset($p['extension']) && $p['extension'] != "")
					{
						$url = substr($url, 0, $l - strlen($p['extension'])-1).$pageNum.".".$p['extension'];
					}
					else
					{
						$url.=$pageNum;
					}
				}
				$url = str_replace("%PageNumber%", $page, $url);
			}
		}
		
		return (defined('DESIGN_MODE') && DESIGN_MODE && !$force_http ? $settings["GlobalHttpsUrl"] : $settings["GlobalHttpUrl"])."/".$url;
	}
	
	/**
	 * Update category path for SEO
	 * @param $db
	 */
	function updateSeoCategoryPath($db)
	{
		$pathArray = array();
		$db->query("SELECT cid, name, parent FROM ".DB_PREFIX."catalog ORDER BY level");
		while (($c = $db->movenext()) != false)
		{
			$n = str_replace("/", " ", $c["name"]);
			$pathArray[$c["cid"]] = array_key_exists($c["parent"], $pathArray) ? ($pathArray[$c["parent"]]."/".$n) : $n;
		}
		foreach ($pathArray as $cid=>$path)
		{
			$db->query("UPDATE ".DB_PREFIX."catalog SET category_path='".$db->escape($path)."' WHERE cid='".$cid."'");
		}
	}
	
	/**
	 * Updates SEO urls
	 * @param $db
	 * @param $settings
	 * @param $categories
	 * @param $products
	 * @param $pages
	 */
	function updateSeoURLs($db, $settings, $categories=true, $products=true, $pages=true, $manufacturers=true)
	{
		set_time_limit(10000);
		
		if ($categories || $products)
		{
			updateSeoCategoryPath($db);
		}
		
		//category template changed
		if ($categories)
		{
			//render URL
			$db->query("UPDATE ".DB_PREFIX."catalog SET url_default='".$db->escape($settings["SearchURLCategoryTemplate"])."'");
			
			$query = setCategoryUrlSQL($settings["SearchAutoGenerateLowercase"] == "YES", $settings["SearchURLJoiner"]);
			$db->query($query);
			
			if ($settings["SearchAutoGenerateLowercase"] == "YES")
			{
				$db->query("UPDATE LOW_PRIORITY IGNORE ".DB_PREFIX."catalog SET url_default=LOWER(url_default);");
			}
			
			//update hash
			$db->query("UPDATE LOW_PRIORITY IGNORE ".DB_PREFIX."catalog SET url_hash=MD5(IF(url_custom='', url_default, url_custom))");
		}
			
		//product template changed
		if ($products)
		{
			//render url
			$db->query("UPDATE ".DB_PREFIX."products SET url_default='".$db->escape($settings["SearchURLProductTemplate"])."'");
			
			$query = setProductUrlSQL($settings["SearchAutoGenerateLowercase"] == "YES", $settings["SearchURLJoiner"]);
			$db->query($query);
			
			if ($settings["SearchAutoGenerateLowercase"] == "YES")
			{
				$db->query("UPDATE LOW_PRIORITY IGNORE ".DB_PREFIX."products SET url_default=LOWER(url_default);");
			}

			//remove possible trailing spaces
			//$db->query("UPDATE LOW_PRIORITY IGNORE ".DB_PREFIX."products SET url_default=REPLACE('//', '/', REPLACE('///', '/', url_default));");
						
			//update hash
			$db->query("UPDATE LOW_PRIORITY IGNORE ".DB_PREFIX."products SET url_hash=MD5(IF(url_custom='', url_default, url_custom))");
		}
		
		//manufacturers
		if ($manufacturers)
		{
			$db->query("UPDATE ".DB_PREFIX."manufacturers SET url_default='".$db->escape($settings["SearchURLManufacturerTemplate"])."'");
			$query = setManufacturerUrlSQL($settings["SearchAutoGenerateLowercase"] == "YES", $settings["SearchURLJoiner"]);
			$db->query($query);
			
			if ($settings["SearchAutoGenerateLowercase"] == "YES")
			{
				$db->query("UPDATE LOW_PRIORITY IGNORE ".DB_PREFIX."manufacturers SET url_default=LOWER(url_default);");
			}
			
			//update hash
			$db->query("UPDATE LOW_PRIORITY IGNORE ".DB_PREFIX."manufacturers SET url_hash=MD5(IF(url_custom='', url_default, url_custom))");
		}
		
		//page template changed
		if ($pages)
		{
			//render url
			$db->query("UPDATE ".DB_PREFIX."pages SET url_default='".$db->escape($settings["SearchURLPageTemplate"])."'");
			
			$query = setPageUrlSQL($settings["SearchAutoGenerateLowercase"] == "YES", $settings["SearchURLJoiner"]);
			$db->query($query);
			
			if ($settings["SearchAutoGenerateLowercase"] == "YES")
			{
				$db->query("UPDATE LOW_PRIORITY IGNORE ".DB_PREFIX."pages SET url_default=LOWER(url_default);");
			}
						
			//update hash
			$db->query("UPDATE LOW_PRIORITY IGNORE ".DB_PREFIX."pages SET url_hash=MD5(IF(url_custom='', url_default, url_custom))");
		}
	}
	
	/**
	 * Basic css minification
	 */
	function minifyCSS($css)
	{
		$css = preg_replace('#\s+#', ' ', $css);
		$css = preg_replace('#/\*.*?\*/#s', '', $css);
		$css = str_replace('; ', ';', $css);
		$css = str_replace(': ', ':', $css);
		$css = str_replace(' {', '{', $css);
		$css = str_replace('{ ', '{', $css);
		$css = str_replace(', ', ',', $css);
		$css = str_replace('} ', '}', $css);
		$css = str_replace(';}', '}', $css);
		
		$colorCodes = array("#00FFFF", "#F0FFFF", "#F5F5DC", "#FFE4C4", "#000000", "#0000FF", "#A52A2A", "#FF7F50", "#00FFFF", "#FFD700", "#808080", "#008000", "#4B0082", "#FFFFF0", "#F0E68C", "#00FF00", "#FAF0E6", "#800000", "#000080", "#808000", "#FFA500", "#DA70D6", "#CD853F", "#FFC0CB", "#DDA0DD", "#800080", "#FF0000", "#FA8072", "#A0522D", "#C0C0C0", "#FFFAFA", "#D2B48C", "#008080", "#FF6347", "#EE82EE", "#F5DEB3", "#FFFFFF", "#FFFF00");
		$colorNames = array("aqua", "azure", "beige", "bisque", "black", "blue", "brown", "coral", "cyan", "gold", "gray", "green", "indigo", "ivory", "khaki", "lime", "linen", "maroon", "navi", "olive", "orange", "orchid", "peru", "pink", "plum", "purple", "red", "salmon", "sienna", "silver", "snow", "tan", "teal", "tomato", "violet", "wheat", "white", "yellow");
		$css = str_ireplace($colorCodes, $colorNames, $css);
		
		$css = str_replace("\r\n", "\n", $css);
		return $css;
	}
	
	/**
	 * Basic html minification
	 * @param $html
	 */
	function minifyHTML($html)
	{
		// $html = preg_replace('/<!--([\\s\\S]*?)-->/', '', $html);
		$html = preg_replace("/(^[\r\n]*|[\r\n]+)[\s\t]*[\r\n]+/", "\n", $html);
		$html = str_ireplace(
			array("\r\n", " >", "\t"), 
			array("\n", ">", ""),
			$html
		);
		return $html;
	}
	
	/**
	 * Removes slashes from filename 
	 * @param $file_name
	 * @return unknown_type
	 */
	function escapeFileName($file_name)
	{
		return str_replace(array("/", "\\"), array("", ""), $file_name);
	}
	
	/**
	 * Returns back url
	 * @return unknown_type
	 */
	function getBackUrl()
	{
		global $_SERVER, $_SESSION, $_POST;
		if (isset($_POST["backurl"]) && $_POST["backurl"] != "")
		{
			return $_POST["backurl"];
		}
		elseif (isset($_SESSION["HTTP_REFERER"]) && $_SESSION["HTTP_REFERER"] != "")
		{
			return $_SESSION["HTTP_REFERER"];
		}
		elseif (isset($_SERVER["HTTP_REFERER"]) && $_SERVER["HTTP_REFERER"] != "")
		{
			return $_SERVER["HTTP_REFERER"];
		}
		return false;
	}
	
	/**
	 * @return string
	 */
	function backurl($backurl = false)
	{
		static $_backurl = false;
		if ($backurl) $_backurl = $backurl;
		return $_backurl;
	}

	/**
	 * 
	 * @param $db
	 * @param $pid
	 */
	function updateProductAttributesCount(&$db, $pid)
	{
		$db->query("SELECT COUNT(*) AS c FROM ".DB_PREFIX."products_attributes WHERE pid='".$pid."' AND is_active='Yes'");
		$db->moveNext();
		$attributes_count = $db->col["c"];
		$db->reset();
		$db->assignStr("attributes_count", $attributes_count);
		$db->update(DB_PREFIX."products", "WHERE pid='".$pid."'");
	}

	function normalizeProductLockFields(array &$lock_fields)
	{
		$map = array(
			'title' => 'name',
			'cid' => 'category',
			'secondary_categories[]' => 'secondary-categories',
			'price2' => 'sale-price',
			'meta_title' => 'meta-title',
			'meta_keywords' => 'meta-keywords',
			'meta_description' => 'meta-description',
			'image' => 'images',
			'image_url' => 'images',
		);

		foreach ($lock_fields as $key=>$lock_field)
		{
			if (isset($map[$lock_field]))
			{
				$lock_fields[$key] = $map[$lock_field];
			}
		}

		return $lock_fields;
	}

	/**
	 * Logs admin action
	 * @param unknown_type $message
	 */
	function adminLog($message, $is_error = false, $force = false)
	{
		global $db, $auth_admin_username, $settings;
		if ($settings["SecurityAdminActivyLogging"] == "YES" || $force)
		{
			$db->query("INSERT INTO ".DB_PREFIX."admins_logs VALUES(NOW(), ".sprintf("%u", ip2long($_SERVER["REMOTE_ADDR"])).",'".($is_error?"1":"0")."','".$db->escape(str_replace(chr(0), '', $auth_admin_username))."','".$db->escape($message)."')");
		}
	}
	
	/**
	 * Reads template file
	 * @param $templateFile
	 * @return text
	 */
	function readTemplateFile($templateFile)
	{
		global $settings;
		$pathBase = "content/engine/design/";
		$pathSkin = "content/skins/".escapeFileName($settings["DesignSkinName"])."/";
		$pathCustom = "content/skins/_custom/skin/";
		
		try
		{
			if (is_file($pathCustom.$templateFile)) return file_get_contents($pathCustom.$templateFile);
			if (is_file($pathSkin.$templateFile)) return file_get_contents($pathSkin.$templateFile);
			if (is_file($pathBase.$templateFile)) return file_get_contents($pathBase.$templateFile);
		}
		catch (Exception $e)
		{

		}
		return false;
	}
	
	/**
	 * Saves content
	 * @param $templateFile
	 * @param $content
	 * @return unknown_type
	 */
	function writeTemplateFile($templateFile, $content)
	{
		global $settings;
		try
		{
			$pathCustom = "content/skins/_custom/skin/";
			$pathCache = "content/cache/skins/".escapeFileName($settings["DesignSkinName"])."/";
		
			$templatePathCustom = dirname($pathCustom.$templateFile);
			$templatePathCache = dirname($pathCache.$templateFile);
		
			if (!is_dir($templatePathCustom)) mkdir($templatePathCustom, getDirectoryPermissionMode(), true);
			if (!is_dir($templatePathCache)) mkdir($templatePathCache, getDirectoryPermissionMode(), true);
		
			file_put_contents($pathCustom.$templateFile, $content);
			file_put_contents($pathCache.$templateFile, $content);
			return true;
		}
		catch (Exception $e)
		{
			return false;
		}
	}

	/**
	 * Returns browser data
	 * @return array
	 * Source: http://php.net/manual/en/function.get-browser.php
	 */
	function getBrowser()
	{
		$u_agent = $_SERVER['HTTP_USER_AGENT'];
		$bname = 'Unknown';
		$platform = 'Unknown';
		$version= "";

		//First get the platform?
		if (preg_match('/linux/i', $u_agent)) {
			$platform = 'linux';
		}
		elseif (preg_match('/macintosh|mac os x/i', $u_agent)) {
			$platform = 'mac';
		}
		elseif (preg_match('/windows|win32/i', $u_agent)) {
			$platform = 'windows';
		}

		// Next get the name of the useragent yes seperately and for good reason
		if(preg_match('/MSIE/i',$u_agent) && !preg_match('/Opera/i',$u_agent))
		{
			$bname = 'Internet Explorer';
			$ub = "MSIE";
		}
		elseif(preg_match('/Firefox/i',$u_agent))
		{
			$bname = 'Mozilla Firefox';
			$ub = "Firefox";
		}
		elseif(preg_match('/Chrome/i',$u_agent))
		{
			$bname = 'Google Chrome';
			$ub = "Chrome";
		}
		elseif(preg_match('/Safari/i',$u_agent))
		{
			$bname = 'Apple Safari';
			$ub = "Safari";
		}
		elseif(preg_match('/Opera/i',$u_agent))
		{
			$bname = 'Opera';
			$ub = "Opera";
		}
		elseif(preg_match('/Netscape/i',$u_agent))
		{
			$bname = 'Netscape';
			$ub = "Netscape";
		}
		else
		{
			$bname = 'Unknown';
			$ub = '';
		}

		// finally get the correct version number
		$known = array('Version', $ub, 'other');
		$pattern = '#(?P<browser>' . join('|', $known) .
		')[/ ]+(?P<version>[0-9.|a-zA-Z.]*)#';
		if (!preg_match_all($pattern, $u_agent, $matches)) {
			// we have no matching number just continue
		}

		// see how many we have
		$i = count($matches['browser']);
		if ($i > 0)
		{
			if ($i != 1)
			{
				//we will have two since we are not using 'other' argument yet
				//see if version is before or after the name
				if (strripos($u_agent,"Version") < strripos($u_agent,$ub))
				{
					$version= $matches['version'][0];
				}
				else
				{
					$version= $matches['version'][1];
				}
			}
			else
			{
				$version= $matches['version'][0];
			}
		}

		// check if we have a number
		if ($version==null || $version=="") {$version="?";}

		return array(
			'userAgent' => $u_agent,
			'name'      => $bname,
			'version'   => $version,
			'platform'  => $platform,
			'pattern'    => $pattern
		);
	}
	
	/**
	 * 
	 * @param $number
	 * @return unknown_type
	 */
	function normalizeNumber($number)
	{
		$s = "";
		$a = array("0", "1", "2", "3", "4", "5", "6", "7", "8", "9");
		$is_dot = false;
		$number = trim($number);
		if ($number != "")
		{
			if ($number[0] == "-")
			{
				$s = "-";
				$start = 1;	
			}
			else
			{
				$start = 0;	
			}
			for ($i = $start; $i < strlen($number); $i++)
			{
				$s.=in_array($number[$i], $a)?$number[$i]:"";
				if ($number[$i] == "." && ($is_dot == false))
				{
					$s.=".";
					$is_dot = true;
				}
			}
		}
		else
		{
			$s = 0;
		}
		return $s;
	}
	
	/**
	 * 
	 * @param $j
	 * @return unknown_type
	 */
	function myNum($j)
	{
		if ($j == "" || $j == 0) return "0";
		for ($i=2; $i<=5; $i++)
		{
			if(number_format($j, $i, ".", "")*1 == $j) return number_format($j, $i, ".", "");
		}
		return number_format($j, 5, ".", "");
	}
	
	/**
	 * 
	 * @param $dir_name
	 * @return unknown_type
	 */
	function clearDir($dir_name)
	{
		$dir = dir($dir_name);
		while (false !== ($entry = $dir->read()))
		{
			if (is_file($dir_name."/".$entry))
			{
	 		 	@unlink($dir_name."/".$entry);
			}
		}
		$dir->close();
	}
	
	/**
	 * 
	 * @param $file_name
	 * @return unknown_type
	 */
	if (!function_exists('getFileExt'))
	{
		function getFileExt($file_name)
		{
			$ar = explode('.', $file_name);
			return strtolower(array_pop($ar));
		}
	}

	
	/**
	 * 
	 */
	function gs($string)
	{
		return phpversion() > 5.2 ? htmlspecialchars($string, ENT_QUOTES, 'UTF-8', false)  :htmlspecialchars($string, ENT_QUOTES, 'UTF-8');
	}
	
	/**
	 * 
	 * @param $value
	 * @return unknown_type
	 */
	function isEmail($value)
	{
        $pattern = "/^([a-zA-Z0-9])+([\.a-zA-Z0-9_-])*@([a-zA-Z0-9_-])+(\.[a-zA-Z0-9_-]+)+/";
        return preg_match($pattern, $value);
    }
    
    define("VALIDATE_NOT_EMPTY", 1);
    define("VALIDATE_ALPHA", 2); 	//only a-zA-Z
    define("VALIDATE_NUMERIC", 4); 	//1, 0.123, etc
    define("VALIDATE_DECIMAL", 8); 	//1,2,3,4
    define("VALIDATE_FLOAT", 16); 	//the same as numeric
    define("VALIDATE_STRING", 32); 	//anything not empty
    define("VALIDATE_NAME", 64); 	//a-zA-Z,-
    define("VALIDATE_EMAIL", 128); 	//
    define("VALIDATE_URL", 256); 	//
    define("VALIDATE_ZIP", 512);
	define("VALIDATE_ZIP_USA", 1024); //12345 or 12345-1234
	define("VALIDATE_ZIP_CANADA", 2048);
	define("VALIDATE_ZIP_UK", 4096);
	
	define("VALIDATE_USERNAME", 8192);
	define("VALIDATE_PASSWORD", 16384);
	
	define("VALIDATE_PHONE", 32768);
	
    function validate($value, $type)
    {
    	$trimmed = trim($value);
    	if (($type & VALIDATE_NOT_EMPTY) && $trimmed == "") return false;
    	if (($type & VALIDATE_ALPHA) && !preg_match('/^([a-zA-Z])+$/', $value)) return false;
    	if (($type & VALIDATE_NUMERIC) && !preg_match('/^([0-9])+(\.([0-9])+)?$/', $value)) return false;
    	if (($type & VALIDATE_DECIMAL) && !preg_match('/^([0-9])+$/', $value)) return false;
    	if (($type & VALIDATE_FLOAT) && !preg_match('/^([0-9])+(\.([0-9])+)?$/', $value)) return false;
    	if (($type & VALIDATE_STRING) && $trimmed == "") return false;
		if (($type & VALIDATE_NAME) && ($trimmed == ""  || preg_match('/([0-9])+/', $value))) return false;
		//if (($type & VALIDATE_NAME) && !preg_match('/^(\pL)([\s\-\.\pL]{0,})/u', $value)) return false;
		//if (($type & VALIDATE_NAME) && !preg_match('/^([^\~\`\!\@\#\$\%\^\&\*\(\)\_\+\=\{\[\}\]\|\\\:\;\<\,\>\?\/0-9])+$/', $value)) return false;
		if (($type & VALIDATE_EMAIL) && !preg_match('/^([a-zA-Z0-9])+([\.a-zA-Z0-9_-])*@([a-zA-Z0-9_-])+(\.[a-zA-Z0-9_-]+)+/', $value)) return false;
    	if (($type & VALIDATE_URL) && !preg_match('/^((https?)\:\/\/)?([a-z0-9+!*(),;?&=\$_.-]+(\:[a-z0-9+!*(),;?&=\$_.-]+)?@)?([a-z0-9-.]*)\.([a-z]{2,3})(\:[0-9]{2,5})?(\/([a-z0-9+\$_-]\.?)+)*\/?(\?[a-z+&\$_.-][a-z0-9;:@&%=+\/\$_.-]*)?(#[a-z_.-][a-z0-9+\$_.-]*)?$/', $value)) return false;
    	if (($type & VALIDATE_ZIP) && !preg_match('/^([a-zA-Z0-9])+([a-zA-Z0-9 \-])+$/', $value)) return false;
    	if (($type & VALIDATE_ZIP_USA) && !preg_match('/^\d{5}$|^\d{5}-\d{4}$/', $value)) return false;
    	if (($type & VALIDATE_ZIP_CANADA) && !preg_match('/^([a-ceghj-npr-tv-z]){1}[0-9]{1}[a-ceghj-npr-tv-z]{1}[0-9]{1}[a-ceghj-npr-tv-z]{1}[0-9]{1}$/', strtolower(str_replace(" ", "", $value)))) return false;
    	if (($type & VALIDATE_ZIP_UK) && !preg_match('/^([A-PR-UWYZ0-9][A-HK-Y0-9][AEHMNPRTVXY0-9]?[ABEHMNPRVWXY0-9]? {1,2}[0-9][ABD-HJLN-UW-Z]{2}|GIR 0AA)$/', $value)) return false;
    	if (($type & VALIDATE_USERNAME) && !preg_match('/^([a-zA-Z0-9-_@.]){4,}$/', $value)) return false;
    	if (($type & VALIDATE_PASSWORD) && ($trimmed == "" || strlen($trimmed) < 6)) return false;
    	if (($type & VALIDATE_PHONE) && ($trimmed == "" || strlen($trimmed) < 1)) return false;
    	return true;
    }
    
    /**
     * Returns hashed password
     * @param $password string
     * @return string
     */
  	function getPasswordHash($password)
  	{
  		return sha1($password);
  	}
    
    /**
     * 
     * @param $pg
     * @param $pages_count
     * @param $pages_in_list
     * @param $url_prefix
     * @param $rewrite
     * @return unknown_type
     */
	function format_pages_line($pg, $pages_count, $pages_in_list, $url_prefix, $rewrite = "NO")
	{
		//global PAGES_IN_LIST;
		$pg = $pg - 1;
		$min = $pg - ($pg % $pages_in_list);
		$min ++;
		$max = $min + $pages_in_list - 1;
		$max = ($max > $pages_count)?$pages_count:$max;
		$t = '';
		$pg++;
		if ($pg>$pages_in_list)
		{
			if ($rewrite == "NO")
			{
				$t.='&nbsp;<span class="pageNavItem"><a href="'.$url_prefix.'&amp;pg=1">First</a></span>';
			}
			else
			{
				$t.='&nbsp;<span class="pageNavItem"><a href="'.str_replace("%n%", 1, $url_prefix).'">First</a></span>';
			}
			$low_limit = $min - $pages_in_list;
			
			if ($rewrite == "NO")
			{
				$t.=
					'&nbsp; <span class="pageNavItem"><a href="'.$url_prefix.'&amp;pg='.($min-1).'">'.
					$low_limit.'-'.($low_limit + $pages_in_list - 1).'</a></span>';
			}
			else
			{
				$t.=
					'&nbsp; <span class="pageNavItem"><a href="'.str_replace("%n%", ($min-1), $url_prefix).'">'.
					$low_limit.'-'.($low_limit + $pages_in_list - 1).'</a></span>';
			}
		}
		
		for ($i=$min; $i<=$max; $i++)
		{
			if ($i != $pg)
			{
				if ($rewrite == "NO")
				{
					$t.= 
						'&nbsp; <span class="pageNavItem"><a href="'.$url_prefix.'&amp;pg='.$i.'">'.
						$i.'</a></span>';
				}
				else
				{
					$t.= 
						'&nbsp; <span class="pageNavItem"><a href="'.str_replace("%n%", $i, $url_prefix).'">'.
						$i.'</a></span>';
				}
			}
			else
			{
				$t.= 
					'&nbsp; <span class="pageNavItemActive">'.$i.'</span>';
			}
		}
		
		if ($pages_count > $max)
		{
			$hi_limit = ($pages_count < ($max + $pages_in_list))?$pages_count:($max + $pages_in_list);
			if ($rewrite == "NO")
			{
				$t.=
					'&nbsp; <span class="pageNavItem"><a href="'.$url_prefix.'&amp;pg='.($max+1).'">'.(($max+1==$hi_limit)?"":(($max+1).'-')).$hi_limit.'</a></span>';
			}
			else
			{
				$t.=
					'&nbsp; <span class="pageNavItem"><a href="'.str_replace("%n%", ($max+1), $url_prefix).'">'.(($max+1==$hi_limit)?"":(($max+1).'-')).$hi_limit.'</a></span>';
			}
			if ($rewrite == "NO")
			{
				$t.='&nbsp; <span class="pageNavItem"><a href="'.$url_prefix.'&amp;pg='.($pages_count).'">Last</a></span>';
			}
			else
			{
				$t.='&nbsp; <span class="pageNavItem"><a href="'.str_replace("%n%", $pages_count, $url_prefix).'">Last</a></span>';
			}
		}
		return $t;
	}
	
	
	function paginate($pg, $pages_count, $pages_in_list, $url_prefix, $rewrite = false, $rewrite_first_page = false)
	{
		//global PAGES_IN_LIST;
		$pg = $pg - 1;
		$min = $pg - ($pg % $pages_in_list);
		$min ++;
		$max = $min + $pages_in_list - 1;
		$max = ($max > $pages_count)?$pages_count:$max;
		$t = array("pages"=>array(), "first"=>false, "range_left"=>false, "prev"=>false, "next"=>false, "range_right"=>false, "last"=>false);
		$pg++;
		
		if ($pg > $pages_in_list)
		{
			$t["first"]["url"] = (!$rewrite) ? $url_prefix.'&amp;pg=1' : ($rewrite_first_page ? $rewrite_first_page : str_replace("%n%", 1, $url_prefix));	
			$low_limit = $min - $pages_in_list;
			$t["range_left"] = 
				(!$rewrite) ?
				array("url" => $url_prefix.'&amp;pg='.($min-1), "text" => $low_limit.'-'.($low_limit + $pages_in_list - 1)) :
				array("url" => str_replace("%n%", ($min-1), $url_prefix), "text" => $low_limit.'-'.($low_limit + $pages_in_list - 1));
		}
		
		for ($i=$min; $i<=$max; $i++)
		{
			if ($i != $pg)
			{
				$t["pages"][] = 
					(!$rewrite) ? 
					array("url"=>$url_prefix.'&amp;pg='.$i, "text"=>$i) :
					array("url"=>(($i == 1 && $rewrite_first_page) ? $rewrite_first_page : str_replace("%n%", $i, $url_prefix)), "text"=>$i);
			}
			else
			{
				$t["pages"][] = array(
					"active"=>true,
					"text" => $i
				);
			}
		}
		
		if ($pages_count > $max)
		{
			$hi_limit = ($pages_count < ($max + $pages_in_list))?$pages_count:($max + $pages_in_list);
			$t["range_right"] = 
				(!$rewrite) ? 
				array("url"=>$url_prefix.'&amp;pg='.($max+1), "text"=>(($max+1==$hi_limit)?"":(($max+1).'-')).$hi_limit) :
				array("url"=>str_replace("%n%", ($max+1), $url_prefix), "text"=>(($max+1==$hi_limit)?"":(($max+1).'-')).$hi_limit);
			
			$t["last"]["url"] = (!$rewrite) ? $url_prefix.'&amp;pg='.($pages_count) : str_replace("%n%", $pages_count, $url_prefix);
		}
		
	
		$page_prev = $pg > 1 ? $pg - 1 : false;
		$page_next = $pg < $pages_count ? $pg + 1 : false;
		
		if ($page_prev)
		{
			$t["prev"] = 
				(!$rewrite) ?
				array("url"=>$url_prefix."&amp;pg=".$page_prev) :
				array("url"=>($page_prev == 1 && $rewrite_first_page) ? $rewrite_first_page : str_replace("%n%", $page_prev , $url_prefix));
		}
		
		if ($page_next)
		{
			$t["next"] = 
				(!$rewrite) ?
				array("url"=>$url_prefix."&amp;pg=".$page_next) :
				array("url"=>str_replace("%n%", $page_next , $url_prefix));
		}
		
		
		return $t;
	}
	
	/**
	 * 
	 * @param $db
	 * @param $stid
	 * @return unknown_type
	 */
	function getState($db, $stid)
	{
		if ($stid < 1) return NULL;
		$db->query("SELECT * FROM ".DB_PREFIX."states WHERE stid='".intval($stid)."'");
		if ($db->moveNext())
		{
			return $db->col["name"];
		}
		return NULL;
	}
	
	function getStateData($db, $stid = false, $stcode = false)
	{
		if ($stid === false && $stcode === false) return NULL;
		
		if ($stid)
			$ret = $db->selectOne("SELECT * FROM ".DB_PREFIX."states WHERE stid='".intval($stid)."'");
		else
			$ret = $db->selectOne("SELECT * FROM ".DB_PREFIX."states WHERE short_name='".$db->escape($stcode)."'");

		if ($ret !== false)
			return $ret;

		return NULL;
	}

	/**
	 * 
	 * @param $db
	 * @param $short_name
	 * @return unknown_type
	 */
	function getStateId($db, $short_name)
	{
		//if($stid < 1)
		//	return NULL;
		$db->query("SELECT * FROM ".DB_PREFIX."states WHERE short_name ='".trim($short_name )."'");
		if ($db->moveNext())
		{
			return $db->col["stid"];
		}
		return NULL;
	}

	/**
	 * 
	 * @param $db
	 * @param $coid
	 * @return unknown_type
	 */
	function getCountry($db, $coid){
		if($coid < 1)
			return NULL;
		$db->query("SELECT * FROM ".DB_PREFIX."countries WHERE coid='".intval($coid)."'");
		if($db->moveNext()){
			return $db->col["name"];
		}
		return NULL;
	}
	
	function getCountryData($db, $id = false, $code = false)
	{
		if ($id === false && $code === false) return NULL;
		
		if ($id)
			$ret = $db->selectOne("SELECT * FROM ".DB_PREFIX."countries WHERE coid='".intval($id)."'");
		else
			$ret = $db->selectOne("SELECT * FROM ".DB_PREFIX."countries WHERE iso_a2='".$db->escape($code)."' OR iso_a3='".$db->escape($code)."'");

		if ($ret !== false)
			return $ret;

		return NULL;
	}
	
	function loadJS($filename, $path = null)
	{
		if (is_null($path)) $path = 'content/admin/javascript/';
		
		$filename .= (strstr($filename, '?'))
			? '&'.APP_VERSION
			: '?'.APP_VERSION;
		
		echo "\r\n";
		echo '<script type="text/javascript" language="JavaScript" src="'.$path.$filename.'"></script>';
		echo "\r\n";
	}
	
	function pageError($errorMsg, $errors = array()) 
	{
		$ret = '';
		$ret = '<div>'.$errorMsg.'</div>';
		foreach ($errors as $err)
			$ret .= '<div>-&nbsp;'.$err.'</div>';
		
		return $ret;
	}
	
	function pageHeader($title, $extra = null, $class = null)
	{
		adminLog("Page: ".$title);
		
		echo '<h1 class="admin-page-header clearfix"><span class="icon icb-icon '.(!is_null($class) ? $class : '').'"></span><span class="text"><strong>'.$title.'</strong>'.(!is_null($extra) && $extra != '' ? ' ('.$extra.')' : '').'</span></h1>';
		
		return;		
	}
	
	function pageNote($data, $top_space = true)
	{
		echo '<ul class="pageNote admin-page-note">';
		foreach ($data as $key=>$note)
		{
			$note["visible"] = array_key_exists("visible", $note) ? $note["visible"] : true;
			
			if ($note["visible"])
			{
				
				echo '<li>';
				
				if (array_key_exists("class", $note) && trim($note["class"]) != "")
				{
					echo '<span class="icon ic-icon '.$note["class"].'">&nbsp;</span>';
				}
				
				if (array_key_exists("url", $note) && trim($note["url"]) != "")
				{
					echo '<a href="'.$note["url"].'">'.$note["text"].'</a>';
				}
				else
				{
					echo $note["text"];
				}
				echo '</li>';
			}	
		}
		echo '</ul>';
	}
	
	/**
	 * Tabs
	 */
	function pageTabs($data, $tabs = 'tabs', $active = false, $return = false)
	{
		$response = '<div class="clearfix admin-tabs-items">
			<ul class="clearfix">';
		$first = true;
		foreach ($data as $key=>$tab)
		{
			if (!$active) $active = $tab["id"];
			$tab["visible"] = isset($tab["visible"]) ? $tab["visible"] : true;
			$tab["active"] = $tab["id"] == $active;
			if ($tab["visible"])
			{
				$response .= '<li class="admin-tab '.($first?'first':'').' '.($tab["active"]?"admin-tab-active":"").'" onClick="setActiveTab(\''.$tabs.'\', \''.$tab["id"].'\')" id="admin-tab-'.$tab["id"].'"><span>'.$tab["title"].'</span></li>';
				$first = false;
			}
		}
		$response .= '</ul>
			</div>
			<script type="text/javascript">$(document).ready(function(){setActiveTab(\''.$tabs.'\', \''.gs($active).'\')});</script>';
			
		if ($return) return $response;
		
		echo $response;
	}
	
	function pageWhat($data)
	{
		echo '<div class="admin-page-what">';
		echo 'What do you want to do now?<ul>';
		foreach ($data as $key=>$note)
		{
			if (isset($note["visible"]) ? $note["visible"] : true)
			{
				echo "<li><a href=\"".$note["url"]."\">".$note["text"]."</a></li>\r\n";
			}
		}
		echo '</ul></div>';
	}
	
	function normalizeSettingsName($ss, $caption = "")
	{
		if($caption != "") return $caption;
		
		$s = "";
		for ($i=0; $i<strlen($ss); $i++)
		{
			if (strtoupper($ss[$i]) == $ss[$i])
			{
				$s = $s." ".$ss[$i];
			}
			else
			{
				$s = $s.$ss[$i];
			}
		}
		return trim($s);
	}
	
	function shortSettingsName($ss){
		$s = "";
		$start=0;
		for($i=0; $i<strlen($ss); $i++){
			if(strtoupper($ss[$i]) == $ss[$i]){
				$start++;
				if($start > 1)
					$s = $s." ".$ss[$i];				
			}
			else{
				if($start > 1)
					$s = $s.$ss[$i];
			}
		}
		return trim($s);
	}
	
	//FIXME: do not use this function, replace with standard php
	function parseFileName($path, &$result)
	{
		$result["filename"] = basename($path);
		$result["path"] = dirname($path);
		$pos = false;
		$n = $result["filename"];
		for($i = strlen($n)-1; $i>=0; $i--){
			if($n[$i] == "."){
				$pos = $i;
				break;
			}
		}
		$result["name"] = $pos?substr($n, 0, $pos):"";
		$result["ext"] = $pos?substr($n, $pos, strlen($n)-$pos):"";
	}
	
	function parseSearchKeywords($search_str = '', &$objects)
	{
		define("ADVANCED_SEARCH_DEFAULT_OPERATOR", "and");	

		$search_str = trim(mb_strtolower($search_str, "UTF-8"));
		$pieces = preg_split('/[\s,]+/', $search_str);
		
		$objects = array();
		$tmpstring = '';
		$flag = '';

		for ($k=0; $k<count($pieces); $k++)
		{
			while (substr($pieces[$k], 0, 1) == '(')
			{
				$objects[] = '(';
				if (strlen($pieces[$k]) > 1)
				{
					$pieces[$k] = substr($pieces[$k], 1);
	        	}
				else
				{
        	  		$pieces[$k] = '';
        		}
			}

			$post_objects = array();

			while (substr($pieces[$k], -1) == ')')
			{
				$post_objects[] = ')';
				if (strlen($pieces[$k]) > 1)
				{
					$pieces[$k] = substr($pieces[$k], 0, -1);
				}
				else
				{
					$pieces[$k] = '';
				}
			}

			// Check individual words
			if ((substr($pieces[$k], -1) != '"') && (substr($pieces[$k], 0, 1) != '"'))
			{
				$objects[] = trim($pieces[$k]);
				for ($j=0; $j<count($post_objects); $j++)
				{
					$objects[] = $post_objects[$j];
				}
			} 
			else
			{
				/* 	This means that the $piece is either the beginning or the end of a string.
   					So, we'll slurp up the $pieces and stick them together until we get to the
   					end of the string or run out of pieces.
				*/

				// Add this word to the $tmpstring, starting the $tmpstring
				$tmpstring = trim(str_replace('"', ' ', $pieces[$k]));

				// Check for one possible exception to the rule. That there is a single quoted word.
				if (substr($pieces[$k], -1 ) == '"')
				{
				// Turn the flag off for future iterations
					$flag = 'off';

					$objects[] = trim($pieces[$k]);

					for ($j=0; $j<count($post_objects); $j++)
					{
						$objects[] = $post_objects[$j];
					}

					unset($tmpstring);

					// Stop looking for the end of the string and move onto the next word.
					continue;
				}

				// Otherwise, turn on the flag to indicate no quotes have been found attached to this word in the string.
				$flag = 'on';

				// Move on to the next word
				$k++;

				// Keep reading until the end of the string as long as the $flag is on
				while (($flag == 'on') && ($k < count($pieces)))
				{
					while (substr($pieces[$k], -1) == ')')
					{
						$post_objects[] = ')';
						if (strlen($pieces[$k]) > 1)
						{
        	      			$pieces[$k] = substr($pieces[$k], 0, -1);
						}
						else
						{
							$pieces[$k] = '';
						}
					}

					// If the word doesn't end in double quotes, append it to the $tmpstring.
					if (substr($pieces[$k], -1) != '"')
					{
						// Tack this word onto the current string entity
						$tmpstring .= ' ' . $pieces[$k];
						// Move on to the next word
						$k++;
						continue;
					}
					else
					{
						/* 	If the $piece ends in double quotes, strip the double quotes, tack the
   							$piece onto the tail of the string, push the $tmpstring onto the $haves,
   							kill the $tmpstring, turn the $flag "off", and return.
						*/
						$tmpstring .= ' ' . trim(str_replace('"', ' ', $pieces[$k]));

						// Push the $tmpstring onto the array of stuff to search for
						$objects[] = trim($tmpstring);

						for ($j=0; $j<count($post_objects); $j++)
						{
							$objects[] = $post_objects[$j];
						}

						unset($tmpstring);

						// Turn off the flag to exit the loop
						$flag = 'off';
					}
				}
			}
	    }

		// add default logical operators if needed
    	$temp = array();
    	for ($i=0; $i<(count($objects)-1); $i++)
    	{
			$temp[] = $objects[$i];
			if (($objects[$i] != 'and') &&
				($objects[$i] != 'or') &&
				($objects[$i] != '(') &&
				($objects[$i+1] != 'and') &&
				($objects[$i+1] != 'or') &&
				($objects[$i+1] != ')') ){
				$temp[] = ADVANCED_SEARCH_DEFAULT_OPERATOR;
			}
		}
		if (array_key_exists($i, $objects))
		{
			$temp[] = $objects[$i];
		}
		else
		{
			$temp[] = "";
		}
		$objects = $temp;

		$keyword_count = 0;
		$operator_count = 0;
		$balance = 0;
		for ($i=0; $i<count($objects); $i++)
		{
			if ($objects[$i] == '(') $balance --;
			if ($objects[$i] == ')') $balance ++;
			if (($objects[$i] == 'and') || ($objects[$i] == 'or'))
			{
				$operator_count ++;
			}
			elseif ( ($objects[$i] || $objects[$i] == "0") && ($objects[$i] != '(') && ($objects[$i] != ')') )
			{
				$keyword_count ++;
			}
		}

		if (($operator_count < $keyword_count) && ($balance == 0))
		{
			return true;
		}
		else
		{
			return false;
		}
  	}

	/**
	 * get state short_name from Zip
	 * @param string $zip5
	 * @return string
	 */
	function validstatezip($zip5)
	{
	   $allstates = array (
		 "AK" => array ("9950099929"),
		 "AL" => array ("3500036999"),
		 "AR" => array ("7160072999", "7550275505"),
		 "AZ" => array ("8500086599"),
		 "CA" => array ("9000096199"),
		 "CO" => array ("8000081699"),
		 "CT" => array ("0600006999"),
		 "DC" => array ("2000020099", "2020020599"),
		 "DE" => array ("1970019999"),
		 "FL" => array ("3200033999", "3410034999"),
		 "GA" => array ("3000031999"),
		 "HI" => array ("9670096798", "9680096899"),
		 "IA" => array ("5000052999"),
		 "ID" => array ("8320083899"),
		 "IL" => array ("6000062999"),
		 "IN" => array ("4600047999"),
		 "KS" => array ("6600067999"),
		 "KY" => array ("4000042799", "4527545275"),
		 "LA" => array ("7000071499", "7174971749"),
		 "MA" => array ("0100002799"),
		 "MD" => array ("2033120331", "2060021999"),
		 "ME" => array ("0380103801", "0380403804", "0390004999"),
		 "MI" => array ("4800049999"),
		 "MN" => array ("5500056799"),
		 "MO" => array ("6300065899"),
		 "MS" => array ("3860039799"),
		 "MT" => array ("5900059999"),
		 "NC" => array ("2700028999"),
		 "ND" => array ("5800058899"),
		 "NE" => array ("6800069399"),
		 "NH" => array ("0300003803", "0380903899"),
		 "NJ" => array ("0700008999"),
		 "NM" => array ("8700088499"),
		 "NV" => array ("8900089899"),
		 "NY" => array ("0040000599", "0639006390", "0900014999"),
		 "OH" => array ("4300045999"),
		 "OK" => array ("7300073199", "7340074999"),
		 "OR" => array ("9700097999"),
		 "PA" => array ("1500019699"),
		 "RI" => array ("0280002999", "0637906379"),
		 "SC" => array ("2900029999"),
		 "SD" => array ("5700057799"),
		 "TN" => array ("3700038599", "7239572395"),
		 "TX" => array ("7330073399", "7394973949", "7500079999", "8850188599"),
		 "UT" => array ("8400084799"),
		 "VA" => array ("2010520199", "2030120301", "2037020370", "2200024699"),
		 "VT" => array ("0500005999"),
		 "WA" => array ("9800099499"),
		 "WI" => array ("4993649936", "5300054999"),
		 "WV" => array ("2470026899"),
		 "WY" => array ("8200083199"));
	
	// if you use a drop down list for state selection, ensuring valid data,
	// isset is not needed.  (Warnings can not be turned off with: @foreach...)
	
	   foreach($allstates as $state=>$zipr){
		  foreach($allstates[$state] as $ziprange){
				if (($zip5 >= substr($ziprange, 0, 5)) && ($zip5 <= substr($ziprange,5))){
			   		$valid = "TRUE";
			   		return ($state);  // on match, jump out of foreach early :)
			 	}
			}
	   }
	   return(false);
	}
	
	
	/**
	 * Cookie Wrapper for better cookie management towards PCI compliancy
	 *
	 * @return object Ddm_Cookie
	 */
	function initCookies()
	{
		require_once('content/classes/class.ddm_cookie.php');
		static $_id = null;
		
		if (is_null($_id))
		{
			$_id = new Ddm_Cookie(array(
				'domain' => '',
				'path' => '/',
				'expires' => time()+3600*24*365,
				'secure' => isSslMode()
			));
		}
		
		return $_id;
	}
	
	/**
	 * Checks whether the page was requested via HTTPS mode
	 * @return bool
	 * @author Sebastian Nievas
	 */
	function isSslMode()
	{
		static $mode = null;
		
		if (is_null($mode))
		{
			$mode = (array_key_exists('HTTPS', $_SERVER) && !empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off' && $_SERVER['SERVER_PORT'] == 443);
		}
		
		return $mode;
	}
	
	/**
	 * Checks password strength
	 * Password have to be at least 8 chars long, and contain lower and upper case letters, and numbers
	 * @param string $password
	 * @return boolean
	 */
	function pwdStrength($password)
	{
		$points = 0;
		
		// 1 point for length over 6
		if (strlen($password) >= 8) $points++;

		// 1 lower & uppercase characters
		if (preg_match("/[a-z]/", $password) && preg_match("/[A-Z]/", $password)) $points++;

		// 1 point for a number
		if (preg_match("/\d+/", $password)) $points++;
	
		// 1 point for special character
		if (preg_match("/.[!,@,#,$,%,^,&,*,?,_,~,-,(,)]/", $password)) $points++;

		// 1 point for length over 12
		if (strlen($password) > 12) $points++;
	
		return $points >= 3;
	}
	
	/**
	 * Fill in white-space string with random chars from charset
	 * @param string $password
	 * @param string $charset
	 * @param int $count
	 */
	function _generataPassword(&$password, $charset, $count)
	{
		$_count = 0;
		while (true)
		{
			$p = round(rand(0, strlen($password) - 1));
			if ($password[$p] == " ")
			{
				$password[$p] = $charset[round(rand(0, strlen($charset) - 1))];
				$_count++;
			}	
			if ($_count >= $count) break;
		}
	}
 
	/**
	 * Generate random password
	 * @return string
	 */
	function generatePassword()
	{
		$length = 12;
		$password = str_pad("", $length, " ");
		$capital_count = round(rand(3,4));
		$capital_list = "ZAQWSXCDERFVBGTYHNMJUIKLOP";
		$lowercase_count = round(rand(5,6));
		$lowercase_list = strtolower($capital_list);
		$number_count = round(rand(1, $number_count = $length - $capital_count - $lowercase_count - 1));
		$number_list = "1425369708";
		$spec_count = $length - $capital_count - $lowercase_count - $number_count;
		$spec_list = "!@#%^&*?_~-()";
		_generataPassword($password, $capital_list, $capital_count);
		_generataPassword($password, $lowercase_list, $lowercase_count);
		_generataPassword($password, $number_list, $number_count);
		_generataPassword($password, $spec_list, $spec_count);
		return $password;
	}
	
	/**
	 * Returns database instance
	 * @return unknown_type
	 */
	function db()
	{
		global $db;
		return $db;
	}
	
	/**
	 * Returns settings instance
	 * @return unknown_type
	 */
	function settings()
	{
		static $_id = null;
		if (is_null($_id)) global $settings;
		return $settings;
	}
	
	/**
	 * 
	 * @return Ddm_View
	 */
	function view()
	{
		static $_id = null;
		if (is_null($_id)) $_id = Ddm_View::getInstance();
		return $_id;
	}
	
	/**
	 * Autoload function
	 * @param string $class_name
	 */
	function ddm_autoload_lowercase($class_name)
	{
		$paths = array(
			'content/classes/',
			'content/vendors/',
			'content/'
		);
		
		foreach ($paths as $path)
		{
			$file1 = strtolower(str_replace('_', '/', $class_name)).'.php';
			$file2 = 'class.'.strtolower($class_name).'.php';
			
			if (file_exists($path.$file1))
			{
				require_once($path.$file1);
				return;
			}
			elseif (file_exists($path.$file2))
			{
				require_once($path.$file2);
				return;
			}
		}
	}
	spl_autoload_register('ddm_autoload_lowercase');

	function ddm_autoload($class_name)
	{
		$paths = array(
			'content/classes/',
			'content/vendors/',
		);

		foreach ($paths as $path)
		{
			$file1 = str_replace('_', '/', $class_name).'.php';

			if (file_exists($path.$file1))
			{
				require_once($path.$file1);
				return;
			}
		}
	}
	spl_autoload_register('ddm_autoload');

	/**
	 * Removes directory recursivelly 
	 * @param string $path
	 */
	function remove_dir($path)
	{
		$d = @dir($path);
		if ($d)
		{
			while (false !== ($entry = $d->read()))
			{
				if (!in_array($entry, array(".", "..")))
				{
					if (is_file($path.DIRECTORY_SEPARATOR.$entry))
					{
						@unlink($path.DIRECTORY_SEPARATOR.$entry);
					}
					if (is_dir($path.DIRECTORY_SEPARATOR.$entry))
					{
						remove_dir($path.DIRECTORY_SEPARATOR.$entry);
					}
				}
			}
			$d->close();
		}

		@rmdir($path);
	}

	/**
	 * Copies directory
	 * @param string $source
	 * @param string $destination
	 * @param array $extend_options Array of options to extend
	 */
	function copy_dir($source, $destination, $extend_options = array())
	{
		if (strlen($source) > 0)
		{
			if (!in_array($source[strlen($source) - 1], array("/", "\\")))
			{
				$source = $source.DS;
			}
		}
		
		if (strlen($destination) > 0)
		{
			if (!in_array($destination[strlen($destination) - 1], array("/", "\\")))
			{
				$destination = $destination.DS;
			}
		}	
		
		$default_options = array(
			'include_folders' => null,
			'exclude_folders' => null,
			'include_extensions' => null,
			'exclude_extensions' => null,
			'overwrite' => false,
			'file_permission' => getFilePermissionMode(),
			'dir_permission' => getDirectoryPermissionMode()
		);
		
		$options = array_merge($default_options, $extend_options);
			
		$d = dir($source);
		if ($d)
		{
			while (false !== ($entry = $d->read()))
			{
				if (!in_array($entry, array(".", "..", ".svn")))
				{	
					$_source = $source.$entry;
					$_destination = $destination.$entry;
					
					if (is_file($_source))
					{
						if (!is_file($_destination))
						{
							copy($_source, $_destination);
						}
						elseif ($options['overwrite'])
						{
							unlink($_destination);
							copy($_source, $_destination);
						}
						
						if ($options['file_permission'] != null)
							chmod($_destination, $options['file_permission']);
					}
					elseif (is_dir($_source))
					{
						if (!is_dir($_destination))
						{
							File_Utility::setDirectoryPermissions($_destination, $options['dir_permission']);
						}
						copy_dir($_source, $_destination, $extend_options);
					}
				}
			}
			$d->close();
		}
	}
	
	function getp(){return base64_decode('cGlubmFjbGVjYXJ0');}
	function getpp($space = false)
	{
		if ($space)
		{
			return base64_decode('UGlubmFjbGUgQ2FydA==');
		}
		return base64_decode('UGlubmFjbGVDYXJ0');
	}
		
	/**
	 * Traverses directories
	 * @param string $starting_directory 
	 * @return void
	 */
	function traverse($starting_directory, $reset = false)
	{ 
		static $files = array();
		
		if ($reset) $files = array();

		if (($d_obj = dir($starting_directory)) != false)
		{ 
			while (($this_entry = $d_obj->read()) != false)
			{ 
				if ($this_entry != "." && $this_entry != ".." && $this_entry != '.svn')
				{
					if (is_dir("$starting_directory/$this_entry"))
					{ 
						traverse("$starting_directory/$this_entry"); 
					}
					else
					{ 
						$files[] = "$starting_directory/$this_entry";
					}
				} 
			}
			$d_obj->close();
		}
		
		return $files;
	}
	
	/**
	 * Checks if a button was clicked (including input images)
	 * @param string $button The name of the button
	 * @return bool
	 */
	function wasClicked($button)
	{
		if (isset($_GET[$button]) || isset($_GET[$button.'_x']) || isset($_GET[$button.'_y'])) return true;
		if (isset($_POST[$button]) || isset($_POST[$button.'_x']) || isset($_POST[$button.'_y'])) return true;
		return false;
	}
	
	/**
	 * Returns the literal end of line (EOL) character set in the admin area to use for mail headers.
	 *
	 * @return string The end of line literal as set in the admin area
	 * @author Sebastian Nievas
	 */
	function getMailHeaderEolCharacter()
	{
		$settings = settings();
		
		switch ($settings['EmailSendmailEolCharacter'])
		{
			case '\r': $eol_character = "\r"; break;
			case '\n': $eol_character = "\n"; break;
			default: $eol_character = "\r\n"; break;
		}
		
		return $eol_character;
	}
	
	/**
	 * Returns the set server directory permission mode
	 *
	 * @return int The directory permission mode
	 */
	function getDirectoryPermissionMode()
	{
		if (defined('DIRECTORY_PERMISSION_MODE')) return DIRECTORY_PERMISSION_MODE;
		
		$settings = settings();
		return octdec($settings['DirectoryPermissionMode']);
	}
	
	/**
	 * Returns the set server file permission mode
	 *
	 * @return int The file permission mode
	 */
	function getFilePermissionMode()
	{
		if (defined('FILE_PERMISSION_MODE')) return FILE_PERMISSION_MODE;
		
		$settings = settings();
		return octdec($settings['FilePermissionMode']);
	}
	
	function getApplicationDefaultTimezone()
	{
		return date_default_timezone_get();
	}
	
	/**
	 * Sets the default timezone to use in PHP and MySQL during runtime of the application
	 *
	 * @param string $db Database connection to set timezone setting
	 * @param string $settings The carts settings, namely LocalizationUseSystemTimezone and LocalizationUseSystemTimezone
	 * @return string Timezone identifier
	 */
	function setApplicationDefaultTimezone(DB $db, $settings)
	{
		if (!isset($settings['LocalizationUseSystemTimezone']) || $settings['LocalizationUseSystemTimezone'] == 'Yes') return date_default_timezone_get();
		
		$timezone = $settings['LocalizationDefaultTimezone'];
		
		// Store the current timezone set on the server
		$current_timezone = date_default_timezone_get();
		
		// Validate the timezone setting
		if (!is_string($timezone) || ($timezone != '' && !date_default_timezone_set($timezone)))
		{
			throw new Exception('Invalid timezone set');
		}
		
		// For PHP 5.0 compatibility, otherwise we can use the 'P' date format
		$offset = date('O');
		$offset = substr($offset, 0, 3).':'.substr($offset, 3);
		
		// Set the mysql timezone offset
		$db->query('SET time_zone="'.$offset.'"');
		
		return date_default_timezone_get();
	}
	
	/**
	 * Loads the cart's config file
	 * 
	 * If the config file cannot be located, it will attempt to redirect the user to the installation directory, otherwise it will fail
	 */
	function loadApplicationConfig()
	{
		if (!file_exists('content/engine/engine_config.php'))
		{
			// Check to see if there's an install folder to redirect to
			if (file_exists('install/index.php') && is_file('install/index.php'))
			{
				header('Location: install/index.php');
				exit;
			}
			else
			{
				$error_message = 'Could not locate the engine config file. Ensure the file is in the content/engine/ folder.';
				error_log($error_message);
				throw new Exception($error_message);
			}
		}
		else
		{
			require_once('content/engine/engine_config.php');
		}
	}
	
	
