<?php 
/**
 * Users actions
 */
define("USER_SIGNUP", "user_signup");
define("USER_LOGIN", "user_login");
define("USER_LOGIN_FACEBOOK", "user_login_fb");
define("USER_LOGOUT", "user_logout");
define("USER_LOGOUT_FACEBOOK", "user_logout_fb");
define("USER_PASSWORD_RESET", "password_reset");
define("USER_UPDATE_PROFILE", "user_updateprofile");
define("USER_EXPRESS_CHECKOUT", "user_expresscheckout");
define("USER_START_CHECKOUT", "user_start_checkout");
define("USER_ADD_ADDRESS", "AddShippingAddress");
define("USER_DELETE_ADDRESS", "DeleteShippingAddress");
define("USER_UPDATE_ADDRESS", "UpdateShippingAddress");
define("USER_ADD_TO_WISHLIST", "UserAddToWishlist");
define("USER_FORM_MAIL", "FormMail");
define("USER_SET_COOKIE", "set_c");
define("USER_UNSET_COOKIE", "unset_c");
define("USER_ADD_REVIEW", "AddReview");
define("USER_ADD_PAYMENT_PROFILE", "AddPaymentProfile");
define("USER_DELETE_PAYMENT_PROFILE", "DeletePaymentProfile");
define("USER_UPDATE_PAYMENT_PROFILE", "UpdatePaymentProfile");
define("USER_QR", "qr");
define("USER_OFFSITE_COMMERCE", "osc");
define("EXPRESS_CHECKOUT_USER", "ExpressCheckoutUser");

/**
 * User class
 * @author Victor
 */
class USER
{
	public $id = 0;
	public $session_id = "";
	public $level = 0;
	public $fname = "";
	public $lname = "";
	public $login = "";
	public $auth_ok = false;
	public $data = array();
	public $messages = array();
	public $is_error = false;
	public $errors = array();
	public $blocked = false;
	public $cookie = "CartUserCookie";
	public $back_user = false;
	public $back_user_login = "";
	public $express = false;
	private $_db;
	private $_settings;
	private $_customFields = null;

	/**
	 * Class constructor
	 * @param $db
	 * @param $settings
	 * @return USER
	 */
	public function __construct(&$db, &$settings)
	{
		$this->_db = $db;
		$this->_settings = $settings;
			
		//set cookie prefix
		$this->cookie = $this->_settings["SecurityCookiesPrefix"];
			
		// init user, load cookies & session
		global $_COOKIE;
			
		// check returning customers
		$this->back_user = in_array($this->cookie."LoginUserName", array_keys($_COOKIE));
		$this->back_user_login = isset($_COOKIE[$this->cookie."LoginUserName"])?$_COOKIE[$this->cookie."LoginUserName"]:"";
			
		// check user session
		$this->id = isset($_SESSION["UserID"]) ? intval($_SESSION["UserID"]) : 0;
		$this->session_id = isset($_SESSION["SessionID"]) ? $_SESSION["SessionID"] : "";
			
		// check user registered (by user id)
		if ($this->id)
		{
			$this->db()->query("
				SELECT uid FROM ".DB_PREFIX."users 
				WHERE 
					uid='".intval($this->id)."' AND removed = 'No' AND 
					DATE_ADD(session_date, INTERVAL ".intval($this->_settings["SecurityUserTimeout"])." SECOND)>NOW() AND
					".($this->_settings["SecurityAccountBlocking"] == "YES" ? "block_date < NOW() AND" : "")."
					session_id='".$this->db()->escape($this->session_id)."'
			");
			
			if ($this->db()->moveNext())
			{
				// user is registered - user_id, session_id correct and not expired
				$this->auth_ok = true;
				$this->getUserData();
				
				// update session_timeout in db
				$this->db()->query("UPDATE ".DB_PREFIX."users SET session_date=NOW() WHERE uid='".intval($this->id)."'");
			}
			else
			{
				// user is not registered - can't use existing user session
				$this->id = 0;
				$this->session_id = uniqid(md5(rand(1, 1000000)));
			}
			
			// store user data to session
			$_SESSION["SessionID"] = $this->session_id;
			$_SESSION["UserID"] = $this->id;
		}
		
		return $this;
	}
	
	/**
	 * Sets error
	 * @param $message
	 * @return unknown_type
	 */
	public function setError($message)
	{
		if (is_array($message))
		{
			foreach($message as $m) $this->errors[] = $m;
		}
		else
		{
			$this->errors[] = $message;
		}
		$this->is_error = true;
	}
	
	/**
	 * Return database instance
	 * @return DB
	 */
	public function db()
	{
		return $this->_db;
	}

	/**
	 * Assign custom fields class
	 * @param $customFields
	 */
	public function customFields($customFields = false)
	{
		if ($customFields) $this->_customFields = $customFields;
		return $this->_customFields = $this->_customFields != null ? $this->_customFields : new CustomFields($this->_db);
	}
	
	/**
	 * Update / edit billing address on express checkout
	 * @param $address
	 */
	public function payPalExpressUserInfo($address)
	{
		//preselect country
		$this->db()->query("SELECT * FROM ".DB_PREFIX."countries WHERE iso_a2 LIKE '".$this->db()->escape($address["COUNTRYCODE"])."'");
		
		$country = $this->db()->moveNext() ? $this->db()->col["coid"] : 1;
		
		//preselect state
		$payPalState = isset($address["STATE"]) ? $address["STATE"] : (isset($address['SHIPTOSTATE']) ? $address['SHIPTOSTATE'] : '');
		$this->db()->query("SELECT * FROM ".DB_PREFIX."states WHERE coid='".$country."' AND (short_name LIKE '".$this->db()->escape($payPalState)."' OR name LIKE '".$this->db()->escape($payPalState)."')");
		
		if ($this->db()->moveNext())
		{
			$state = $this->db()->col["stid"];
			$province = $this->db()->col["name"];
		}
		else
		{
			$state = 0;
			$province = $payPalState;
		}
		
		$this->db()->reset();
		$this->db()->assignStr("fname", 
			trim(
				$address["FIRSTNAME"]
			)
		);
		$this->db()->assignStr("lname", $address["LASTNAME"]);
		
		$this->db()->assignStr("address1", isset($address["STREET"]) ? $address["STREET"] : (isset($address['SHIPTOSTREET']) ? $address['SHIPTOSTREET'] : ''));
		$this->db()->assignStr("address2", isset($address["STREET2"]) ?$address["STREET2"] : (isset($address['SHIPTOSTREET2']) ? $address['SHIPTOSTREET2'] : ''));
		$this->db()->assignStr("city", isset($address["CITY"]) ? $address["CITY"] : (isset($address['SHIPTOCITY']) ? $address['SHIPTOCITY'] : ''));
		$this->db()->assignStr("state", $state);
		$this->db()->assignStr("province", $province);
		$this->db()->assignStr("country", $country);
		$this->db()->assignStr("phone", isset($address["PHONENUM"]) ? $address["PHONENUM"] : '');
		$this->db()->assignStr("zip", isset($address["ZIP"]) ? $address["ZIP"] : isset($address['SHIPTOZIP']) ? $address['SHIPTOZIP'] : '');
		
		$this->db()->assignStr("email", $address["EMAIL"]);
		$this->db()->assign('last_update', 'NOW()');
		$this->db()->update(DB_PREFIX."users", "WHERE uid='".intval($this->id)."'");
	}
		
	/**
	 * Add address from PayPal to User's Address Book
	 * @param $paypal_address
	 * @return boolean
	 */
	public function payPal2Shipping($address)
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."users WHERE uid='".intval($this->id)."' AND removed='No'");
		if ($this->db()->moveNext())
		{
			//check primary
			$this->db()->query("SELECT * FROM ".DB_PREFIX."users_shipping WHERE uid='".intval($this->id)."' AND is_primary='Yes'");
			$is_primary = $this->db()->moveNext()?"No":"Yes";
			//get country id
			$paypalCountry = isset($address['SHIPTOCOUNTRY']) ? $address['SHIPTOCOUNTRY'] : (isset($address['SHIPTOCOUNTRYCODE']) ? $address['SHIPTOCOUNTRYCODE'] : '');

			$this->db()->query("SELECT * FROM ".DB_PREFIX."countries WHERE iso_a2 = '".$this->db()->escape($paypalCountry)."'");
			if ($this->db()->moveNext())
			{
				$country = $this->db()->col["coid"];

				$payPalState = $address["SHIPTOSTATE"];
				$this->db()->query("SELECT * FROM ".DB_PREFIX."states WHERE coid='".intval($country)."' AND (short_name LIKE '".$this->db()->escape($payPalState)."' OR name LIKE '".$this->db()->escape($payPalState)."')");
				if ($this->db()->moveNext())
				{
					$state = $this->db()->col["stid"];
					$province = $this->db()->col["name"];
				}
				else
				{
					$state = 0;
					$province = $address["SHIPTOSTATE"];
				}
			}
			else
			{
				$country = 1;
				$state = 0;
				$province = $address["SHIPTOSTATE"];
			}
			
			//check for duplicate
			$this->db()->query("
				SELECT usid
				FROM ".DB_PREFIX."users_shipping 
				WHERE
					uid = '".intval($this->id)."' AND
					country = '".intval($country)."' AND
					state = '".intval($state)."' AND
					name = '".$this->db()->escape($address['SHIPTONAME'])."' AND
					company = '".$this->db()->escape(isset($address["SHIPTOBUSINESS"]) ? $address["SHIPTOBUSINESS"] : '')."' AND
					address1 = '".$this->db()->escape($address['SHIPTOSTREET'])."' AND
					address2 = '".$this->db()->escape(isset($address["SHIPTOSTREET2"]) ? $address["SHIPTOSTREET2"] : '')."' AND
					city = '".$this->db()->escape($address['SHIPTOCITY'])."' AND
					province = '".$this->db()->escape($province)."' AND
					zip = '".$this->db()->escape($address["SHIPTOZIP"])."'
				ORDER BY usid DESC 
				LIMIT 1
			");
			
			if ($this->db()->moveNext())
			{
				$address_id = $this->db()->col['usid'];
			}
			else
			{
				$this->db()->reset();
				$this->db()->assign("uid", $this->id);
				$this->db()->assign("country", intval($country));
				$this->db()->assign("state", intval($state));
				$this->db()->assignStr("is_primary", $is_primary);
				$this->db()->assignStr("name", $address["SHIPTONAME"]);
				$this->db()->assignStr("company", isset($address["SHIPTOBUSINESS"]) ? $address["SHIPTOBUSINESS"] : '');
				$this->db()->assignStr("address1", $address["SHIPTOSTREET"]);
				$this->db()->assignStr("address2", isset($address["SHIPTOSTREET2"]) ? $address["SHIPTOSTREET2"] : '');
				$this->db()->assignStr("city", $address["SHIPTOCITY"]);
				$this->db()->assignStr("province", $province);
				$this->db()->assignStr("zip", $address["SHIPTOZIP"]);
				$address_id = $this->db()->insert(DB_PREFIX."users_shipping");
			}
			return $address_id;
		}
		return false;
	}

	/**
	 * Add shipping address to users account
	 * @param $address
	 * @return mixed
	 */
	public function addShippingAddress($address, $customFieldsData = false, $checkForDuplicate = false)
	{
		$this->is_error = false;
		$this->errors = array();
		
		if ($this->_settings["FormsShippingCompany"] == 'Required' && (!isset($address["company"]) || !validate($address["company"], VALIDATE_STRING))) $this->setError("Enter a valid company name");
		if ($this->_settings["FormsShippingAddressLine2"] == 'Required' && (!isset($address["address2"]) || !validate($address["address2"], VALIDATE_STRING))) $this->setError("Enter a valid address line 2");
		
		$this->validateAddress($address);
		
		//validate custom fields for shipping
		if (is_array($customFieldsData))
		{
			$this->customFields()->resetErrors();
			$this->customFields()->validateCustomFields("shipping", $customFieldsData);
			if ($this->customFields()->is_error) $this->setError($this->customFields()->errors);
		}
		
		if (!$this->is_error)
		{
			//check is there already shipping addresses for user
			$this->db()->query("SELECT * FROM ".DB_PREFIX."users_shipping WHERE uid='".intval($this->id)."'");
			if( !$this->db()->moveNext())
			{
				//this shipping address is first one
				//set it as primary
				$address["set_is_primary"] = "Yes";
			}
			else
			{
				//check if there attempt to set primary address
				if (isset($address["set_is_primary"]) && ($address["set_is_primary"] == "Yes"))
				{
					//reset all existing addresses to "not primary"
					$this->db()->reset();
					$this->db()->assignStr("is_primary", "No");
					$this->db()->update(DB_PREFIX."users_shipping", "WHERE uid='".intval($this->id)."'");
					//set current to primary
					$address["set_is_primary"] = "Yes";
				}
				else
				{
					//unset primary (this stuff for check bof control which does not pass data if not checked)
					$address["set_is_primary"] = "No";
				}
			}
			
			$duplicate = false;
			
			if ($checkForDuplicate)
			{
				//check for duplicate
				$this->db()->query("
					SELECT * 
					FROM ".DB_PREFIX."users_shipping 
					WHERE
						uid = '".intval($this->id)."' AND
						address_type = '".intval(isset($address["address_type"]) ? $address["address_type"] : 'Residential')."' AND
						country = '".intval($address["country"])."' AND
						state = '".intval($address["state"])."' AND
						name = '".$this->db()->escape($address['name'])."' AND
						company = '".$this->db()->escape($address['company'])."' AND
						address1 = '".$this->db()->escape($address['address1'])."' AND
						address2 = '".$this->db()->escape($address['address2'])."' AND
						city = '".$this->db()->escape($address['city'])."' AND
						province = '".$this->db()->escape($address['province'])."' AND
						zip = '".$this->db()->escape($address['zip'])."'
					ORDER BY usid DESC 
					LIMIT 1
				");
			
				if ($this->db()->moveNext())
				{
					$address_id = $this->db()->col['usid'];

					$duplicate = true;

					$currentCustomFieldsData = $this->customFields()->getCustomFieldsValues("shipping", $this->id, 0, $address_id);

					if (is_array($currentCustomFieldsData) && is_array($customFieldsData))
					{
						if (count($currentCustomFieldsData) == count($customFieldsData))
						{
							foreach ($currentCustomFieldsData as $cf)
							{
								if ($cf['value'] != $customFieldsData[$cf['field_id']]) $duplicate = false;
							}
						}
						else
						{
							$duplicate = false;
						}
					}
					else
					{
						if (is_array($currentCustomFieldsData) || (is_array($customFieldsData) && count($customFieldsData) > 0)) $duplicate = false;
					}
				}
			}
			
			if (!$duplicate)
			{
				//insert address values
				$this->db()->reset();
				$this->db()->assign("uid", intval($this->id));
				$this->db()->assignStr("address_type", isset($address["address_type"]) ? $address["address_type"] : 'Residential');
				$this->db()->assign("country", intval($address["country"]));
				$this->db()->assign("state", intval($address["state"]));
				$this->db()->assignStr("is_primary", $address["set_is_primary"]);
				$this->db()->assignStr("name", $address["name"]);
				$this->db()->assignStr("company", $address["company"]);
				$this->db()->assignStr("address1", $address["address1"]);
				$this->db()->assignStr("address2", $address["address2"]);
				$this->db()->assignStr("city", $address["city"]);
				$this->db()->assignStr("province", $address["province"]);
				$this->db()->assignStr("zip", $address["zip"]);
				$address_id = $this->db()->insert(DB_PREFIX."users_shipping");

				//store custom fields data
				if (is_array($customFieldsData))
				{
					$this->customFields()->storeCustomFields("shipping", $customFieldsData, $this->id, 0, $address_id);
				}
			}
			
			return $address_id;
		}
		return false;
	}
	
	/**
	 * Update shipping address
	 * @param $address_id
	 * @param $address
	 * @return boolean
	 */
	public function updateShippingAddress($address_id, $address, $customFieldsData = false)
	{
		$this->is_error = false;
		$this->errors = array();
		$this->validateAddress($address);
		
		if ($this->_settings["FormsShippingCompany"] == 'Required' && (!isset($address["company"]) || !validate($address["company"], VALIDATE_NOT_EMPTY))) $this->setError("Enter a valid company name");
		if ($this->_settings["FormsShippingAddressLine2"] == 'Required' && (!isset($address["address2"]) || !validate($address["address2"], VALIDATE_NOT_EMPTY))) $this->setError("Enter a valid address line 2");
		
		//validate custom fields for shipping
		if (is_array($customFieldsData))
		{
			$this->customFields()->resetErrors();
			$this->customFields()->validateCustomFields("shipping", $customFieldsData);
			if ($this->customFields()->is_error) $this->setError($this->customFields()->errors);
		}
		
		if (!$this->is_error)
		{
			//check is there already shipping addresses for user
			$this->db()->query("SELECT * FROM ".DB_PREFIX."users_shipping WHERE uid='".intval($this->id)."'");
			if (!$this->db()->moveNext())
			{
				//this shipping address is first one
				//set it as primary
				$address["set_is_primary"] = "Yes";
			}
			else
			{
				//check if there attempt to set primary address
				if (isset($address["set_is_primary"]) && ($address["set_is_primary"] == "Yes"))
				{
					//reset all existing addresses to "not primary"
					$this->db()->reset();
					$this->db()->assignStr("is_primary", "No");
					$this->db()->update(DB_PREFIX."users_shipping", "WHERE uid='".intval($this->id)."'");
					//set current to primary
					$address["set_is_primary"] = "Yes";
				}
				else
				{
					//unset primary (this stuff for check bof control which does not pass data if not checked)
					$address["set_is_primary"] = "No";
				}
			}
			
			//insert address values
			$this->db()->reset();
			$this->db()->assign("uid", intval($this->id));
			$this->db()->assignStr("address_type", isset($address["address_type"]) ? $address["address_type"] : 'Residential');
			$this->db()->assign("country", intval($address["country"]));
			$this->db()->assign("state", intval($address["state"]));
			$this->db()->assignStr("is_primary", $address["set_is_primary"]);
			$this->db()->assignStr("name", $address["name"]);
			$this->db()->assignStr("company", $address["company"]);
			$this->db()->assignStr("address1", $address["address1"]);
			$this->db()->assignStr("address2", $address["address2"]);
			$this->db()->assignStr("city", $address["city"]);
			$this->db()->assignStr("province", $address["province"]);
			$this->db()->assignStr("zip", $address["zip"]);
			$this->db()->update(DB_PREFIX."users_shipping", "WHERE uid='".intval($this->id)."' AND usid='".$address_id."'");
			
			//update custom shipping data
			if (is_array($customFieldsData))
			{
				$this->customFields()->storeCustomFields("shipping", $customFieldsData, $this->id, 0, $address_id);	
			}
			
			return true;
		}
		return false;
	}
		
	/**
	 * Remove shipping address from user's profile
	 * @param $id
	 * @return boolean
	 */
	public function deleteShippingAddress($address_id)
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."users_shipping WHERE uid='".intval($this->id)."' AND usid='".intval($address_id)."'");
		if ($this->db()->moveNext())
		{
			$this->db()->query("DELETE FROM ".DB_PREFIX."users_shipping WHERE uid='".intval($this->id)."' AND usid='".intval($address_id)."'");
			$this->db()->query("DELETE FROM ".DB_PREFIX."custom_fields_values WHERE profile_id='".intval($this->id)."' AND address_id='".intval($address_id)."'");
			return true;
		}
		return false;
	}
		
	/**
	 * Get Shipping Addresses
	 * @return array
	 */
	public function getShippingAddresses()
	{
		$this->db()->query("
			SELECT 
				".DB_PREFIX."users_shipping.*,
				".DB_PREFIX."countries.name AS country_name,
				".DB_PREFIX."states.name AS state_name
			FROM ".DB_PREFIX."users_shipping 
			LEFT JOIN ".DB_PREFIX."countries ON ".DB_PREFIX."countries.coid = ".DB_PREFIX."users_shipping.country
			LEFT JOIN ".DB_PREFIX."states ON ".DB_PREFIX."states.stid = ".DB_PREFIX."users_shipping.state
			WHERE uid='".intval($this->id)."'
			ORDER BY is_primary
		");
		return $this->db()->numRows()>0?$this->db()->getRecords():false; 
	}
	
	/**
	 * Get primary shipping address
	 */
	public function getShippingAdressPrimary()
	{
		$r = $this->db()->query("
			SELECT 
				".DB_PREFIX."users_shipping.*,
				".DB_PREFIX."countries.name AS country_name,
				".DB_PREFIX."states.name AS state_name
			FROM ".DB_PREFIX."users_shipping 
			LEFT JOIN ".DB_PREFIX."countries ON ".DB_PREFIX."countries.coid = ".DB_PREFIX."users_shipping.country
			LEFT JOIN ".DB_PREFIX."states ON ".DB_PREFIX."states.stid = ".DB_PREFIX."users_shipping.state
			WHERE uid='".intval($this->id)."' AND is_primary = 'Yes'
			ORDER BY is_primary
		");
		return $this->db()->numRows($r) > 0 ? $this->db()->moveNext($r, MYSQL_ASSOC) : false; 
	}
		
	/**
	 * Get Shipping Address by ID
	 * @param $address_id
	 * @return mixed
	 */
	public function getShippingAddressById($address_id)
	{
		$this->db()->query("SELECT * FROM ".DB_PREFIX."users_shipping WHERE uid='".intval($this->id)."' AND usid='".intval($address_id)."'");
		return $this->db()->moveNext() ? $this->db()->col : false;
	}
	
	/**
	 * Validate address 
	 * @param $data
	 * @return boolean
	 */
	public function validateAddress(&$form)
	{
		$errCount = count($this->errors);
		
		if (isset($form["address_type"]))
		{
			if (trim($form["address_type"]) == '') $this->setError("Please select address type");
		}
		
		if (isset($form["name"]))
		{
			if (!validate($form["name"], VALIDATE_NAME)) $this->setError("Enter valid name");
		}
		else
		{
			if (!isset($form["fname"]) || !validate($form["fname"], VALIDATE_NAME)) $this->setError("Enter valid first name");
			if (!isset($form["lname"]) || !validate($form["lname"], VALIDATE_NAME)) $this->setError("Enter valid last name");
		}
		
		if (!isset($form["address1"]) || !validate($form["address1"], VALIDATE_STRING)) $this->setError("Enter valid address line 1");
		if (!isset($form["address2"])) $form["address2"] = "";
		if ($this->_settings["FormsShippingCompany"] == 'Required' && (!isset($form["company"]) || !validate($form["company"], VALIDATE_STRING))) $this->setError("Enter a valid company name");
		if ($this->_settings["FormsShippingAddressLine2"] == 'Required' && (!isset($form["address2"]) || !validate($form["address2"], VALIDATE_STRING))) $this->setError("Enter a valid address line 2");
		if (!isset($form["city"]) || !validate($form["city"], VALIDATE_STRING)) $this->setError("Enter city name");
			
		//validate country, state and province & zip
		if (!isset($form["country"]) || !validate($form["country"], VALIDATE_DECIMAL))
		{
			$this->setError("Select country");
		}
		else
		{
			$this->db()->query("SELECT * FROM ".DB_PREFIX."countries WHERE coid='".intval($form["country"])."'");
			if (($country = $this->db()->moveNext()) != false)
			{
				$this->db()->query("SELECT COUNT(*) AS c FROM ".DB_PREFIX."states WHERE coid='".intval($form["country"])."'");
				$c = $this->db()->moveNext();
				if ($c["c"] > 0)
				{
					if (!isset($form["state"]) || !validate($form["state"], VALIDATE_DECIMAL))
					{
						$this->setError("Select state/province");
					}
					else
					{
						$this->db()->query("SELECT * FROM ".DB_PREFIX."states WHERE coid='".intval($form["country"])."' AND stid='".intval($form["state"])."'");
						if (($c = $this->db()->moveNext()) != false)
						{
							$form["province"] = $c["name"]; 
						}
						else
						{
							$this->setError("Select state/province");
						}
					}
				}
				else
				{
					if (!isset($form["province"]) || !validate($form["province"], VALIDATE_STRING)) $this->setError("Enter state/province name"); else $form["state"] = 0;
				}
				
				if (isset($form["zip"]))
				{
					if ($country["iso_a3"] == "USA" && !validate($form["zip"], VALIDATE_ZIP_USA)) $this->setError("Enter valid zip code");
					elseif ($country["iso_a3"] == "CAN" && !validate($form["zip"], VALIDATE_ZIP_CANADA)) $this->setError("Enter valid postal code");
					elseif ($country["iso_a3"] == "GBP" && !validate($form["zip"], VALIDATE_ZIP_UK)) $this->setError("Enter valid postal code");
					elseif (!validate($form["zip"], VALIDATE_ZIP)) $this->setError("Enter valid zip/postal code");					
				}
				else
				{
					$this->setError("Enter valid zip/postal code");	
				}
			}
			else
			{
				$this->setError("Select country");
			}
		}
		return $errCount == count($this->errors);
	}
	
	/**
	 * Sign-up procedure
	 * @param $form
	 * @param $costomeFieldsData
	 * @param $guestUserId - used when need to convert guest user to normal one
	 * @return boolean
	 */
	public function signUp(&$form, &$customFieldsData = false, $guestUserId = false)
	{
		$this->is_error = false;
		$this->errors = array();
		
		//check form variable
		if (is_array($form))
		{
			//trim form
			foreach ($form as $key=>$value) $form[$key] = trim($value);
			
			//if (!isset($form["fname"]) || !validate($form["fname"], VALIDATE_NAME)) $this->setError("Enter valid first name");
			//if (!isset($form["lname"]) || !validate($form["lname"], VALIDATE_NAME)) $this->setError("Enter valid last name");
			
			$this->validateAddress($form);
			
			//validate custom fields for billing
			if (is_array($customFieldsData))
			{
				$this->customFields()->resetErrors();
				$this->customFields()->validateCustomFields("billing", $customFieldsData);
				if ($this->customFields()->is_error) $this->setError($this->customFields()->errors);
			}
			
			// Standard settings in the admin area
			if ($this->_settings["FormsBillingCompany"] == 'Required' && (!isset($form["company"]) || !validate($form["company"], VALIDATE_NOT_EMPTY))) $this->setError("Enter a valid company name");
			if ($this->_settings["FormsBillingAddressLine2"] == 'Required' && (!isset($form["address2"]) || !validate($form["address2"], VALIDATE_NOT_EMPTY))) $this->setError("Enter a valid address line 2");
			if ($this->_settings["FormsBillingPhone"] == 'Required' && (!isset($form["phone"]) || !validate($form["phone"], VALIDATE_PHONE))) $this->setError("Enter valid phone number");
			
			if (!isset($form["login"]) || !validate($form["login"], VALIDATE_USERNAME))
			{
				$this->setError("Please enter username");	
			}
			else
			{
				$this->db()->query("
					SELECT * 
					FROM ".DB_PREFIX."users 
					WHERE 
						login LIKE '".$this->db()->escape(trim($form["login"]))."' 
						AND removed='No'
				");
				
				if ($this->db()->moveNext()) $this->setError("Login already exists");
			}
			
			if (!isset($form["password"]) || !validate($form["password"], VALIDATE_PASSWORD))
			{
				$this->setError("Please check your password");	
			}
			else
			{
				if (!isset($form["password2"]) || (isset($form["password2"]) && $form["password"] != $form["password2"]))
				{
					$this->setError("Please check password confirmation");	
				}
			}
			
			if (!isset($form["email"]) || !validate($form["email"], VALIDATE_EMAIL))
			{
				$this->setError("Enter valid email address");	
			}
			else
			{
				$this->db()->query("
					SELECT uid
					FROM ".DB_PREFIX."users 
					WHERE 
						LOWER(email) = '".$this->db()->escape(strtolower(trim($form["email"])))."'
						AND login <> '".EXPRESS_CHECKOUT_USER."' AND removed='No'
				");
				
				if ($this->db()->moveNext()) $this->setError("Email address already exists");
			}
			
			// validate custom fields for account and signup
			if (is_array($customFieldsData))
			{
				$this->customFields()->resetErrors();
				$this->customFields()->validateCustomFields("account", $customFieldsData);
				$this->customFields()->validateCustomFields("signup", $customFieldsData);
				if ($this->customFields()->is_error) $this->setError($this->customFields()->errors);
			}
			
			$form["email_mode"] = isset($form["email_mode"]) ? (in_array($form["email_mode"], array("text", "html")) ? $form["email_mode"]:"text") : "text";
			$form["email_newsletters"] = isset($form["email_newsletters"]) ? "Yes" : "No";
			$form["email_updates"] = isset($form["email_updates"]) ? "Yes" : "No";
			
			$shipping_thesame = isset($form["thesame"]);
			$_SESSION["shipping_thesame"] = $shipping_thesame;
			
			if (!$this->is_error)
			{
				$this->db()->reset();
				//account settings
				$this->db()->assign("created_date", "NOW()");
				$this->db()->assign("last_update", "NOW()");
				$this->db()->assignStr("newsletters", $form["email_newsletters"]);
				$this->db()->assignStr("updates", $form["email_updates"]);
				$this->db()->assignStr("email", $form["email"]);
				$this->db()->assignStr("email_mode", $form["email_mode"]);
				$this->db()->assignStr("login", $form["login"]);
				$this->db()->assignStr("password", getPasswordHash($form["password"]));
				//billing info
				$this->db()->assignStr("fname", $form["fname"]);
				$this->db()->assignStr("lname", $form["lname"]);
				$this->db()->assignStr("company", $form["company"]);
				//address
				$this->db()->assignStr("address1", $form["address1"]);
				$this->db()->assignStr("address2", $form["address2"]);
				$this->db()->assignStr("city", $form["city"]);
				$this->db()->assignStr("state", intval($form["state"]));
				$this->db()->assignStr("province", $form["province"]);
				$this->db()->assignStr("zip", $form["zip"]);
				$this->db()->assignStr("country", intval($form["country"]));
				//phone
				$this->db()->assignStr("phone", $form["phone"]);
				
				//check is it really new user
				if (!$guestUserId)
				{
					//create account
					$this->id = $this->db()->insert(DB_PREFIX."users");
				}
				else
				{
					//conver existing user
					$this->id = intval($guestUserId);
					$this->db()->update(DB_PREFIX."users", "WHERE uid='".$this->id."'");
				}
				
				if (is_array($customFieldsData))
				{
					$this->customFields()->storeCustomFields("billing", $customFieldsData, $this->id, 0, 0);
					$this->customFields()->storeCustomFields("account", $customFieldsData, $this->id, 0, 0);
					$this->customFields()->storeCustomFields("signup", $customFieldsData, $this->id, 0, 0);
				}
				
				//store shipping address if checked "the same"
				if ($shipping_thesame)
				{
					$shippingAddress = $form;
					$shippingAddress["is_primary"] = "Yes";
					$shippingAddress["name"] = $form["fname"]." ".$form["lname"];
					$this->addShippingAddress($shippingAddress);
				}
				
				if ($form['email_newsletters'] == 'Yes')
				{
					Newsletter_Connector::subscribe($form['email'], $form['email_mode']);
				}
				
				$this->getUserData();
			}
		}
		else
		{
			$this->is_error = true;
			$this->setError("Incorrect form params");
		}
		return !$this->is_error;
	}
	
	/**
	 * Update profile procedure
	 * @param $form
	 * @return boolean
	 */
	public function updateProfile(&$form, &$customFieldsData = false)
	{
		$this->is_error = false;
		$this->errors = array();
		//check form variable
		if (is_array($form))
		{
			//trim form
			foreach ($form as $key=>$value) $form[$key] = trim($value);
			
			//if (!isset($form["fname"]) || !validate($form["fname"], VALIDATE_NAME)) $this->setError("Enter valid first name");
			//if (!isset($form["lname"]) || !validate($form["lname"], VALIDATE_NAME)) $this->setError("Enter valid last name");
			
			if ($this->_settings["FormsBillingCompany"] == 'Required' && (!isset($form["company"]) || !validate($form["company"], VALIDATE_NOT_EMPTY))) $this->setError("Enter a valid company name");
			if ($this->_settings["FormsBillingAddressLine2"] == 'Required' && (!isset($form["address2"]) || !validate($form["address2"], VALIDATE_NOT_EMPTY))) $this->setError("Enter a valid address line 2");
			
			$this->validateAddress($form);
			
			if ($this->_settings["FormsBillingPhone"] == 'Required' && (!isset($form["phone"]) || !validate($form["phone"], VALIDATE_PHONE))) $this->setError("Enter valid phone number");
			
			//validate custom fields for billing
			if (is_array($customFieldsData))
			{
				$this->customFields()->resetErrors();
				$this->customFields()->validateCustomFields("billing", $customFieldsData);
				if ($this->customFields()->is_error) $this->setError($this->customFields()->errors);
			}
			
			if (isset($form["password"]) && trim($form["password"]) != "")
			{
				if (!validate($form["password"], VALIDATE_PASSWORD))
				{
					$this->setError("Please check your password");	
				}
				else
				{
					if (!isset($form["password2"]) || (isset($form["password2"]) && $form["password"] != $form["password2"]))
					{
						$this->setError("Please check password confirmation");	
					}
				}
			}
			
			if (!isset($form["email"]) || !validate($form["email"], VALIDATE_EMAIL))
			{
				$this->setError("Enter valid email address");	
			}
			else
			{
				$this->db()->query("
					SELECT uid
					FROM ".DB_PREFIX."users 
					WHERE 
						LOWER(email) = '".$this->db()->escape(strtolower(trim($form["email"])))."'
						AND login <> '".EXPRESS_CHECKOUT_USER."'
						AND uid <> '".intval($this->id)."'
				");
				if ($this->db()->moveNext()) $this->setError("Email address already exists");
			}
		
			//validate custom fields for account
			if (is_array($customFieldsData))
			{
				$this->customFields()->resetErrors();
				$this->customFields()->validateCustomFields("account", $customFieldsData);
				if ($this->customFields()->is_error) $this->setError($this->customFields()->errors);
			}
			
			$form["email_mode"] = isset($form["email_mode"]) ? (in_array($form["email_mode"], array("text", "html"))?$form["email_mode"]:"text") : "text";
			$form["newsletters"] = isset($form["newsletters"]) ? "Yes" : "No";
			$form["updates"] = isset($form["updates"]) ? "Yes" : "No";
			
			if (!$this->is_error)
			{
				$previousNewsletterSetting = 'No';
				$previousEmail = '';
				$this->db()->query("SELECT email, newsletters FROM ".DB_PREFIX."users WHERE uid = '".intval($this->id)."' AND removed='No'");
				$tempRow = $this->db()->moveNext();
				if ($tempRow)
				{
					$previousNewsletterSetting = $tempRow['newsletters'];
					$previousEmail = $tempRow['email'];
				}
				
				$this->db()->reset();
				//account settings
				$this->db()->assign("last_update", "NOW()");
				$this->db()->assignStr("newsletters", $form["newsletters"]);
				$this->db()->assignStr("updates", $form["updates"]);
				$this->db()->assignStr("email", $form["email"]);
				$this->db()->assignStr("email_mode", $form["email_mode"]);
				
				if (isset($form["password"]) && $form["password"] != "")
				{
					$this->db()->assignStr("password", getPasswordHash($form["password"]));
				}
				
				//billing info
				$this->db()->assignStr("fname", $form["fname"]);
				$this->db()->assignStr("lname", $form["lname"]);
				$this->db()->assignStr("company", $form["company"]);
				//address
				$this->db()->assignStr("address1", $form["address1"]);
				$this->db()->assignStr("address2", $form["address2"]);
				$this->db()->assignStr("city", $form["city"]);
				$this->db()->assignStr("state", intval($form["state"]));
				$this->db()->assignStr("province", $form["province"]);
				$this->db()->assignStr("zip", $form["zip"]);
				$this->db()->assignStr("country", intval($form["country"]));
				//phone
				$this->db()->assignStr("phone", $form["phone"]);
				
				$this->db()->update(DB_PREFIX."users", "WHERE ".DB_PREFIX."users.uid='".intval($this->id)."'");
				
				if (is_array($customFieldsData))
				{
					$this->customFields()->storeCustomFields("billing", $customFieldsData, intval($this->id), 0, 0);
					$this->customFields()->storeCustomFields("account", $customFieldsData, intval($this->id), 0, 0);
				}
				
				if ($previousNewsletterSetting != $form['newsletters'])
				{
					if ($form['newsletters'] == 'Yes')
					{
						Newsletter_Connector::subscribe($form['email'], $form['email_mode']);
					}
					else
					{
						Newsletter_Connector::unsubscribe($form['email']);
					}
				}
			}
		}
		else
		{
			$this->setError("Incorrect form params");
		}
		return !$this->is_error;
	}
	
	/**
	 *
	 * @param type $paymentProfile 
	 */
	public function updateBillingInfoFromPaymentProfile($paymentProfile)
	{
		$this->db()->reset();
		$d = $paymentProfile["data"];
		$this->db()->assignStr("fname", isset($d["first_name"]) ? $d["first_name"] : "");
		$this->db()->assignStr("lname", isset($d["last_name"]) ? $d["last_name"] : "");
		$this->db()->assignStr("company", isset($d["company"]) ? $d["company"] : "");
		$this->db()->assignStr("address1", isset($d["address1"]) ? $d["address1"] : "");
		$this->db()->assignStr("address2", isset($d["address2"]) ? $d["address2"] : "");
		$this->db()->assignStr("city", isset($d["city"]) ? $d["city"] : "");
		$this->db()->assignStr("state", isset($d["state_id"]) && $d["state_id"] > 0 ? $d["state_id"] : "0");
		$this->db()->assignStr("province", isset($d["state"]) ? $d["state"] : "");
		$this->db()->assignStr("zip", isset($d["zip"]) ? $d["zip"] : "");
		$this->db()->assignStr("country", isset($d["country_id"]) ? $d["country_id"] : "0");
		if (isset($d["phone"])) $this->db()->assign("phone", $d["phone"]); // do not ampty phone
		
		$this->db()->update(DB_PREFIX."users", "WHERE ".DB_PREFIX."users.uid='".intval($this->id)."'");
	}
	
	
	/**
	 * Update billing info
	 * @param $form
	 * @return boolean
	 */
	public function updateBillingInfo(&$form, &$customFieldsData = false)
	{
		$this->is_error = false;
		$this->errors = array();
		//check form variable
		if (is_array($form))
		{
			//trim form
			foreach ($form as $key=>$value) $form[$key] = trim($value);
			
			//if (!isset($form["fname"]) || !validate($form["fname"], VALIDATE_NAME)) $this->setError("Enter valid first name");
			//if (!isset($form["lname"]) || !validate($form["lname"], VALIDATE_NAME)) $this->setError("Enter valid last name");
			
			$this->validateAddress($form);
			
			if ($this->_settings["FormsBillingCompany"] == 'Required' && (!isset($form["company"]) || !validate($form["company"], VALIDATE_STRING))) $this->setError("Enter a valid company name");
			if ($this->_settings["FormsBillingAddressLine2"] == 'Required' && (!isset($form["address2"]) || !validate($form["address2"], VALIDATE_STRING))) $this->setError("Enter a valid address line 2");
			if ($this->_settings["FormsBillingPhone"] == 'Required' && (!isset($form["phone"]) || !validate($form["phone"], VALIDATE_PHONE))) $this->setError("Enter valid phone number");
			
			if (!isset($form["email"]) || !validate($form["email"], VALIDATE_EMAIL))
			{
				$this->setError("Enter valid email address");
			}
			else
			{
				//validate email only if it is not express checkout user
				if (!$this->express)
				{
					$this->db()->query("
						SELECT uid
						FROM ".DB_PREFIX."users 
						WHERE 
							LOWER(email) = '".$this->db()->escape(strtolower(trim($form["email"])))."'
							AND login <> '".EXPRESS_CHECKOUT_USER."'
							AND uid <> '".intval($this->id)."'
							AND removed='No'
					");
					if ($this->db()->moveNext()) $this->setError("Email address already exists");
				}
			}
			
			if (is_array($customFieldsData))
			{
				$this->customFields()->resetErrors();
				$this->customFields()->validateCustomFields("billing", $customFieldsData);
				if ($this->customFields()->is_error) $this->setError($this->customFields()->errors);
			}
			
			$this->is_error = count($this->errors) > 0;
			
			if (!$this->is_error)
			{
				$this->db()->reset();
				//profile settings
				$this->db()->assignStr("email", $form["email"]);
				
				if (isset($form["password"]) && $form["password"] != "")
				{
					$this->db()->assignStr("password", getPasswordHash($form["password"]));
				}
				
				//billing info
				$this->db()->assign("last_update", "NOW()");
				$this->db()->assignStr("fname", $form["fname"]);
				$this->db()->assignStr("lname", $form["lname"]);
				$this->db()->assignStr("company", $form["company"]);
				//address
				$this->db()->assignStr("address1", $form["address1"]);
				$this->db()->assignStr("address2", $form["address2"]);
				$this->db()->assignStr("city", $form["city"]);
				$this->db()->assignStr("state", intval($form["state"]));
				$this->db()->assignStr("province", $form["province"]);
				$this->db()->assignStr("zip", $form["zip"]);
				$this->db()->assignStr("country", intval($form["country"]));
				//phone
				$this->db()->assignStr("phone", $form["phone"]);
				$this->db()->update(DB_PREFIX."users", "WHERE ".DB_PREFIX."users.uid='".intval($this->id)."'");
				
				if (is_array($customFieldsData))
				{
					$this->customFields()->storeCustomFields("billing", $customFieldsData, $this->id, 0, 0);
				}
			}
		}
		else
		{
			$this->is_error = true;
			$this->setError("Incorrect form params");
		}
		return !$this->is_error;
	}
	
	/**
	 * Login procedure
	 * @param $login
	 * @param $password
	 * @param $remember_me
	 * @return boolean
	 */
	public function login($login, $password, $remember_me = false)
	{
		$this->db()->query("
			SELECT uid, IF(block_date > NOW(), 1, 0) AS blocked 
			FROM ".DB_PREFIX."users 
			WHERE LCASE(login) = '".$this->db()->escape(strtolower($login))."' AND removed='No' AND (password = '".getPasswordHash($password)."' OR password = MD5('".$this->db()->escape($password)."') OR password = PASSWORD('".$this->db()->escape($password)."'))
		");
		
		if ($this->db()->moveNext())
		{
			$this->id = $this->db()->col["uid"];
			
			// check is account blocked
			if ($this->_settings["SecurityAccountBlocking"] == "YES" && $this->db()->col["blocked"] == "1")
			{
				$this->blocked = true;
				$this->auth_ok = false;
				return false;
			}
			
			$this->auth_ok = true;
			
			$this->getUserData();
			
			$this->session_id = uniqid(md5(rand(1, 1000000)));
			
			$this->db()->query("UPDATE ".DB_PREFIX."users SET session_id='".$this->db()->escape($this->session_id)."', session_date=NOW() WHERE uid='".intval($this->id)."'");
			
			$cookies = initCookies();
			
			if ($remember_me)
			{
				$cookies->set($this->cookie."LoginUserName", $login);
			}
			else
			{
				$cookies->remove($this->cookie."LoginUserName");
			}

			$_SESSION["SessionID"] = $this->session_id;
			$_SESSION["UserID"] = $this->id;
			
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Facebook synk
	 */
	public function facebookSync($fbUserId, $fbUserProfile)
	{
		$this->blocked = false;
		$this->auth_ok = true;
		$this->session_id = uniqid(md5(rand(1, 1000000)));
		
		$result = $this->db()->query("SELECT *, IF(block_date > NOW(), 1, 0) AS blocked FROM ".DB_PREFIX."users WHERE facebook_user=1 AND facebook_id='".$this->db()->escape($fbUserId)."' AND removed='No'");
		
		/**
		 * User exists
		 */
		if (($data = $this->db()->moveNext($result, MYSQL_ASSOC)) != false)
		{
			$this->id = $data["uid"];
			
			// check is account blocked
			if ($this->_settings["SecurityAccountBlocking"] == "YES" && $data["blocked"] == "1")
			{
				$this->blocked = true;
				$this->auth_ok = false;
				return false;
			}
			
			// update userdata
			$this->db()->query("UPDATE ".DB_PREFIX."users SET session_id='".$this->db()->escape($this->session_id)."', session_date=NOW() WHERE uid='".intval($this->id)."'");
			unset($_SESSION["facebook_new_account"]);
		}
		/**
		 * Need to create a new user account
		 */
		else
		{
			$this->auth_ok = true;
			$this->session_id = uniqid(md5(rand(1, 1000000)));
			
			$this->db()->reset();
			$this->db()->assignStr("facebook_user", 1);
			$this->db()->assignStr("facebook_id", $fbUserId);
			$this->db()->assignStr("login", "fb-".$fbUserId);
			$this->db()->assignStr("password", md5(uniqid(rand())));
			$this->db()->assignStr("fname", $fbUserProfile["first_name"]);
			$this->db()->assignStr("lname", $fbUserProfile["last_name"]);
			$this->db()->assignStr('email', $fbUserProfile['email']);
			$this->db()->assignStr("country", 0);
			$this->db()->assignStr("state", 0);
			$this->db()->assignStr("newsletters", "No");
			$this->db()->assignStr("updates", "No");
			$this->db()->assign("last_update", "NOW()");
			$this->db()->assign("created_date", "NOW()");
			$this->db()->assignStr("session_id", $this->session_id);
			$this->db()->assign("session_date", "NOW()");
			$this->db()->insert(DB_PREFIX."users");
			$this->id = $this->db()->insertId();
			
			$_SESSION["facebook_new_account"] = true;
		}
		
		$this->getUserData();
		
		$_SESSION["SessionID"] = $this->session_id;
		$_SESSION["UserID"] = $this->id;
		
		return true;
	}
	
	/**
	 * Express Checkout Login procedure
	 * @return boolean
	 */
	public function expressLogin()
	{
		$this->db()->query("INSERT INTO ".DB_PREFIX."users (login, password, country, state, newsletters, updates, last_update, created_date) VALUES ('".EXPRESS_CHECKOUT_USER."', '".(md5(uniqid(rand())))."', '0', '0', 'No', 'No', NOW(), NOW())");
		$this->id = $this->db()->insertId();
		$this->auth_ok = true;
		$this->session_id = uniqid(md5(rand(1, 1000000)));
		$this->db()->query("UPDATE ".DB_PREFIX."users SET session_id='".$this->session_id."', session_date=NOW() WHERE uid='".intval($this->id)."'");
		$_SESSION["SessionID"] = $this->session_id;
		$_SESSION["UserID"] = $this->id;
		return true;
	}
	
	/**
	 * Password Reset
	 * @param $login
	 * @param $email
	 * @return boolean
	 */
	public function passwordReset($login, $email)
	{
		$this->db()->query("
			SELECT * 
			FROM ".DB_PREFIX."users 
			WHERE 
				login = '".$this->db()->escape($login)."' AND 
				LOWER(email) = '".strtolower($this->db()->escape($email))."' AND 
				facebook_user = 0 AND 
				login <> '".$this->db()->escape(EXPRESS_CHECKOUT_USER)."' AND 
				removed='No'
		");
		if (($user_data = $this->db()->moveNext()) != false)
		{
			$new_password = generatePassword();
			$this->db()->query("UPDATE ".DB_PREFIX."users SET password='".$this->db()->escape(getPasswordHash($new_password))."', last_update = NOW() WHERE uid='".intval($user_data["uid"])."'");
			emailUserResetPassword($user_data, $this->db()->col["uid"], $new_password);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Account Blocking on payment failures
	 * @param $hours
	 */
	public function block($hours)
	{
		$this->db()->query("UPDATE ".DB_PREFIX."users SET block_date=DATE_ADD(NOW(), INTERVAL ".intval($hours)." HOUR), last_update = NOW() WHERE uid='".intval($this->id)."'");
	}
	
	/**
	 * Get Orders History List
	 * @return mixed
	 */
	public function getOrders()
	{
		if ($this->auth_ok)
		{
			$this->db()->query("SELECT *, DATE_FORMAT(status_date, '".$this->_settings["LocalizationDateTimeFormat"]."') AS status_date FROM ".DB_PREFIX."orders WHERE uid='".intval($this->id)."' AND status <> 'Abandon' AND status <> 'New' ORDER BY placed_date DESC");
			if ($this->db()->numRows() > 0)
			{
				return $this->db()->getRecords();
			}
		}
		return false;
	}
	
	/**
	 * Get Order Data
	 * @param $order_id
	 * @return mixed
	 */
	public function getOrderData($order_id)
	{
		if ($this->auth_ok)
		{
			$r = $this->db()->query("
				SELECT ".DB_PREFIX."orders.*,
					DATE_FORMAT(".DB_PREFIX."orders.status_date, '".$this->_settings["LocalizationDateTimeFormat"]."') AS status_date,
					s_countries.name AS shipping_country_name,
					s_states.name AS shipping_state_name
				FROM ".DB_PREFIX."orders 
				LEFT JOIN ".DB_PREFIX."countries AS s_countries ON s_countries.coid = ".DB_PREFIX."orders.shipping_country
				LEFT JOIN ".DB_PREFIX."states AS s_states ON s_states.stid = ".DB_PREFIX."orders.shipping_state
				WHERE oid='".intval($order_id)."' 
					AND uid='".intval($this->id)."' 
					AND status<>'Abandon' 
					AND status<>'New'
			");
			return $this->db()->moveNext($r, MYSQL_ASSOC) ? $this->db()->col : false;
		}
		return false;
	}

	/**
	 * Get Order Items
	 * @param $order_id
	 * @return mixed
	 */
	public function getOrderItems($order_id)
	{
		if ($this->auth_ok)
		{
			$this->db()->query("SELECT * FROM ".DB_PREFIX."orders_content WHERE oid='".intval($order_id)."' ORDER BY product_id");
			if ($this->db()->numRows()>0)
			{
				$products = $this->db()->getRecords();
				
				for ($i=0; $i<count($products); $i++)
				{
					if ($products[$i]["product_sub_id"] != "")
					{
						$products[$i]["product_id"] = $products[$i]["product_id"]." / ".$products[$i]["product_sub_id"];
					}
					if ($this->_settings["DisplayPricesWithTax"] == "YES" && $products[$i]["is_taxable"] == "Yes")
					{
						if ($products[$i]["tax_rate"] >= 0)
						{
							$products[$i]["admin_price"] = $products[$i]["admin_price"] + ($products[$i]["admin_price"] / 100 * $products[$i]["tax_rate"]);
						}
					}
				}
			}
			return $products;
		}
		return false;
	}
	
	/**
	 * Logout procedure
	 */
	public function logOut()
	{
		if ($this->auth_ok)
		{
			// generate protect-on-logout session id
			$s = uniqid(md5(rand(1, 100000)));
			$this->db()->query("UPDATE ".DB_PREFIX."users SET session_id='".$s."' WHERE uid='".intval($this->id)."'");
		}
		
		unset($_SESSION["SessionID"]);
		unset($_SESSION["UserID"]);
		unset($_SESSION["order_promo_code"]);
		unset($_SESSION["order_gift_certificate"]);
		$this->id = 0;
		$this->level = 0;
		$this->auth_ok = false;
		$this->session_id = "";
	}
	
	/**
	 * Return user data
	 * @return mixed
	 */
	public function getUserData()
	{
		if ($this->id)
		{
			$result = $this->db()->query("
				SELECT 
					".DB_PREFIX."users.*,
					CONCAT(".DB_PREFIX."users.fname, ' ', ".DB_PREFIX."users.lname) AS name,
					".DB_PREFIX."countries.name AS country_name,
					".DB_PREFIX."countries.iso_a2 AS country_iso_a2,
					".DB_PREFIX."countries.iso_a3 AS country_iso_a3,
					".DB_PREFIX."countries.iso_number AS country_iso_number,
					".DB_PREFIX."states.name AS state_name,
					".DB_PREFIX."states.short_name AS state_abbr
				FROM ".DB_PREFIX."users 
				LEFT JOIN ".DB_PREFIX."countries ON ".DB_PREFIX."countries.coid = ".DB_PREFIX."users.country
				LEFT JOIN ".DB_PREFIX."states ON ".DB_PREFIX."states.stid = ".DB_PREFIX."users.state
				WHERE ".DB_PREFIX."users.uid='".intval($this->id)."' AND removed='No'
			");
			
			if ($this->db()->moveNext($result, MYSQL_ASSOC))
			{
				$this->data = $this->db()->col;
				$this->level = $this->db()->col["level"];
				$this->data["usid"] = "billing";
				$this->express = $this->data["login"] == EXPRESS_CHECKOUT_USER;
				return $this->data;
			}
		}
		return false;
	}
	
	/**
	 * Returns current user billing address, phone and email
	 */
	public function getBillingInfo()
	{
		if ($this->id)
		{
			$this->db()->query("
				SELECT 
					".DB_PREFIX."users.fname,
					".DB_PREFIX."users.lname,
					CONCAT(".DB_PREFIX."users.fname, ' ', ".DB_PREFIX."users.lname) AS name,
					".DB_PREFIX."users.company,
					".DB_PREFIX."users.address1,
					".DB_PREFIX."users.address2,
					".DB_PREFIX."users.city,
					".DB_PREFIX."users.state,
					".DB_PREFIX."users.province,
					".DB_PREFIX."users.country,
					".DB_PREFIX."users.zip,
					".DB_PREFIX."users.phone,
					".DB_PREFIX."users.email,
					".DB_PREFIX."countries.name AS country_name,
					".DB_PREFIX."countries.iso_a2 AS country_iso_a2,
					".DB_PREFIX."countries.iso_a3 AS country_iso_a3,
					".DB_PREFIX."countries.iso_number AS country_iso_number,
					".DB_PREFIX."states.name AS state_name,
					".DB_PREFIX."states.short_name AS state_abbr
				FROM ".DB_PREFIX."users 
				LEFT JOIN ".DB_PREFIX."countries ON ".DB_PREFIX."countries.coid = ".DB_PREFIX."users.country
				LEFT JOIN ".DB_PREFIX."states ON ".DB_PREFIX."states.stid = ".DB_PREFIX."users.state
				WHERE ".DB_PREFIX."users.uid='".intval($this->id)."' AND removed='No'
			");
			if ($this->db()->moveNext(false, MYSQL_ASSOC))
			{
				return $this->db()->col;
			}
		}
		return false;
	}
	
	/**
	 * Set user cookies
	 */
	public function setCookie($uid = null, $level = null, $session_id = null)
	{
		setcookie(
			"_pcud", 
			base64_encode(
				serialize(
					array(
						"i"=>!is_null($uid) ? $uid : $this->id,
						"l"=>!is_null($level) ? $level : $this->level,
						"s"=>!is_null($session_id) ? $session_id : md5($this->session_id)
					)
				)
			), 
			time() + 31536000, 
			"/", 
			$_SERVER["SERVER_PORT"] == "443" ? $this->_settings["GlobalHttpsCookieDomain"] : $this->_settings["GlobalHttpCookieDomain"]
		);
	}
	
	/**
	 * unset user cookie
	 */
	public function unsetCookie()
	{
		setcookie(
			"_pcud", 
			"", 
			time() - 31536000, 
			"/", 
			$_SERVER["SERVER_PORT"] == "443" ? $this->_settings["GlobalHttpsCookieDomain"] : $this->_settings["GlobalHttpCookieDomain"]
		);
	}
}

